﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;



using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape;

using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.Layouters;





namespace GraphicalObjects
{
    public class DisplayInitializator
    {
        private Diagram diagram;
        private Project project;
        private string name;
        private List<Modules> actualList;
        private List<Modules> moduleList;
        private Modules parentModule;

        private bool visualizeNameOfPort = true;
        private bool visualizeNameOfSignal = true;
        private bool visualiteNameOfModule = true;
     
        private CachedRepository repository;


        public DisplayInitializator(ref Diagram diagram, ref Project project, ref CachedRepository repository, ref List<Modules> list, string name, Modules parentModule)
        {
            this.diagram = diagram;
            this.project = project;
            this.repository = repository;
            this.name = name;
            this.moduleList = list;
            this.parentModule = parentModule;
        }

        //----------------------------------------------Set a Get  funkcie------------------------------------------
        public void SetVisualizeNameOfPort(bool isVisual)
        {
            this.visualizeNameOfPort = isVisual;
        }
        public void SetVisualizeNameOfSignal(bool isVisual)
        {
            this.visualizeNameOfSignal = isVisual;
        }
        public void SetVisualizeNameOfModule(bool isVisual)
        {
            this.visualiteNameOfModule = isVisual;
        }


        public List<Modules> GetActualList()
        {
            return this.actualList;
        }
        public List<Modules> GetModulesList()
        {
            return this.moduleList;
        }
        public Modules GetParentModule()
        {
            return this.parentModule;
        }
        




        //--------------------------------------------Funkcie pre repozitar------------------------------------------

        public void UpdateDiagramInRepository()
        {

            if (this.repository.GetDiagrams().Count() == 0)
            {
                this.repository.InsertDiagram(this.diagram);
            }
            else
            {
                
               
                this.repository.UpdateDiagram(this.diagram);
            }
        }
        public bool RepositoryIsChanged()
        {
            return this.repository.IsModified;
        }

        //---------------------------------------------Funkcie pre diagram--------------------------------------------
        public void ClearDiagram()
        {
            this.diagram.Clear();
        }





        //------------------------------------------Funkcie pre display--------------------------------------------------
        public void InitializeDisplay()
        {       
            this.project.Repository = repository;
            this.project.Name = "Vizualizator";
            this.project.Create();

            ((RoleBasedSecurityManager)project.SecurityManager).SetPermissions('A', "Designer", Permission.All);
            ((RoleBasedSecurityManager)project.SecurityManager).AddDomain('G', "Guest");
            ((RoleBasedSecurityManager)project.SecurityManager).SetPermissions('G', "Guest", Permission.None);

            this.project.AddLibrary(typeof(Box).Assembly);          
        }






 


        //-----------------------------------------------------Vizualizacne funkcie -Signals     ----------------------

        public void VisualizeSignal(ref Signals signal)
        {
            signal.Visualize(ref diagram, ref project);
        }
        public void VisualizeSignals(ref List<Signals> list)
        {
            list.ForEach(delegate(Signals signal)
            {
                VisualizeSignal(ref signal);
            });
        }


        //------------------------------FUNKCIA NA KOMPLETNE PREKRESLENIE DIAGRAMu-------------------------


        public void VisualizeAllGraphicalObjects(ref VHDLDisplay display, ref PositionGenerator Generator, List<Modules> actualList, Modules parentModule)
        {

            this.actualList = actualList;
            this.parentModule = parentModule;
            if (actualList.Count != 0)
            {
                display.Clear();
                ClearDiagram();


                if (Generator.IsPositionSet(ref this.actualList, ref this.parentModule) == false)
                {
                    Generator.SetModulesPosition(ref this.actualList, ref this.parentModule);
                    Generator.SetPortsPosition(ref this.actualList, ref this.parentModule);
                    Generator.SetSignalsPositions(ref this.actualList, ref this.parentModule);
                }



                foreach (Modules module2 in this.actualList)
                {
                    module2.Visualize(ref diagram, ref project, this.visualiteNameOfModule);
                    foreach (Ports port in module2.GetPortList())
                    {
                        port.Visualize(ref diagram, ref project, visualizeNameOfPort);
                    }
                }
                if (this.parentModule != null)
                {
                    foreach (Ports port in this.parentModule.GetPortList())
                    {
                        port.VisualizeParentPorts(ref diagram, ref project, visualizeNameOfPort);
                    }
                }
                List<Signals> actualSignals = Generator.getActualSignalList(ref this.actualList);
                if (actualSignals != null)
                {
                    foreach (Signals signal in actualSignals)
                    {
                        signal.Visualize(ref diagram, ref project);
                    }
                }

                this.UpdateDiagramInRepository();

                // Toto bude sluzit na úpravu velikosti diagramu , aby sa to prisposobovalo 
                int maxX = 0 , maxY = 0;
                
                foreach(Modules module in actualList) 
                {
                    if (module.GetPosX2() > maxX)
                    {
                        maxX = module.GetPosX1();
                    }
                    if (module.GetPosY2() > maxY)
                    {
                        maxY = module.GetPosY2();
                    }
                }
                diagram.Height = maxY + 300;
                diagram.Width = maxX + 450;


                display.LoadDiagram(this.name);
            }
        }
        


        //-------------------------------Funkcie na zaplnenie stromovej struktury---------------------------------------

        public void SearchForModuleToTreeview(ref List<Modules> list,ref TreeNode node, string path, int count)
        {
            
            path = path + ".";
            foreach (Modules module in list)
            {
                if (((module.GetPath().StartsWith(path)) == true) && (module.GetPath().ToCharArray().Count(x => x == '.') == count))
                {
                    TreeNode childNode = new TreeNode(module.GetName());
                    node.Nodes.Add(childNode);
                    SearchForModuleToTreeview(ref list, ref childNode, module.GetPath(), count + 1);                  
                }
            }
        }

        public void AddingModuleToTreeView(ref TreeView tree, ref List<Modules> list)
        {
            foreach (Modules module in list)
            {
                if (module.GetPath().IndexOfAny(".".ToCharArray()) == -1)
                {
                    TreeNode node = new TreeNode(module.GetName());
                    tree.Nodes.Add(node);
                    SearchForModuleToTreeview(ref list, ref node, module.GetPath(), 1);
                }
            }
        }

    }
}
