﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

using Dataweb.NShape;
using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape.Layouters;



namespace GraphicalObjects
{
    public class Modules
    {
        private string name;
        private string nameLang;
        private string type;
        private string path;

        private int posX1;
        private int posY1;
        private int posX2;
        private int posY2;

        private List<Ports> list = new List<Ports>();

        private RectangleBase rectangularShape;
        private TextBase nameShape;
 
        public Modules()
        {
            name = "";
            type = "";
            path = "";

            posX1 = 0;
            posX2= 0;
            posY1 = 0;
            posY2 = 0;
        }

        public Modules(string name)
        {
            this.name = name ;
            type = "";
            path = "";

            posX1 = 0;
            posY1= 0;
            posX2 = 0;
            posY2 = 0;
        }

        //-------------------------------------------- Write to XHDL form ------------------------------------------
        public string GetXHDLString()
        {
            string str = "\t\t<Module>\n";
                str += "\t\t\t<Name>" + name + "</Name>\n";
                str += "\t\t\t<NameLang>" + nameLang + "</NameLang>\n";
                str += "\t\t\t<Type>" + type + "</Type>\n";
                str += "\t\t\t<Path>" + path + "</Path>\n";
                str += "\t\t\t<PosX1>" + posX1.ToString() + "</PosX1>\n";
                str += "\t\t\t<PosY1>" + posY1.ToString() + "</PosY1>\n";
                str += "\t\t\t<PosX2>" + posX2.ToString() + "</PosX2>\n";
                str += "\t\t\t<PosY2>" + posY2.ToString() + "</PosY2>\n";
            str += "\t\t</Module>\n";
            return str;
        }

        //------------------------------------------------Metody pe konzolu---------------------------------

        public void WriteToConsole()
        {
            Console.WriteLine("Name: {0}, NameLang: {1}", name, nameLang);
            Console.WriteLine("Path: {0}, Type: {1}", path, type);
            Console.WriteLine("X1: {0}, Y1: {1}, X2: {2}, Y2: {3}", posX1, posY1, posX2, posY2);
            Console.WriteLine("ModulePorts:");
            for (int i = 0; i < list.Count; i++)
                Console.WriteLine("\tPortName: {0}, ModulePath: {1}", list[i].GetName(), list[i].GetModulePath());
        }


        //---------------------------Metody SET pre triedu Moduls---------------------------------------------------------- 


        public void SetName(string NewName)
        {
            this.name = NewName;
        }
        public void SetNameLang(string NewNameLang)
        {
            this.nameLang = NewNameLang;
        }

        public void SetModuleType(string NewType)
        {
            this.type = NewType;
        }

        public void SetPath(string NewPath)
        {
            this.path = NewPath;
        }

        public void SetPosX1(int NewPosX1)
        {
            this.posX1 = NewPosX1;
        }
        public void SetPosX2(int NewPosX2)
        {
            this.posX2 = NewPosX2;
        }

        public void SetPosY1(int NewPosY1)
        {
            this.posY1 = NewPosY1;
        }
        public void SetPosY2(int NewPosY2)
        {
            this.posY2 = NewPosY2;
        }
        public void SetPosXY(int NewPosX1, int NewPosX2, int NewPosY1, int NewPosY2)
        {
            this.posX1 = NewPosX1;
            this.posX2 = NewPosX2;
            this.posY1 = NewPosY1;
            this.posY2 = NewPosY2;
        }
        public void AddPort(Ports NewPort)
        {
            list.Add(NewPort);
        }  
        //------------------------------Metody GET pre triedu Moduls-----------------------------------------------------

        public string GetName()
        {
            return this.name;
        }
        public string GetPath()
        {
            return this.path;
        }
        public string GetNameLang()
        {
            return this.nameLang;
        }

        public string GetModuleType()
        {
            return this.type;
        }
        public int GetPosX1()
        {
            return this.posX1;
        }

        public int GetPosX2()
        {
            return this.posX2;
        }

        public int GetPosY1()
        {
            return this.posY1;
        }

        public int GetPosY2()
        {
            return this.posY2;
        }
        public List<Ports> GetPortList()
        {
            return this.list;
        }
        public int GetPortCount()
        {
            return list.Count;
        }
        public int GetPortCountPerDirection(HDLVisualizerDefinitions.Direction direction)
        {
            int left = 0, down = 0, up = 0, right = 0;
            foreach (Ports port in list)
            {
                switch (port.GetRotate())
                {
                    case HDLVisualizerDefinitions.Direction.Down: 
                        down++;
                        break;
                    case HDLVisualizerDefinitions.Direction.Left:
                        left++;
                        break;
                    case HDLVisualizerDefinitions.Direction.Right:
                        right++;
                        break;
                    case HDLVisualizerDefinitions.Direction.Up:
                        up++;
                        break;
                    default: break;
                }
            }
            switch (direction)
            {
                case HDLVisualizerDefinitions.Direction.Down:
                    return down;
                case HDLVisualizerDefinitions.Direction.Left:
                    return left;
                case HDLVisualizerDefinitions.Direction.Right:
                    return right;
                case HDLVisualizerDefinitions.Direction.Up:
                    return up;
                default: break;
            }
            return 0;
        }



        public int GetPortCountPerDirectionMax()
        {
            int left = 0, down = 0, up = 0, right = 0;
            foreach (Ports port in list)
            {
                switch (port.GetRotate())
                {
                    case HDLVisualizerDefinitions.Direction.Down:
                        down++;
                        break;
                    case HDLVisualizerDefinitions.Direction.Left:
                        left++;
                        break;
                    case HDLVisualizerDefinitions.Direction.Right:
                        right++;
                        break;
                    case HDLVisualizerDefinitions.Direction.Up:
                        up++;
                        break;
                    default: break;
                }
            }
            return Math.Max(left, right);
        }
   
        public Ports GetPort(int index)
        {
            if (list[index] == null)
            {
                return null;
            }
            else
            {
               return list[index];
            }
        }

        public void Visualize(ref Diagram diagram, ref Project project, bool isVisible)
        {
            rectangularShape = (RectangleBase)project.ShapeTypes["Box"].CreateInstance();
            nameShape = (TextBase)project.ShapeTypes["Text"].CreateInstance();

            nameShape.Text = this.name;
            nameShape.Height = 0;
            nameShape.Width = 0;
            nameShape.X = this.posX1 + 100;
            nameShape.Y = this.posY1 + 20;
            nameShape.SecurityDomainName = 'G';

            rectangularShape.Tag = (object)this;
            rectangularShape.X = this.posX1 + 100;
            rectangularShape.Y = this.posY1 + ((this.posY2 - this.posY1) / 2);
            rectangularShape.Width = 200;
            rectangularShape.Height = this.posY2 - this.posY1;
            

            diagram.Shapes.Add(rectangularShape);      

            if (isVisible == true)
            {
                diagram.Shapes.Add(nameShape);
            }
        }
    }
}
