﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

using Dataweb.NShape;
using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape.Layouters;



namespace GraphicalObjects
{
    public class Nodes
    {

        private int x;
        private int y;
        private int width;
        private Ports port = null;
        private List<Nodes> list = new List<Nodes>();
       
        private CircleBase circleShape;

        public Nodes(Ports port)
        {
            this.port = port;
        }
        public Nodes()
        { 
        }


        public Nodes(int PosX, int PosY)
        {
            this.x = PosX;
            this.y = PosY;
        }
        
        //----------------------------------------------------------Write to XHDL string format------------------------------

        private string Tabs(int tabs)
        {
            string str = "";
            for(int i = 0; i<tabs; i++) str += "\t";
            return str;
        }

        public string GetXHDLString(int tabs) // commonly tabs = 4
        {
            string str = Tabs(tabs) + "<Node>\n";
                tabs++;
                str += Tabs(tabs) + "<PosX>" + x.ToString() + "</PosX>\n";
                str += Tabs(tabs) + "<PosY>" + y.ToString() + "</PosY>\n";
                str += Tabs(tabs) + "<Width>" + width.ToString() + "</Width>\n";
                
                if( port != null ) {
                    str += Tabs(tabs) + "<Map>\n";
                    tabs++;
                        str += Tabs(tabs) + "<PortName>" + port.GetName() + "</PortName>\n";
                        str += Tabs(tabs) + "<ModulePath>" + port.GetModulePath() + "</ModulePath>\n";
                    tabs--;
                    str += Tabs(tabs) + "</Map>\n";
                }

                // CROSS
                if (list.Count > 1)
                {
                    str += Tabs(tabs) + "<Cross>\n";
                    tabs++;
                        for (int i = 0; i < list.Count; i++) str += GetXHDLString(tabs);
                    tabs--;
                    str += Tabs(tabs) + "</Cross>\n";
                }

                tabs--;
            str += Tabs(tabs) + "</Node>\n";

            // if no CROSS
            if (list.Count == 1)
            {
                str += GetXHDLString(tabs);
            }
            return str;
        }

        //----------------------------------------------------------Metody pre Konzolu--------------------------------------
        private static int nodePrintCount = 0;
        public void WriteToConsole()
        {
            if (port == null) Console.WriteLine("NodePortName = null NextNodesCount: {0}", list.Count);
            else
            {
                Console.WriteLine("NodePortName: {0} NextNodesCount: {1}", port.GetName(), list.Count);
                for (int i = 0; i < nodePrintCount; i++) Console.Write("\t");
                Console.WriteLine("PortModulePath: {0}", port.GetModulePath());
            }
            for (int i = 0; i < nodePrintCount; i++) Console.Write("\t");
            Console.WriteLine("PosX: {0}, PosY: {1}, Width: {2}", x, y, width);

            for (int i = 0; i < nodePrintCount; i++) Console.Write("\t");
            if (list.Count > 1)
            {
                nodePrintCount++;
                Console.WriteLine();
            }
            else if (list.Count == 1)
            {
                Console.WriteLine((char)25);
            }
            else if (list.Count == 0)
            {
                Console.WriteLine();
            }

            for (int i = 0; i < list.Count; i++)
            {
                if (list.Count > 1)
                {
                    for (int j = 0; j < nodePrintCount - 1; j++)
                    {
                        Console.Write("\t");
                    }
                    Console.Write("({0}.of{1} )", i + 1, list.Count);
                }
                else
                {
                    for (int j = 0; j < nodePrintCount; j++)
                    {
                        Console.Write("\t");
                    }
                }
                list[i].WriteToConsole();

            }
            if (list.Count > 1) nodePrintCount--;
        }
        //---------------------------------------------------------Set Metody------------------------------------------



        public void SetPosX(int NewPosX)
        {
            this.x = NewPosX;
        }

        public void SetPort(Ports NewPort)
        {
            this.port = NewPort;
        }


        public void SetPosY(int NewPosY)
        {
            this.y = NewPosY;
        }

        public void AddNextNode(Nodes NextNode)
        {
            list.Add(NextNode);
        }

        public void SetWidth(int NewWidth)
        {
            this.width = NewWidth;
        }
        public int GetPosX()
        {
            return this.x;
        }

        public int GetPosY()
        {
            return this.y;
        }

        public Nodes GetNextNode(int Count)
        {
            if (list[Count] == null)               
            {
                return null;                
            }   
            else
            {
                return list[Count];         
            }
        }

        public Ports GetPort()
        {
            return this.port;
        }

        public CircleBase GetcircleBase()
        {
            return this.circleShape;
        }

        public bool IsEndPort()
        {
            if (this.port == null)
            {
                return false;
            }
            else
            {
                return true;
            }
        }


        public bool IsCross()
        {
            if (list.Count <= 1)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        public int GetNodesCount()
        {
            return list.Count;
        }
        public int GetWidth()
        {
            return this.width;
        }



        //------------------------------------------------------------Vizualizacne metody------------------------

        public void Visualize(ref Diagram diagram, ref Project project)
        {
            double tmp;

            circleShape = (CircleBase)project.ShapeTypes["Circle"].CreateInstance();
            circleShape.X = this.x;
            circleShape.Y = this.y;
            circleShape.FillStyle = project.Design.FillStyles.Black;
            circleShape.SecurityDomainName = 'A';

            if (list.Count > 1)
            {
                circleShape.Diameter = 10;
                diagram.Shapes.Add(circleShape);
            }
            else
            {
                circleShape.Diameter = 0;
                diagram.Shapes.Add(circleShape);
            }

            
            foreach (Nodes node in list)
            {    
                Polyline arrow = (Polyline)project.ShapeTypes["Polyline"].CreateInstance();
                
                if (this.port != null && double.TryParse((this.port.GetValue()), out tmp) == true)
                {
                    if (tmp == 1)
                    {
                        arrow.LineStyle = project.Design.LineStyles.Red;
                    }
                }
                
                arrow.SecurityDomainName = 'G';
                diagram.Shapes.Add(arrow);           
                node.Visualize(ref diagram, ref project);
                arrow.Connect(ControlPointId.FirstVertex, (Shape)circleShape, ControlPointId.Reference);
                arrow.Connect(ControlPointId.LastVertex, (Shape)node.GetcircleBase(), ControlPointId.Reference);
                
            }
            
        }
    }
}
