﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

using Dataweb.NShape;
using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape.Layouters;

using HDLVisualizerDefinitions;

namespace GraphicalObjects
{
    public class Ports
    {
        

        private string name;
        private string orientation;
        private string dataType;
        private string modulePath;
        private List<Signals> signal = new List<Signals>();
        private Nodes node = null;
        private Modules module = null;

        private int PosX1;
        private int PosY1;
        private int PosX2;
        private int PosY2;
        private Direction rotate;
        private int width;
        private string value;

        private RectangleBase rectangularShape;
        private IsoscelesTriangleBase triangleShape;
        private TextBase nameShape;



        //-------------------------------------------- Write to XHDL form ------------------------------------------
        public string GetXHDLString()
        {
            string str = "\t\t<Port>\n";

                str += "\t\t\t<Name>" + name + "</Name>\n";
                str += "\t\t\t<Orientation>" + orientation + "</Orientation>\n";
                str += "\t\t\t<DataType>" + dataType + "</DataType>\n";
                str += "\t\t\t<ModulePath>" + modulePath + "</ModulePath>\n";
                str += "\t\t\t<PosX1>" + PosX1.ToString() + "</PosX1>\n";
                str += "\t\t\t<PosY1>" + PosY1.ToString() + "</PosY1>\n";
                str += "\t\t\t<PosX2>" + PosX2.ToString() + "</PosX2>\n";
                str += "\t\t\t<PosY2>" + PosY2.ToString() + "</PosY2>\n";
                str += "\t\t\t<Rotate>" + ((int)rotate).ToString() + "</Rotate>\n";

            str += "\t\t</Port>\n";
            return str;
        }


        //-------------------------------------------MEtody pre konzolu---------------------------------------------------
        public void WriteToConsole()
        {
            Console.WriteLine("Name: {0}, DataType: {1}, ModulePath: {2}", name, dataType, modulePath);
            Console.WriteLine("\tOrientation: {0}, Width: {1}, Rotate: {2}", orientation, width, rotate);
            Console.WriteLine("\tX1: {0}, Y1: {1}, X2: {2}, Y2: {3}", PosX1, PosY1, PosX2, PosY2);
        }










        //------------------------------------------Set metody pre triedu Ports----------------------------------------------

        public void SetModule(Modules NewModule)
        {
            this.module = NewModule;
        }
        public void SetValue(string value)
        {
            this.value = value;
        }
        public string GetValue()
        {
            return this.value;
        }

        
        
        
        public void SetName(string NewName)
        {

            this.name = NewName;
        }
        public void SetNode(Nodes NewNode)
        {
            this.node = NewNode;
        }
        

        public void SetOrientation(string NewOrientation)
        {
            this.orientation = NewOrientation;
        }
        public void SetModulePath(string NewModulePath)
        {
            this.modulePath = NewModulePath;
        }

        public void SetDataType(string NewDataType)
        {
            this.dataType = NewDataType;
        }

        public void SetPosX1(int NewPosX1)
        {
            this.PosX1 = NewPosX1;
        }

        public void SetPosX2(int NewPosX2)
        {
            this.PosX2 = NewPosX2;
        }

        
        public void SetPosY1(int NewPosY1)
        {
            this.PosY1 = NewPosY1;
        }
        public void SetPosY2(int NewPosY2)
        {
            this.PosY2 = NewPosY2;
        }


        public void SetPosXY(int NewPosX1, int NewPosX2, int NewPosY1, int NewPosY2)
        {
            this.PosX1 = NewPosX1;
            this.PosX2 = NewPosX2;
            this.PosY1 = NewPosY1;
            this.PosY2 = NewPosY2;
        }

        public void SetRotate(Direction NewRotate)
        {
            this.rotate = NewRotate;
        }

        public void SetWidth(int NewWidth)
        {
            this.width = NewWidth;
        }
        

        //-----------------------------------Get metody pre triedu Ports --------------------------------------------
        public string GetName()
        {
            return this.name;
        }

        public string GetModulePath()
        {
            return this.modulePath;
        }

        public string GetOrientation()
        {
            return this.orientation;
        }

        public Nodes GetNode()
        {
            return this.node;
        }

        public string GetDataType()
        {
            return this.dataType;
        }

        public int GetPosX1()
        {
            return this.PosX1;
        }

        public int GetPosX2()
        {
            return this.PosX2;
        }

        public int GetPosY1()
        {
            return this.PosY1;
        }
        public int GetPosY2()
        {
            return this.PosY2;
        }
        public Modules GetModule()
        {
            return this.module;
        }



        public Direction GetRotate()
        {
            return this.rotate;
        }
        public int GetWidth()
        {
            return this.width;
        }
        public Signals GetSignal(int index)
        {
            if (signal[index] == null)
            {
                return null;
            }
            else
            {
                return signal[index];
            }
        }

        public int GetSignalCount()
        {
            return signal.Count;
        }

        public List<Signals> GetSignalList(Boolean actualLevel, String modulePath)
        {
            /*
             * vrati zoznam signalov podla podmienky actualLevel
             * ak aktualLevel == true - signaly prepajajuce moduly na rovnakej alebo vyssej hierarchickej urovni
             * ak aktualLevel == false - signaly prepajajuce moduly na nizsej hierarchickej urovni
             * 
             * */
            int modulePathDots = 0;
            int dots = 0;
            int maxDots = 0;
            List<Signals> tmp = new List<Signals>();

            foreach (char znak in modulePath)
            {
                if (znak == '.')
                    modulePathDots++;
            }

            foreach (Signals item in signal)
            {
                maxDots = 0;
                foreach (Ports port in item.GetPortList())
                {
                    String portPath = port.GetModulePath();
                    dots = 0;
                    foreach (char znak in portPath)
                    {
                        if (znak == '.')
                            dots++;
                    }
                    if (dots > maxDots)
                        maxDots = dots;
                }
                if (actualLevel && maxDots <= modulePathDots)
                {
                    tmp.Add(item);
                }
                else if (!actualLevel && maxDots > modulePathDots)
                {
                    tmp.Add(item);
                }
            }
            if (tmp.Count != 0)
                return tmp;
            else
                return null;
        }

        public void AddSignal(Signals NewSignal)
        {
            signal.Add(NewSignal);
        }
        public RectangleBase GetRectangularShape()
        {
            return this.rectangularShape;
        }
        public IsoscelesTriangleBase GetTriangleShape()
        {
            return this.triangleShape;
        }


        public void Visualize(ref Diagram diagram, ref Project project, bool isVisible)
        {
            double tmp;

            rectangularShape = (RectangleBase)project.ShapeTypes["Box"].CreateInstance();
            triangleShape = (IsoscelesTriangleBase)project.ShapeTypes["IsoscelesTriangle"].CreateInstance();
            nameShape = (TextBase)project.ShapeTypes["Text"].CreateInstance();

            nameShape.Text = this.name;
            nameShape.Height = 0;
            nameShape.Width = 0;
            nameShape.SecurityDomainName = 'G';
            
           

            if (this.rotate == Direction.Left)
            {
                rectangularShape.X = PosX1 - 10;
                rectangularShape.Y = PosY1 + 10;
                rectangularShape.Width = 20;
                rectangularShape.Height = 20;
                rectangularShape.FillStyle = project.Design.FillStyles.Black;

                triangleShape.X = PosX1 + 7;
                triangleShape.Y = PosY1 + 10;
                triangleShape.Width = 20;
                triangleShape.Height = 20;
                triangleShape.Rotate(900, triangleShape.X, triangleShape.Y);
                triangleShape.FillStyle = project.Design.FillStyles.Black;

                
                nameShape.X = PosX1 + 50;
                nameShape.Y = PosY1 + 10;
           
            }
            if (this.rotate == Direction.Right)
            {
                rectangularShape.X = PosX1 - 10;
                rectangularShape.Y = PosY1 + 10;
                rectangularShape.Width = 20;
                rectangularShape.Height = 20;
                

                triangleShape.X = PosX1 + 7;
                triangleShape.Y = PosY1 + 10;
                triangleShape.Width = 20;
                triangleShape.Height = 20;
                triangleShape.Rotate(900, triangleShape.X, triangleShape.Y);
                

                nameShape.X = PosX1 - 45;
                nameShape.Y = PosY1 + 10;
 
            }
            if (double.TryParse(this.value, out tmp) == true)
            {
                if (tmp == 0)
                {
                    rectangularShape.FillStyle = project.Design.FillStyles.Black;
                    rectangularShape.Text = "0";
                    triangleShape.FillStyle = project.Design.FillStyles.Black;
                }
                if (tmp == 1)
                {
                    rectangularShape.FillStyle = project.Design.FillStyles.Red;
                    rectangularShape.Text = "1";
                    triangleShape.FillStyle = project.Design.FillStyles.Red;
                }
                if (tmp != 1 && tmp != 0)
                {
                    triangleShape.FillStyle = project.Design.FillStyles.Black;
                    rectangularShape.Text = "V";
                    rectangularShape.FillStyle = project.Design.FillStyles.Black;
                }
            }
            else 
            {
                rectangularShape.FillStyle = project.Design.FillStyles.Black;
                triangleShape.FillStyle = project.Design.FillStyles.Black;
            }
            
            
            rectangularShape.SecurityDomainName = 'G';
            diagram.Shapes.Add(rectangularShape);
            triangleShape.SecurityDomainName = 'G';
            diagram.Shapes.Add(triangleShape);
            if (isVisible == true)
            {
                diagram.Shapes.Add(nameShape);
            }
        }

        //funkcia na vykreslenie portov z vyssej urovne
        public void VisualizeParentPorts(ref Diagram diagram, ref Project project, bool isVisible)
        {
            rectangularShape = (RectangleBase)project.ShapeTypes["Box"].CreateInstance();
            triangleShape = (IsoscelesTriangleBase)project.ShapeTypes["IsoscelesTriangle"].CreateInstance();
            nameShape = (TextBase)project.ShapeTypes["Text"].CreateInstance();

            nameShape.Text = this.name;
            nameShape.Height = 0;
            nameShape.Width = 0;

            rectangularShape.X = PosX2 - 10;
            rectangularShape.Y = PosY2 + 10;
            rectangularShape.Width = 20;
            rectangularShape.Height = 20;
            rectangularShape.FillStyle = project.Design.FillStyles.Black;
                

            triangleShape.X = PosX2 + 7;
            triangleShape.Y = PosY2 + 10;
            triangleShape.Width = 20;
            triangleShape.Height = 20;
            triangleShape.Rotate(900, triangleShape.X, triangleShape.Y);
            triangleShape.FillStyle = project.Design.FillStyles.Black;
            
            //if (this.width != 1)
            //{
                CircleBase d1;
                CircleBase d2;
                Polyline arrow = (Polyline)project.ShapeTypes["Polyline"].CreateInstance();

                d1 = (CircleBase)project.ShapeTypes["Circle"].CreateInstance();
                d1.FillStyle = project.Design.FillStyles.Black;
                d1.SecurityDomainName = 'G';

                d2 = (CircleBase)project.ShapeTypes["Circle"].CreateInstance();
                d2.FillStyle = project.Design.FillStyles.Black;
                d2.SecurityDomainName = 'G';

                arrow.Connect(ControlPointId.FirstVertex, (Shape)d1, ControlPointId.Reference);
                arrow.Connect(ControlPointId.LastVertex, (Shape)d2, ControlPointId.Reference);

                //if (this.rotate == Direction.Left)
                //{
                    d1.X = PosX2 - 30;
                    d1.Y = PosY2 + 20;

                    d2.X = PosX2 - 30;
                    d2.Y = PosY2 - 20;
                //}
                    diagram.Shapes.Add(arrow);
              
            //}

            rectangularShape.SecurityDomainName = 'G';
            diagram.Shapes.Add(rectangularShape);
            triangleShape.SecurityDomainName = 'G';
            diagram.Shapes.Add(triangleShape);
            if (isVisible == true)
            {
                diagram.Shapes.Add(nameShape);
                nameShape.SecurityDomainName = 'G';
            }
        }
    }
}
