﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

using Dataweb.NShape;
using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape.Layouters;


namespace GraphicalObjects
{
    public class Signals
    {
        private string name;
        private List<Ports> list = new List<Ports>();
        private Nodes startNode;
        private string dataType;
        private bool isClock;
        private bool value;

        public Signals()
        { 
        }
        

        public Signals(Nodes StartNode)
        {
            this.startNode = StartNode;
        }


        //-------------------------------------------- Write to XHDL form ------------------------------------------
        public string GetXHDLString()
        {
            string str = "\t\t<Signal>\n";
            
                str += "\t\t\t<Name>" + name + "</Name>\n";
                str += "\t\t\t<DataType>" + dataType + "</DataType>\n";
                str += "\t\t\t<IsClock>" + isClock.ToString() + "</IsClock>\n";

                str += "\t\t\t<PortMap>\n";                    
                    for(int i=0; i<list.Count; i++) {
                        str += "\t\t\t\t<Map>\n";
                            str += "\t\t\t\t\t<PortName>" + list[i].GetName() + "</PortName>\n";
                            str += "\t\t\t\t\t<ModulePath>" + list[i].GetModulePath() + "</ModulePath>\n";
                        str += "\t\t\t\t</Map>\n";
                    }
                str += "\t\t\t</PortMap>\n";

                str += "\t\t\t<Route>\n";
                    str += startNode.GetXHDLString(4);
                str += "\t\t\t</Route>\n";

            str += "\t\t</Signal>\n";

            return str;
        }

        //---------------------------------------------Metody pre Konzolu-------------------------------------------
        public void WriteToConsole()
        {
            Console.WriteLine("Name: {0}, DataType: {1}, isClock: {2}", name, dataType, isClock);
            Console.WriteLine("\nNodes:");
            if (startNode != null) startNode.WriteToConsole();
        }

        //------------------------------------------Metody Set----------------------------------------



        public void SetName(string NewName)
        {
            this.name = NewName;
        }
        public void SetValue(bool value)
        {
            this.value = value;
        }

        public void SetDataType(string NewDataType)
        {
            this.dataType = NewDataType;
        }
        public void AddPort(Ports Port)
        {
            list.Add(Port);
        }

        public void SetStartNode(Nodes StartNode)
        {
            this.startNode = StartNode;
            
        }
        public Ports GetPort(int index)
        {
            if (list[index] == null)
            {
                return null;
            }
            else
            {
                return list[index];
            }
        }
        public int GetPortCount()
        {
            return list.Count;
        }
        public List<Ports> GetPortList()
        {
            return list;
        }
        public string GetDataType()
        {
            return this.dataType;
        }

        public Nodes GetStartNode()
        {
            return startNode;
        }
        public void SetIsClock(bool IsClock)
        {
            this.isClock = IsClock;
        }
        public bool GetIsClock()
        {
            return this.isClock;
        }
        public string GetName()
        {
            return this.name;
        }
        public bool GetValue()
        {
            return this.value;
        }

        public void Visualize(ref Diagram diagram, ref Project project)
        {
            if (startNode != null)
                startNode.Visualize(ref diagram, ref project);
        }
    }
}