﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace HDLVisualizer
{

    public class Project
    {

        private string Name;
        private List<File> SourceFiles;
        private string Type;
        private string Path;
        private File xhdl_file;
        private File vcd_file;
        private SystemC_SimContext SystemCContext;

        public Project(string name, string type, string path)
        {
            Name = name;
            Type = type;
            Path = path;

            SourceFiles = new List<File>();
            xhdl_file = null;
            vcd_file = null;
            SystemCContext = new SystemC_SimContext();
        }

        public SystemC_SimContext GetSystemCContext()
        {
            return SystemCContext;
        }

        public void setName(string name)
        {
            Name = name;
        }

        public string getName()
        {
            return Name;
        }

        public string getType()
        {
            return Type;
        }

        public void AddSourceFile(string path)
        {
            bool b = false;
            foreach (File f in getAllFiles())
            {
                if (f.getPath() == path)
                {
                    b = true;
                    break;
                }
            }
            if (!b)
            {
                SourceFiles.Add(new File(System.IO.Path.GetFileName(path), path));
            }
        }

        public List<File> getSourceFiles()
        {
            return SourceFiles;
        }

        public List<File> getAllFiles()
        {
            List<File> files = new List<File>();

            foreach (File f in SourceFiles)
                files.Add(f);

            if (xhdl_file != null)
                files.Add(xhdl_file);
            if (vcd_file != null)
                files.Add(vcd_file);

            return files;
        }

        public void addXHDL(string path)
        {
            xhdl_file = new File(System.IO.Path.GetFileName(path), path);
        }

        public File getXHDL()
        {
            return xhdl_file;
        }

        public void addVCD(string path)
        {
            vcd_file = new File(System.IO.Path.GetFileName(path), path);
        }

        public File getVCD()
        {
            return vcd_file;
        }

        public void setPath(string path)
        {
            Path = path;
        }

        public string getPath()
        {
            return Path;
        }
    }

    public class File
    {
        private string Name;
        private string Path;

        public File(string name, string path)
        {
            Name = name;
            Path = path;
        }

        public void setName(string name)
        {
            Name = name;
        }
        public string getName()
        {
            return Name;
        }

        public void setPath(string path)
        {
            Path = path;
        }

        public string getPath()
        {
            return Path;
        }
    }

}
