﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.IO;

namespace HDLVisualizer
{
    public class SystemC_SimContext
    {
        private string[] FileNames;
        private string WorkingDir; 
        private string cygwinWorkingDir; //cygwin style path
        private string Makefile_str = "Makefile";
        private string Makefile_defs_str = "Makefile.defs";
        private string Script_str = "start_make.sh";
        private string SystemC_lib_dir; //cygwin style path


        public SystemC_SimContext() { FileNames = null; }

        public SystemC_SimContext(string ProjectWorkingDirectory, string SystemC_Library_Directory)
        {
            WorkingDir = ProjectWorkingDirectory;
            cygwinWorkingDir = toCygwinPath(ProjectWorkingDirectory);
            SystemC_lib_dir = toCygwinPath(SystemC_Library_Directory);
            FileNames = null;
        }


        public SystemC_SimContext(string SystemC_Library_Directory)
        {
            SystemC_lib_dir = toCygwinPath(SystemC_Library_Directory);
            FileNames = null;
        }

        public void SetSystemC_Library(string lib)
        {
            SystemC_lib_dir = toCygwinPath(lib);
        }

        public void AddFiles(string[] Files)
        {
            string[] tmp = new string[FileNames.Count() + Files.Count()];
            for (int i = 0; i < FileNames.Count(); i++)
                tmp[i] = FileNames[i];
            for (int i = FileNames.Count(); i < tmp.Count(); i++)
                tmp[i] = Files[i-FileNames.Count()];
        }

        /* cygwin style path wrapping */
        private string toCygwinPath(string WindowsPath)
        {
            string str = "";
            string tmp_str = WindowsPath.Trim();

            str += "/cygdrive/";
            /* TBD checking if it is absolute path */
            str += Char.ToLower(tmp_str[0]);
            str += '/';

            tmp_str = tmp_str.Remove(0,3); // napr.: C:\
            int space = -1;
            int back_slash = -1;
     
            while(tmp_str.Count() > 0){
                space = tmp_str.IndexOf(' ');
                back_slash = tmp_str.IndexOf('\\');
                if( space == -1 && back_slash == -1) {
                    str += tmp_str;
                    break;
                } else if( (back_slash != -1 && back_slash < space) || space == -1 )
                {
                    str += tmp_str.Substring(0, back_slash);
                    tmp_str = tmp_str.Remove(0,back_slash+1);
                    if(back_slash > 1) str += '/';
                    //tmp_str.Remove(0,1);
                } else { // space < back_slash
                    str += tmp_str.Substring(0, space);
                    str += "\\ "; // cygwin stype space wrapping
                    tmp_str = tmp_str.Remove(0,space+1);
                }
            }

            return str;
        }

        private bool iscpp(string path)
        {
            if( path.LastIndexOf('.') == -1) return false;
            if( path.Substring(path.LastIndexOf('.')) == ".cpp") return true;
            return false;
        }
   
        public void SetFileNames(string[] Names)
        {
            FileNames = Names;
        }

        public string[] GetFileNames()
        {
            return FileNames;
        }

        public void SetWorkingDir(string dir)
        {
            WorkingDir = dir;
            cygwinWorkingDir = toCygwinPath(dir);    
        }

        public string GetWorkingDir()
        {
            return WorkingDir;
        }

        public int MakeProject()
        {
            if (FileNames == null) return 3;
            TextWriter Makefile;
            TextWriter Makefile_defs;
            TextWriter Script;

            Makefile = new StreamWriter(WorkingDir + "/" + Makefile_str);
            Makefile.NewLine = "\n";

            Makefile_defs = new StreamWriter(WorkingDir + "/" + Makefile_defs_str);
            Makefile_defs.NewLine = "\n";

            Script = new StreamWriter(WorkingDir + "/" + Script_str);
            Script.NewLine = "\n";

            Makefile.WriteLine("CC = g++");
            Makefile.WriteLine("OPT = -O2");
            Makefile.WriteLine("DEBUG = -g");
            Makefile.WriteLine("OTHER = -Wall");
            Makefile.WriteLine("CFLAGS = -DSC_INCLUDE_FX -DSC_USE_PTHREADS  $(DEBUG) $(OPT) $(OTHER)");
            Makefile.WriteLine("MODULE = run");
            Makefile.Write("SRCS =");
            
            //Makefile.WriteLine("stimulus.cpp display.cpp fir_data.cpp fir_fsm.cpp fir.cpp main_rtl.cpp");
            foreach (string str in FileNames)
                if (iscpp(str)) Makefile.Write(str.Substring(str.LastIndexOf('\\')+1) + " ");
            Makefile.WriteLine();
            //for (int i = 0; i < FileNames.Count(); i++) Makefile.Write(" "+FileNames[i]);
            Makefile.WriteLine();
            Makefile.WriteLine("OBJS = $(SRCS:.cpp=.o)");
            Makefile.WriteLine("include Makefile.defs");
            Makefile.Close();

            Makefile_defs.Write("SYSTEMC = ");
            Makefile_defs.WriteLine(SystemC_lib_dir);
            Makefile_defs.WriteLine( "INCDIR = -I. -I.. -I$(SYSTEMC)/include\n" +
                                        "LIBDIR = -L. -L.. -L$(SYSTEMC)\n" +
                                        "LIBS   =  -lsystemc -lm $(EXTRA_LIBS)\n" +
                                        "EXE    = $(MODULE).exe\n" +
                                        ".SUFFIXES: .cc .cpp .o .x\n" +
                                        "$(EXE): $(OBJS)\n" +
                                            "\t$(CC) $(CFLAGS) $(INCDIR) $(LIBDIR) -o $@ $(OBJS) $(LIBS) 2>&1 | c++filt\n" +
                                        ".cpp.o:\n" +
                                            "\t$(CC) $(CFLAGS) $(INCDIR) -c $<\n" +
                                        ".cc.o:\n" +
                                            "\t$(CC) $(CFLAGS) $(INCDIR) -c $<\n" +
                                        "clean::\n" +
                                            "\trm -f $(OBJS) *~ core #$(EXE) core\n" +
                                        "ultraclean: clean\n" +
                                            "\trm -f Makefile.deps");
            Makefile_defs.Close();

            Script.Write("#! /bin/bash\ncd ");
            Script.WriteLine(cygwinWorkingDir);
            Script.WriteLine(  "make\n" +
                                "make clean\n" +
                                "./run.exe\n");
            Script.WriteLine();
            Script.Close();

            return 0;

        }
    }
}
