﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;


using Dataweb.NShape;
using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape.Layouters;

using GraphicalObjects;

using HDL_XML_Parser;
using HDLVisualizerDefinitions;

using VCDToXML;

namespace HDLVisualizer
{
    public partial class TabPageProject : TabPage
    {
        List<File> files;
        Project project;

        public DisplayInitializator Initializator;
        public VHDLDisplay display;
        public List<Modules> list;
        public PositionGenerator Generator;


        public TabPageProject(string name, Project pr)
        {
            InitializeComponent();
            this.Name = name;
            this.Text = name;
            project = pr;

            files = pr.getAllFiles();
            foreach (File f in files)
                this.SourceFiles.Items.Add(f.getName());
            if(SourceFiles.Items.Count != 0)
                SourceFiles.SelectedIndex = 0;
        }

        public void visualization(ref VHDLDisplay display, ref List<Modules> list, ref DisplayInitializator Initializator, ref PositionGenerator generator)
        {
            Initializator.AddingModuleToTreeView(ref treeView1, ref list);
            this.Initializator = Initializator;
            this.display = display;
            this.list = list;
            this.Generator = generator;
        }
        private void treeView1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int count = 0;
            string path = "";
            Modules parentModule = null;

            List<Modules> actualList = new List<Modules>();

            foreach (Modules module in list)
            {
                if (module.GetName() == (treeView1.SelectedNode.Text))
                {
                    count = module.GetPath().ToCharArray().Count(x => x == '.');
                    path = module.GetPath();
                    parentModule = module;
                    break;
                }
            }
            actualList.Clear();
            foreach (Modules module in list)
            {
                if (((module.GetPath().StartsWith(path)) == true) && (module.GetPath().ToCharArray().Count(x => x == '.') == count + 1))
                {
                    actualList.Add(module);
                }

            }

            Initializator.VisualizeAllGraphicalObjects(ref display, ref Generator, actualList, parentModule);
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            List<Modules> actualList = new List<Modules>();
            Modules parentModule = null;
            actualList = Initializator.GetActualList();
            parentModule = Initializator.GetParentModule();
            Initializator.SetVisualizeNameOfModule(checkBox1.Checked);
            Initializator.VisualizeAllGraphicalObjects(ref display, ref Generator, actualList, parentModule);            
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            List<Modules> actualList = new List<Modules>();
            Modules parentModule = null;
            actualList = Initializator.GetActualList();
            parentModule = Initializator.GetParentModule();
            Initializator.SetVisualizeNameOfPort(checkBox2.Checked);
            Initializator.VisualizeAllGraphicalObjects(ref display, ref Generator, actualList, parentModule);
        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e)
        {
            List<Modules> actualList = new List<Modules>();
            Modules parentModule = null;
            actualList = Initializator.GetActualList();
            parentModule = Initializator.GetParentModule();
            Initializator.SetVisualizeNameOfSignal(checkBox3.Checked);
            Initializator.VisualizeAllGraphicalObjects(ref display, ref Generator, actualList, parentModule);
        }
        private void button6_Click(object sender, EventArgs e)
        {

            List<Modules> actualList = new List<Modules>();
            Modules parentModule = null;
            if (Initializator.GetActualList().Count() != 0)
            {
                Modules m = Initializator.GetActualList()[0];
                int count;
                string path;

                path = m.GetPath();
                if (path.LastIndexOf('.') > 0)
                {
                    path = path.Remove(path.LastIndexOf('.'));
                }
                if (path.LastIndexOf('.') > 0)
                {
                    path = path.Remove(path.LastIndexOf('.'));
                }
                actualList.Clear();
                count = path.ToCharArray().Count(x => x == '.');
                foreach (Modules module in list)
                {
                    if (((module.GetPath().StartsWith(path)) == true) && (module.GetPath().ToCharArray().Count(x => x == '.') == count + 1))
                    {
                        actualList.Add(module);
                    }
                    if (module.GetPath().Equals(path))
                    {
                        parentModule = module;
                    }
                }
                Initializator.VisualizeAllGraphicalObjects(ref display, ref Generator, actualList, parentModule);
            }
        }

        int lastindex = -1;
        private void SourceFiles_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (lastindex != -1)
                {
                    Directory.CreateDirectory(Path.GetDirectoryName(project.getAllFiles()[lastindex].getPath()));
                    StreamWriter writer = new StreamWriter(project.getAllFiles()[lastindex].getPath());
                    writer.Write(this.code.Text);
                    writer.Close();
                }
            }
            catch
            {
            }

            try
            {
                TextReader reader = new StreamReader(project.getAllFiles()[SourceFiles.SelectedIndex].getPath());
                this.code.Text = reader.ReadToEnd();
                reader.Close();
            }
            catch
            {
                this.code.Text = "";
            }

            lastindex = SourceFiles.SelectedIndex;
        }

        public void RefreshList()
        {
            object item = SourceFiles.SelectedItem;
            SourceFiles.Items.Clear();
            List<File> list = project.getAllFiles();
            foreach (File f in list)
            {
                SourceFiles.Items.Add(f.getName());
            }

            if(item != null && SourceFiles.Items.Contains(item))

            {
                SourceFiles.SelectedItem = item;
            }
            else if(SourceFiles.Items.Count != 0)
            {
                SourceFiles.SelectedIndex = 0;
            }
        }

        VCDToXML.VCDToXML vcdToxml;
                List<Variable> variables;

        public void simulation(string path)
        {
            groupBox2.Enabled = true;
            vcdToxml = new VCDToXML.VCDToXML();
            vcdToxml.Parse(path);
            variables = vcdToxml.GetVars();

            int maxtime = 0;

            foreach(Variable v in variables)
            {
                int max = v.getMaxTime();
                if (max > maxtime)
                    maxtime = max;
            }

            trackBar1.Minimum = 0;
            trackBar1.Maximum = maxtime;
            trackBar1.TickFrequency = 1;
        }
        

        private void trackBar1_Scroll(object sender, EventArgs e)
        {

            toolTip1.SetToolTip(trackBar1, trackBar1.Value.ToString());

            //prechadzame vsetky vykreslene moduly
            foreach(Modules module in Initializator.GetActualList())
            {
                //prechadzame vsetky porty modulov
                foreach (Ports port in module.GetPortList())
                {
                    //hladame variable s rovnakou cestou ako port
                    foreach (Variable var in variables)
                    {
                        //ak najdeme, tak nastavime hodnotu portu
                        string a = var.FullName;
                        string b = port.GetModulePath() + "." + port.GetName();
                        if (var.FullName == (port.GetModulePath() + "." + port.GetName()))
                        {
                            //setting port value
                            string value = var.GetValuaAtTime(trackBar1.Value);
                            port.SetValue(value);
                            Initializator.VisualizeAllGraphicalObjects(ref display, ref Generator, Initializator.GetActualList(), Initializator.GetParentModule());
                            break;
                        }
                    }
                }
            }
        }

        public string BitsToInt(string bits)
        {
            int cislo = 0;
            int hodnota = 1;
            for (int i = bits.Length - 2; i >= 0; i--)
            {
                if (bits[i] == '1')
                    cislo += hodnota;
                hodnota *= 2;
            }
            if (cislo != 0)
                return cislo.ToString();
            else
                return bits;
        }
    }
}
