grammar VCD;

options
{
	language=CSharp2;
	output=AST;
	//ASTLabelType= pANTLR3_BASE_TREE;
	//tokenVocab = Verilog;  //call the vocabulary 'Verilog'
	//testLiterals = false;  //don't automatically test for literals
	k = 5;                  // 3 characters of lookahead
}

	
vcd_file_content
	:
	(declaration_command)*
	enddefinitions
	(simulation_command)*
	EOF
	;
	
declaration_command
	:	
	((KW_DATE | KW_COMMENT | KW_VERSION | KW_UPSCOPE) command_text KW_END)
	| KW_TIMESCALE BINARY_VALUE KW_END
	| KW_TIMESCALE VALUE TIMESCALE_UNIT KW_END
	| KW_SCOPE SCOPE_TYPE IDENTIFIER KW_END
	| variable
	;	 
variable
	:	
	KW_VAR variable_type variable_size variable_code variable_reference KW_END
	;

variable_type
	:	VAR_TYPE
	;

variable_size
	:	VALUE // kvoli problemu s lexerom, ktory nedokaze rozoznat DECIMAL od VALUE a TIMESCALE_NUMBER, kedze vsetky mozu zacinat 0 alebo 1, je VALUE aj dekadicke cislo, aj stvorstavova premenna
	;

variable_code:	
	(VALUE) => VALUE | IDENTIFIER
	;

variable_reference:	
	IDENTIFIER
	;

enddefinitions
	:	KW_ENDDEFINITIONS KW_END
	;

simulation_command
	:	
	simulation_with_kw
	| simulation_time
	| simulation_text
	;
	
simulation_with_kw
	:	simulation_kw (simulation_text)*  KW_END
	;
	
simulation_kw
	:	(KW_DUMPALL | KW_DUMPOFF | KW_DUMPON | KW_DUMPVARS)
	;

simulation_time
	:	SIMULATION_TIME
	;

simulation_text
	:		
	simulation_value | simulation_vector_value | simulation_float_value 
	;
	
simulation_value
	:	VALUE IDENTIFIER | BINARY_VALUE
	;

simulation_vector_value
	:	VECTOR_VALUE IDENTIFIER
	;
	
simulation_float_value
	:	FLOAT IDENTIFIER
	;



WS	:	(' '|'\t'|'\n')+ {$channel=HIDDEN;};

KW_END	:	'$end';

KW_DATE	:	'$date';

KW_SCOPE:	'$scope';

KW_TIMESCALE
	:	'$timescale';
	
KW_UPSCOPE
	:	'$upscope';

KW_VAR	:	'$var';

KW_VERSION
	:	'$version';
	
KW_COMMENT
	:	'$comment';

KW_ENDDEFINITIONS
	:	'$enddefinitions';
	
KW_DUMPALL
	:	'$dumpall';
	
KW_DUMPOFF
	:	'$dumpoff';
	
KW_DUMPON
	:	'$dumpon';

KW_DUMPVARS
	:	'$dumpvars';
		
	
VAR_TYPE:
	('event' | 'integer' | 'parameter' | 'real' | 'reg' | 'supply0' | 'supply1' | 'time'
	| 'tri' | 'triand' | 'trior' | 'trireg' | 'tri0' | 'tri1' | 'wand' | 'wire' | 'wor')
	;

SCOPE_TYPE
	:	
	'begin'|'fork'|'function'|'module'|'task'
	;

SIMULATION_TIME
	:	
	'#'('0'..'9')('0'..'9')*
	;

VECTOR_VALUE
	:
	('b'|'B')('0' | '1' | 'x' |'X' | 'z' | 'Z')+ (' ')
	;

	
BINARY_VALUE
	:	('0'|'1'| 'x' |'X' | 'z' | 'Z')('\u0021'..'\u007F')('\u0021'..'\u007F')*
	;
	
VALUE	:	('0'..'9')+ | 'x' |'X' | 'z' | 'Z'
	;

	
//TIMESCALE_NUMBER
//	:
//	('1' | '10' | '100')('s' | 'ms' | 'us' | 'ns' | 'ps' | 'fs')
//	;
	
TIMESCALE_UNIT:
	('s' | 'ms' | 'us' | 'ns' | 'ps' | 'fs')
	;

FLOAT
    :   ('r'|'R') 
    (('0'..'9')+ '.' ('0'..'9')* EXPONENT?
    |   '.' ('0'..'9')+ EXPONENT?
    |   ('0'..'9')+ EXPONENT)
    ;

fragment
EXPONENT : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

IDENTIFIER
	: 
	('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')* '[' ('0'..'9')+ ']'
	| ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')* '[' ('0'..'9')+ ':' ('0'..'9')+ ']'
	| ('\u0021'..'\u007F') ('\u0021'..'\u007F')*
	;

command_text
	:	
	(~('$end'))* 
	;
