﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using System.Xml;
using System.Xml.XPath;
using System.Drawing;
using System.Xml.Linq;

namespace VCDToXML
{
    public class VCDToXML
    {
        //private XDocument _xdoc;
        private String _version;
        private String _date;
        private TimeScale _timescaleunit;
        private int _timescalevalue;
        private String _comment;
        //private TimeScale _currentscale;
        private Time _currenttime;

        private List<Variable> _vars;
        private List<String> _scopename;
        private List<ScopeType> _scopetype;

                public event EventHandler ProgressChanged;

        public VCDToXML()
        {
            _vars = new List<Variable>();
            _scopename = new List<string>();
            _scopetype = new List<ScopeType>();
            //this.ProgressBarUp = new EventHandler();
        }
        public void Parse(String filename)
        {
            try
            {
                if((new System.IO.FileInfo(filename).Length > 524288))
                    throw new Exception("Error: Size of VCD file is too big! Parser can handle only VCD file with size smaller than 0,5 MB.");
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
                VCDLexer lex = new VCDLexer(new ANTLRFileStream(filename));
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
                CommonTokenStream tokens = new CommonTokenStream(lex);
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
                VCDParser parser = new VCDParser(tokens);
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());

                CommonTree tree = (CommonTree)parser.vcd_file_content().Tree;
                int index = 0;

                ParseDate(tree, ref index);
                ParseVersion(tree, ref index);
                ParseTimeScale(tree, ref index);
                ParseVarsVHDL(tree, ref index); //for TopArch in VHDL
                while(((CommonTree)tree.Children[index]).Type != VCDLexer.EOF && ((CommonTree)tree.Children[index]).Type == VCDLexer.KW_SCOPE)
                    ParseScope(tree, ref index);
                ParseEnddefinitions(tree, ref index);

                while (((CommonTree)tree.Children[index]).Type != VCDLexer.EOF)
                {
                    if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_DUMPVARS
                        || ((CommonTree)tree.Children[index]).Type == VCDLexer.KW_DUMPALL
                        || ((CommonTree)tree.Children[index]).Type == VCDLexer.KW_DUMPON
                        || ((CommonTree)tree.Children[index]).Type == VCDLexer.KW_DUMPOFF)
                        ParseDump(tree, ref index);
                    else if (((CommonTree)tree.Children[index]).Type == VCDLexer.SIMULATION_TIME)
                        _currenttime = new Time(_timescaleunit, _timescalevalue, ((CommonTree)tree.Children[index++]).Text.Remove(0, 1));
                    else if ((((CommonTree)tree.Children[index]).Type == VCDLexer.VALUE && ((CommonTree)tree.Children[index+1]).Type == VCDLexer.IDENTIFIER)
                        || ((CommonTree)tree.Children[index]).Type == VCDLexer.VECTOR_VALUE
                        || ((CommonTree)tree.Children[index]).Type == VCDLexer.FLOAT)
                    {
                        foreach (Variable v in _vars)
                            if (v.ID == ((CommonTree)tree.Children[index + 1]).Text)
                                v.AddValue(_currenttime, ((CommonTree)tree.Children[index]).Text);
                        index += 2;
                    }
                    else if (((CommonTree)tree.Children[index]).Type == VCDLexer.BINARY_VALUE)
                    {
                        if (((CommonTree)tree.Children[index]).Text.Length > 1)
                            foreach (Variable v in _vars)
                                if (v.ID == ((CommonTree)tree.Children[index]).Text.Substring(1))
                                    v.AddValue(_currenttime, "" + ((CommonTree)tree.Children[index]).Text[0]);
                        index++;
                    }
                    else
                        index++;
                }
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
            
        private void ParseDate(CommonTree tree, ref int index) // naparsuje datum simulacie do stringu _date
        {
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_DATE)
            {
                while (((CommonTree)tree.Children[index + 1]).Type != VCDLexer.KW_END && ((CommonTree)tree.Children[index + 1]).Type != VCDLexer.EOF)
                    _date += ((CommonTree)tree.Children[++index]).Text + " ";
                if (((CommonTree)tree.Children[++index]).Type == VCDLexer.KW_END)
                    index++;
                else if (((CommonTree)tree.Children[index]).Type == VCDLexer.EOF)
                    throw new Exception("Unexpected EOF! After keyword \"$date\" must follow date string and \"$end\" keyword.");
                else
                    throw new Exception("Syntax error! After keyword \"$date\" must follow date string and \"$end\" keyword.");
                _date.Trim();
            }
            else
                _date = "No data found!";
        }
        private void ParseVersion(CommonTree tree, ref int index) // naparsuje verziu simulatora do stringu _version
        {
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_VERSION)
            {
                while (((CommonTree)tree.Children[index + 1]).Type != VCDLexer.KW_END && ((CommonTree)tree.Children[index + 1]).Type != VCDLexer.EOF)
                    _version += ((CommonTree)tree.Children[++index]).Text + " ";
                if (((CommonTree)tree.Children[++index]).Type == VCDLexer.KW_END)
                    index++;
                else if (((CommonTree)tree.Children[index]).Type == VCDLexer.EOF)
                    throw new Exception("Unexpected EOF! After keyword \"$version\" must follow date string and \"$end\" keyword.");
                else
                    throw new Exception("Syntax error! After keyword \"$version\" must follow date string and \"$end\" keyword.");
                _version.Trim();
            }
            else
                _version = "No data found!";
        }
        private void ParseTimeScale(CommonTree tree, ref int index) // naparsuje timescale simulacie
        {
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_TIMESCALE)
            {
                if (((CommonTree)tree.Children[++index]).Type == VCDLexer.BINARY_VALUE)
                    switch (((CommonTree)tree.Children[index]).Text)
                    {
                        case "1s":
                            _timescaleunit = TimeScale.s;
                            _timescalevalue = 1;
                            break;
                        case "10s":
                            _timescaleunit = TimeScale.s;
                            _timescalevalue = 10;
                            break;
                        case "100s":
                            _timescaleunit = TimeScale.s;
                            _timescalevalue = 100;
                            break;
                        case "1ms":
                            _timescaleunit = TimeScale.ms;
                            _timescalevalue = 1;
                            break;
                        case "10ms":
                            _timescaleunit = TimeScale.ms;
                            _timescalevalue = 10;
                            break;
                        case "100ms":
                            _timescaleunit = TimeScale.ms;
                            _timescalevalue = 100;
                            break;
                        case "1us":
                            _timescaleunit = TimeScale.us;
                            _timescalevalue = 1;
                            break;
                        case "10us":
                            _timescaleunit = TimeScale.us;
                            _timescalevalue = 10;
                            break;
                        case "100us":
                            _timescaleunit = TimeScale.us;
                            _timescalevalue = 100;
                            break;
                        case "1ns":
                            _timescaleunit = TimeScale.ns;
                            _timescalevalue = 1;
                            break;
                        case "10ns":
                            _timescaleunit = TimeScale.ns;
                            _timescalevalue = 10;
                            break;
                        case "100ns":
                            _timescaleunit = TimeScale.ns;
                            _timescalevalue = 100;
                            break;
                        case "1ps":
                            _timescaleunit = TimeScale.ps;
                            _timescalevalue = 1;
                            break;
                        case "10ps":
                            _timescaleunit = TimeScale.ps;
                            _timescalevalue = 10;
                            break;
                        case "100ps":
                            _timescaleunit = TimeScale.ps;
                            _timescalevalue = 100;
                            break;
                        case "1fs":
                            _timescaleunit = TimeScale.fs;
                            _timescalevalue = 1;
                            break;
                        case "10fs":
                            _timescaleunit = TimeScale.fs;
                            _timescalevalue = 10;
                            break;
                        case "100fs":
                            _timescaleunit = TimeScale.fs;
                            _timescalevalue = 100;
                            break;
                        default:
                            _timescaleunit = TimeScale.s;
                            _timescalevalue = 1;
                            break;
                    }
                else if (((CommonTree)tree.Children[index]).Type == VCDLexer.VALUE
                    && ((CommonTree)tree.Children[index + 1]).Type == VCDLexer.TIMESCALE_UNIT)
                {
                    switch (((CommonTree)tree.Children[index]).Text)
                    {
                        case "1":
                            _timescalevalue = 1;
                            break;
                        case "10":
                            _timescalevalue = 10;
                            break;
                        case "100":
                            _timescalevalue = 100;
                            break;
                        default:
                            _timescalevalue = 1;
                            break;
                    }
                    switch (((CommonTree)tree.Children[++index]).Text)
                    {
                        case "s":
                            _timescaleunit = TimeScale.s;
                            break;
                        case "ms":
                            _timescaleunit = TimeScale.ms;
                            break;
                        case "us":
                            _timescaleunit = TimeScale.us;
                            break;
                        case "ns":
                            _timescaleunit = TimeScale.ns;
                            break;
                        case "ps":
                            _timescaleunit = TimeScale.ps;
                            break;
                        case "fs":
                            _timescaleunit = TimeScale.fs;
                            break;
                        default:
                            _timescaleunit = TimeScale.s;
                            break;
                    }
                }
                else
                    throw new Exception("Syntax error! After keyword \"$timescale\" must follow timescale string like \"1 s\" and \"$end\" keyword.");

                if (((CommonTree)tree.Children[++index]).Type == VCDLexer.KW_END)
                    index++;
                else if (((CommonTree)tree.Children[index]).Type == VCDLexer.EOF)
                    throw new Exception("Unexpected EOF! After keyword \"$timescale\" must follow date string and \"$end\" keyword.");
                else
                    throw new Exception("Syntax error! After keyword \"$timescale\" must follow date string and \"$end\" keyword.");
            }
            else
            {
                _timescaleunit = TimeScale.s;
                _timescalevalue = 0;
            }
        }
        private void ParseComment(CommonTree tree, ref int index) // naparsuje komentar do stringu _comment
        {
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_COMMENT)
            {
                while (((CommonTree)tree.Children[index + 1]).Type != VCDLexer.KW_END && ((CommonTree)tree.Children[index + 1]).Type != VCDLexer.EOF)
                    _comment += ((CommonTree)tree.Children[++index]).Text + " ";
                if (((CommonTree)tree.Children[++index]).Type == VCDLexer.KW_END)
                    index++;
                else if (((CommonTree)tree.Children[index]).Type == VCDLexer.EOF)
                    throw new Exception("Unexpected EOF! After keyword \"$comment\" must follow date string and \"$end\" keyword.");
                else
                    throw new Exception("Syntax error! After keyword \"$comment\" must follow date string and \"$end\" keyword.");
                _version.Trim();
            }
            else
                _version = "No data found!";
        }

        private void ParseScope(CommonTree tree, ref int index) // naparsuje premenne vzhladom na hierarchiu modulov
        {
            while (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_SCOPE
                && ((CommonTree)tree.Children[index + 1]).Type == VCDLexer.SCOPE_TYPE
                && ((CommonTree)tree.Children[index + 2]).Type == VCDLexer.IDENTIFIER
                && ((CommonTree)tree.Children[index + 3]).Type == VCDLexer.KW_END)
            {

                switch (((CommonTree)tree.Children[++index]).Text)
                {
                    case "module":
                        _scopetype.Add(ScopeType.module);
                        break;
                    case "begin":
                        _scopetype.Add(ScopeType.begin);
                        break;
                    case "fork":
                        _scopetype.Add(ScopeType.fork);
                        break;
                    case "function":
                        _scopetype.Add(ScopeType.function);
                        break;
                    case "task":
                        _scopetype.Add(ScopeType.task);
                        break;
                    default:
                        _scopetype.Add(ScopeType.module);
                        break;
                }
                _scopename.Add(((CommonTree)tree.Children[++index]).Text);
                index += 2;
                if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_SCOPE)
                    ParseScope(tree, ref index);
                if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_VAR)
                    ParseVars(tree, ref index);
                if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_UPSCOPE)
                    ParseUpscope(tree, ref index);
                else
                    throw new Exception("Syntax error! After \"$scope\" declaration must follow variable declaration or next scope declaration or upscope.\n" +
                        ((CommonTree)tree.Children[index]).Text + " " +
                        ((CommonTree)tree.Children[index + 1]).Text + " " +
                        ((CommonTree)tree.Children[index + 2]).Text + " " +
                        ((CommonTree)tree.Children[index + 3]).Text + " " +
                        ((CommonTree)tree.Children[index + 4]).Text + " " +
                        ((CommonTree)tree.Children[index+5]).Text + "\n" +
                        ((CommonTree)tree.Children[index]).Type.ToString() + " sa nerovna " + VCDLexer.KW_VAR.ToString()
                );
            }
        }

        private void ParseVars(CommonTree tree, ref int index) // naparsuje deklaracie vsetkych premennych v danom module
        {
            while (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_VAR
                && ((CommonTree)tree.Children[index + 1]).Type == VCDLexer.VAR_TYPE
                && ((CommonTree)tree.Children[index + 2]).Type == VCDLexer.VALUE
                //&& (((CommonTree)tree.Children[index + 3]).Type == VCDLexer.IDENTIFIER || ((CommonTree)tree.Children[index + 3]).Type == VCDLexer.VALUE)
                && ((CommonTree)tree.Children[index + 4]).Type == VCDLexer.IDENTIFIER
                && ((CommonTree)tree.Children[index + 5]).Type == VCDLexer.KW_END)
            {
                _vars.Add(
                    new Variable(_scopename, _scopetype,
                        ((CommonTree)tree.Children[++index]).Text,
                        System.Convert.ToInt16(((CommonTree)tree.Children[++index]).Text),
                        ((CommonTree)tree.Children[++index]).Text,
                        ((CommonTree)tree.Children[++index]).Text));
                index+=2;
            }
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_SCOPE)
                ParseScope(tree, ref index);
        }

        private void ParseVarsVHDL(CommonTree tree, ref int index) // naparsuje deklaracie vsetkych premennych v danom module
        {
            while (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_VAR
                && ((CommonTree)tree.Children[index + 1]).Type == VCDLexer.VAR_TYPE
                && ((CommonTree)tree.Children[index + 2]).Type == VCDLexer.VALUE
                //&& (((CommonTree)tree.Children[index + 3]).Type == VCDLexer.IDENTIFIER || ((CommonTree)tree.Children[index + 3]).Type == VCDLexer.VALUE)
                && ((CommonTree)tree.Children[index + 4]).Type == VCDLexer.IDENTIFIER
                && ((CommonTree)tree.Children[index + 5]).Type == VCDLexer.KW_END)
            {
                /*
                _vars.Add(
                    new Variable(_scopename, _scopetype,
                        ((CommonTree)tree.Children[++index]).Text,
                        System.Convert.ToInt16(((CommonTree)tree.Children[++index]).Text),
                        ((CommonTree)tree.Children[++index]).Text,
                        ((CommonTree)tree.Children[++index]).Text));
                 * */
                index += 6;
                
            }
            //if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_SCOPE)
              // ParseScope(tree, ref index);
        }

        private void ParseUpscope(CommonTree tree, ref int index) // parsuje upscope konstrukciu, ktora predstavuje prechod na vysiu hierarchicku uroven medzi modulmi
        {
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_UPSCOPE
                && ((CommonTree)tree.Children[index + 1]).Type == VCDLexer.KW_END)
            {
                _scopename.RemoveAt(_scopename.Count - 1);
                _scopetype.RemoveAt(_scopetype.Count - 1);
                index+=2;
            }
        }

        private void ParseEnddefinitions(CommonTree tree, ref int index) // parsuje enddefinitions, ktora ukoncuje cast deklaracii
        {
            if (((CommonTree)tree.Children[index]).Type == VCDLexer.KW_ENDDEFINITIONS
                && ((CommonTree)tree.Children[index + 1]).Type == VCDLexer.KW_END)
                index += 2;
        }
        private void ParseDump(CommonTree tree, ref int index) // parsuje hodnoty premennych v danom case v kontexte dumpvars, dumpall, dumpon a dumpoff
        {
            while (((CommonTree)tree.Children[++index]).Type != VCDLexer.KW_END)
                for (int i = 0; i < _vars.Count;i++ )
                {
                    if ((((CommonTree)tree.Children[index]).Type == VCDLexer.BINARY_VALUE)
                        && _vars[i].ID == ((CommonTree)tree.Children[index]).Text.Substring(1))
                        _vars[i].AddValue(_currenttime, "" + ((CommonTree)tree.Children[index]).Text[0]);
                    else if (_vars[i].ID == ((CommonTree)tree.Children[index + 1]).Text)
                    {
                        _vars[i].AddValue(_currenttime, ((CommonTree)tree.Children[index]).Text);
                        i++;
                    }
                    
                }
            index++;
        }


        public XDocument GenerateXML(XDocument x)
        {
            XDocument xdoc = new XDocument(x);
            /*if (xdoc.Root.Attribute("Simulation").Value == "true")
                foreach (XElement xmodule in xdoc.Element("VerilogProject").Element("Modules").Elements("Module"))
                {
                    if (xmodule.Element("VerilogVariables") != null)
                        xmodule.Element("VerilogVariables").Remove();
                    foreach (XElement xinstance in xmodule.Element("VerilogInstances").Elements("ModuleInstance"))
                        if (xinstance.Element("VerilogVariables") != null)
                            xinstance.Element("VerilogVariables").Remove();
                }*/

            foreach (Variable var in _vars)
            {
                XElement minstance = FindInstance(xdoc, var.ScopeHierarchy);
                if (minstance.Element("VerilogVariables") == null)
                {
                    minstance.Add(new XElement("VerilogVariables"));
                    minstance.Element("VerilogVariables").Add(
                    new XElement("VerilogVariable",
                        new XAttribute("Name", var.FullName),
                        new XAttribute("Type", var.Type),
                        new XAttribute("Scope", var.Scope),
                        var.GetXTimeline()));
                }
                else
                {
                    if (minstance.Element("VerilogVariables").Elements("VerilogVariable") != null)
                        foreach (XElement xvar in minstance.Element("VerilogVariables").Elements("VerilogVariable"))
                            if (xvar.Attribute("Name").Value == var.FullName)
                                xvar.Remove();
                    minstance.Element("VerilogVariables").Add(
                        new XElement("VerilogVariable",
                            new XAttribute("Name", var.FullName),
                            new XAttribute("Type", var.Type),
                            new XAttribute("Scope", var.Scope),
                            var.GetXTimeline()));
                }
            }
            xdoc.Element("VerilogProject").Attribute("Simulation").Value = "true";
            if (ProgressChanged != null)
                ProgressChanged.Invoke(this, new EventArgs());
            return xdoc;
        }

        private XElement FindInstance(XDocument xdoc, List<String> scopename)
        {
            IEnumerable<XElement> modules;
            IEnumerable<XElement> instances;
            XElement instance = null;
            XElement module = null;
            bool b = true;
            modules = xdoc.Element("VerilogProject").Element("Modules").Elements("Module");
            foreach (XElement mod in modules)
                if (mod.Attribute("ModuleName").Value == scopename[0])
                {
                    module = mod;
                    break;
                }
            if (module == null)
                throw (new Exception("Corrupted VCD file or not VCD file of current verilog project!"));
            for (int i = 1; i < scopename.Count; i++)
            {
                instances = module.Element("VerilogInstances").Elements("ModuleInstance");
                b = true;
                foreach (XElement ins in instances)
                    if (ins.Element("InstanceName").Value == scopename[i])
                    {
                        instance = ins;
                        b = false;
                        break;
                    }
                if (instance == null)
                    return module;  // premenna je v main module
                if (b)
                    return instance; // premenna je v instancii aktualneho modulu
                module = null;
                foreach (XElement mod in modules)
                    if (mod.Attribute("ModuleName").Value == instance.Element("ModuleName").Value)
                    {
                        module = mod;
                        break;
                    }
                if (module == null)
                    return instance;
            }
            if (instance != null)
                return instance;
            else
                return module;
        }

        //testing
        public List<Variable> GetVars()
        {
            return this._vars;
        }
        //testing

    }
}
