﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VerilogToXML;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Linq;

namespace VCDToXML
{
    public enum SignalType{register, net, _event, parameter};
    public enum ScopeType{begin, fork, function, module, task};
    //public enum Value{v0, v1, vX, vZ};
    public class Variable
    {
        private String _name;
        private String _id;
        private SignalType _type;
        private int _size;
        private NetType _nettype;
        private RegType _regtype;
        private List<String> _scopename;
        private List<ScopeType> _scopetype;

        private List<Object> _data;
        private List<Time> _time;

        public String ID
        {
            get { return _id; }
            set { _id = value; }
        }
        public String FullName
        {
            get
            {
                String n = "";
                foreach (String s in _scopename)
                    n += s + ".";
                n += _name;
                return n;
            }
        }
        public List<String> ScopeHierarchy
        {
            get { return _scopename; }
        }
        public String Type
        {
            get
            {
                if (_type == SignalType.register && (_regtype == RegType.real || _regtype == RegType.time))
                    return "float";
                else if ((_type == SignalType.register && _regtype == RegType.integer) || _size > 1)
                    return "vector";
                else
                    return "bit";
            }
        }
        public String Scope
        {
            get
            {
                switch (_scopetype[_scopetype.Count - 1])
                {
                    case ScopeType.begin:
                        return "begin";
                    case ScopeType.fork:
                        return "fork";
                    case ScopeType.function:
                        return "function";
                    case ScopeType.task:
                        return "task";
                    case ScopeType.module:
                        return "module";
                    default:
                        return "";
                }
            }
        }
        public String Name
        {
            get { return _name; }
        }

        public Variable(List<String> scopenames, List<ScopeType> scopetypes, String type, int size, String icode, String varname)
        {
            _scopename = new List<String>();
            foreach (String s in scopenames)
                _scopename.Add(s);
            _scopetype = new List<ScopeType>();
            foreach (ScopeType t in scopetypes)
                _scopetype.Add(t);
            switch (type)
            {
                case "event":
                    _type = SignalType._event;
                    break;
                case "integer":
                    _type = SignalType.register;
                    _regtype = RegType.integer;
                    break;
                case "parameter":
                    _type = SignalType.parameter;
                    break;
                case "real":
                    _type = SignalType.register;
                    _regtype = RegType.real;
                    break;
                case "reg":
                    _type = SignalType.register;
                    _regtype = RegType.reg;
                    break;
                case "supply0":
                    _type = SignalType.net;
                    _nettype = NetType.supply0;
                    break;
                case "supply1":
                    _type = SignalType.net;
                    _nettype = NetType.supply1;
                    break;
                case "time":
                    _type = SignalType.register;
                    _regtype = RegType.time;
                    break;
                case "tri":
                    _type = SignalType.net;
                    _nettype = NetType.tri;
                    break;
                case "triand":
                    _type = SignalType.net;
                    _nettype = NetType.triand;
                    break;
                case "trior":
                    _type = SignalType.net;
                    _nettype = NetType.trior;
                    break;
                case "trireg":
                    _type = SignalType.net;
                    _nettype = NetType.trireg;
                    break;
                case "tri0":
                    _type = SignalType.net;
                    _nettype = NetType.tri0;
                    break;
                case "tri1":
                    _type = SignalType.net;
                    _nettype = NetType.tri1;
                    break;
                case "wand":
                    _type = SignalType.net;
                    _nettype = NetType.wand;
                    break;
                case "wire":
                    _type = SignalType.net;
                    _nettype = NetType.wire;
                    break;
                case "wor":
                    _type = SignalType.net;
                    _nettype = NetType.wor;
                    break;
                default:
                    throw new Exception("Syntax error in $var construction, expected variable type!");
            }
            _size = size;
            _id = icode;
            _name = varname;
            _data = new List<object>();
            _time = new List<Time>();
        }
    
        public void AddValue(Time time, String value)
        {

            _time.Add(time);
            _data.Add(value);
        }

        public String GetAllValues()
        {
            String v = "";
            foreach (String s in _data)
                v += s + "; ";
            return v;
        }

        public XElement GetXTimeline()
        {
            List<XElement> timepoints = new List<XElement>();
            for (int i = 0; i < _time.Count; i++)
                timepoints.Add(new XElement("TimePoint", new XAttribute("Time", _time[i].Value), new XAttribute("Value", (String)_data[i])));
            return new XElement("Timeline", new XAttribute("Order", ""), new XAttribute("DataType", this.Type), new XAttribute("TimeScale", _time.Count>0?_time[0].TimeScaleString:""), timepoints.ToArray());
        }
        public bool IsInModule(String mname)
        {
            int i;
            for (i = _scopetype.Count - 1; i >= 0; i--)
                if (_scopetype[i] == ScopeType.module)
                    break;
            if (i >= 0 && _scopename[i] == mname)
                return true;
            else
                return false;
        }

        public List<Object> getValues()
        {
            for (int i = 0; i < _data.Count; i++)
            {
                _data[i] = _data[i].ToString().Replace("b","");
            }

            return this._data;

        }

        public List<Time> getTimes()
        {
            return this._time;

        }

        public String GetValuaAtTime(int time)
        {
            for(int i = 0; i< _time.Count; i++)
            {
                if (Convert.ToInt32(_time[i].Value) > time)
                {
                    return _data[i - 1].ToString();
                }
            }
            return _data[_data.Count-1].ToString();
        }

        public int getMaxTime()
        {
            int max = 0;
            foreach (Time t in _time)
                if (Convert.ToInt32(t.Value) > max)
                    max = Convert.ToInt32(t.Value);

            return max;
        }
    }
}
