using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Antlr.Runtime.Tree;

namespace VHDL2XML
{
    public class Entity_Component_Instance : ILocation
    {
        string _instanceName="";

        public string InstanceName
        {
            get { return _instanceName; }
            set { _instanceName = value.Trim(); }
        }

        bool _isComponentInstance = true;

        /// <summary>
        /// If true, the instance entity is first the component instance; xxx : [component] instance; 
        /// else is the entity instance  xxx: entity sdf(architecture)
        /// </summary>
        public bool IsComponentInstance
        {
            get { return _isComponentInstance; }
            set { _isComponentInstance = value; }
        }

        EntityDeclaration _EntityInstance;

        /// <summary>
        /// Reference to the entity, the instance is instantiated from.
        /// </summary>
        public EntityDeclaration EntityInstance
        {
            get { return _EntityInstance; }
            set { _EntityInstance = value; }
        }

        ComponentDeclaration _ComponentInstance;

        /// <summary>
        /// Reference to the component, the instance is instantiated from.
        /// </summary>
        public ComponentDeclaration ComponentInstance
        {
            get { return _ComponentInstance; }
            set { _ComponentInstance = value; }
        }

        string _ComponentName="";
        /// <summary>
        /// Name of the component, the instance is instantiated from.
        /// </summary>
        public string ComponentName
        {
            get { return _ComponentName; }
            set { _ComponentName = value.Trim(); }
        }

        //string _EntityPackageName="";

        //public string EntityPackageName
        //{
        //    get { return _EntityPackageName; }
        //    set { _EntityPackageName = value; }
        //}

        string _EntityLibraryName="";

        public string EntityLibraryName
        {
            get { return _EntityLibraryName; }
            set { _EntityLibraryName = value; }
        }
        
        string _EntityName="";
        /// <summary>
        /// Name of the entity, the instance is instantiated from.
        /// </summary>
        public string EntityName
        {
            get { return _EntityName; }
            set
            { 
                _EntityName = value.Trim();
                EntityLibraryName = VHDLToXML.ParseName(ref _EntityName);
            }
        }

        public string EntityFullName
        {
            get { return VHDLToXML.DotConcateName(EntityLibraryName,"", EntityName); }
        }

        public string ArchitectureFullName
        {
            get { return VHDLToXML.DotConcateName(ArchitectureLibraryName,"", ArchitectureName); }
        }

        ArchitectureDeclaration _architectureInstance;

        /// <summary>
        /// Reference to architecture of the component. If null, the arch. is behavioural, and use ArchitectureName, 
        /// </summary>
        public ArchitectureDeclaration ArchitectureInstance
        {
            get { return _architectureInstance; }
            set {
                _architectureInstance = value;
                _architectureName = _architectureInstance.ArchitectureName;
            }
        }

        //private string _ArchitecturePackageName = "";
        ///// <summary>
        ///// meno balika, v ktorom je architektura
        ///// </summary>
        //public string ArchitecturePackageName
        //{
        //    get { return _ArchitecturePackageName; }
        //    set { _ArchitecturePackageName = value.Trim(); }
        //}

        private string _ArchitectureLibraryName = "";
        /// <summary>
        /// meno balika, v ktorom je architektura
        /// </summary>
        public string ArchitectureLibraryName
        {
            get { return _ArchitectureLibraryName; }
            set { _ArchitectureLibraryName = value.Trim(); }
        }

        string _architectureName="";

        /// <summary>
        /// Architecture name, set automatically if Architecture is not null.
        /// </summary>
        public string ArchitectureName
        {
            get { return _architectureName; }
            set 
            {
                _architectureName = value.Trim();
                ArchitectureLibraryName = VHDLToXML.ParseName(ref _architectureName);
            }
        }

        ITree _VHDLPortMap = null;
        /// <summary>
        /// Node s VHDL kodom PortMap (); ku danej instancii
        /// </summary>
        public ITree VHDLPortMap
        {
            get { return _VHDLPortMap; }
            set { _VHDLPortMap = value; }
        }


        int _SugestedHeight = -1;

        public int SugestedHeight
        {
            get { return _SugestedHeight; }
            set { _SugestedHeight = value; }
        }

        #region ILocation Members

        private Rectangle _Position;

        public Rectangle Position
        {
            get { return _Position; }
            set { _Position = value; }
        }

        private EAnchor _Anchor;
        public EAnchor Anchor
        {
            get
            {
                return _Anchor;
            }
            set
            {
                _Anchor = value;
            }
        }

        private int _PositionIndex = -1;

        public int PositionIndex
        {
            get { return _PositionIndex; }
            set { _PositionIndex = value; }
        }

        private int _OrderIndex = -1;

        public int OrderIndex
        {
            get { return _OrderIndex; }
            set { _OrderIndex = value; }
        }

        private List<EntityPort> _Ports;
        /// <summary>
        /// Je kopiou s portov prislusnej instancie entity resp. komponentu.
        /// To preto, lebo kazda instancia ma vlastnu specificku graficku polohu portov, ako aj dalsie vlastnosti.
        /// Nemozno vztahovat polohu portov len na entitu, ale na jej instanciu.
        /// </summary>
        public List<EntityPort> Ports
        {
            get { return _Ports; }
            set { _Ports = value; }
        }

        #endregion

        public string GetEntityArchitectureName()
        {
            return this._EntityName + "(" + this._architectureName + ")";
        }
    }
}
