﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace VHDL2XML
{

    public enum EAnchor
    {
        No = 0,
        Left = 1,
        Right = 2,
        Top = 3,
        Bottom = 4
    }
    
    

    /// <summary>
    /// Rozhranie na objekty VHDL kodu (napr. EntityPort, Component, Entity, Instancia Entity.
    /// Rozhranie specifikuje graficku polohu objektov na obrazovke.
    /// </summary>
    public interface ILocation
    {
        Rectangle Position {get;set;}
        EAnchor Anchor{get;set;}
        /// <summary>
        /// Index = vertical level in positioning
        /// </summary>
        int PositionIndex { get; set; }
    }


    public static class VHDL2XMLExtensionMethods
    {
        /// <summary>
        /// Converts EAnchor to my specific string value.
        /// </summary>
        public static string ToMyString(this EAnchor anchor)
        {
            switch (anchor)
            {
                case EAnchor.No:
                    return "no";
                case EAnchor.Left:
                    return "left";
                case EAnchor.Right:
                    return "right";
                case EAnchor.Top:
                    return "top";
                case EAnchor.Bottom:
                    return "bottom";
                default:
                    return "";
            }
        }

        /// <summary>
        /// Converts EAnchor to my specific string value.
        /// </summary>
        public static string ToMyString(this portDirection direction)
        {
            switch (direction)
            {
                case portDirection.IN:
                    return "IN";
                case portDirection.OUT:
                    return "OUT";
                case portDirection.INOUT:
                    return "INOUT";
                case portDirection.BUFFER:
                    return "BUFFER";
                case portDirection.LINKAGE:
                    return "LINKAGE";
                case portDirection.GLOBAL:
                    return "GLOBAL";
                default:
                    return "IN";
            }
        }
    }   

}
