﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using AntlrVerilogParser;

namespace VerilogToXML
{
    public class Module
    {
        public Module(String name, int index, CommonTree tree)
        {
            Name = name;
            _treeIndex = index;
            _tree = tree;
            PortList = new List<Port>();
            ModuleInstanceList = new List<ModuleInstance>();
            NetList = new List<Net>();
            RegList = new List<Register>();
        }

        public Module()
        {
            PortList = new List<Port>();
            ModuleInstanceList = new List<ModuleInstance>();
            NetList = new List<Net>();
            RegList = new List<Register>();
        }

        public String Name;
            // meno modulu, v pripade konkretnej instancie modulu toto meno vystupuje ako typ

        public Port FindOrMakePort(String name)
            // najde port v portliste podla mena, alebo vytvori a vlozi novy port s danym menom
        {
            foreach (Port p in PortList)
            {
                if(p.GetName() == name)
                    return p;
            }
            Port q = new Port(this);
            q.SetName(name);
            PortList.Add(q);
            return q;
        }

        public Register MakeRegister(String name, RegType type)
            // vytvori a vlozi novy register s danym menom a typom do RegListu
        {
            Register r = new Register(this,type,name);
            RegList.Add(r);
            return r;
        }
        public Net MakeNet(String name, NetType type)
            // vytvori a vlozi novu net s danym menom a typom do NetListu
        {
            Net n = new Net(this, type, name);
            NetList.Add(n);
            return n;
        }


            // tieto parsovacie funkcie nerataju s moznostou, ze velkost size v [x:y] moze okrem x,y cisla aj urcena funkciou, alebo inym vyrazom
        public int ParsePortDeclaration(CommonTree tree, int index) // parsuje cast od input|output|inout po koniec  - ;|input|output|inout
            // index ukazuje na prvy token po deklaracii typu portov 
        {
            PortType t;
            Port p;
            int d=0,u=0;
            if (tree.GetChild(index).Type == VerilogParser.T__70)
                t = PortType.input;
            else if (tree.GetChild(index).Type == VerilogParser.T__71)
                t = PortType.output;
            else if (tree.GetChild(index).Type == VerilogParser.T__72)
                t = PortType.inout;
            else
                throw new Exception("Syntax error!\r\nUnexpected language construction. Expected 'input','output' or 'inout' keyword."); // -*-
            index++;
            if (tree.GetChild(index).Type == VerilogParser.LBRACK // hladam [x:y]
                && tree.GetChild(index + 1).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 2).Type == VerilogParser.COLON
                && tree.GetChild(index + 3).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 4).Type == VerilogParser.RBRACK)
            {
                d = System.Convert.ToInt32(tree.GetChild(index + 1).Text);
                u = System.Convert.ToInt32(tree.GetChild(index + 3).Text);
                index = index + 4;
            }
            else if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER) // aspon jeden port za input... musi byt
            {
                FindOrMakePort(tree.GetChild(index).Text).SetPortType(t);
            }
            else
                throw new Exception("Syntax error!\r\nMaybe unsupported language construction in Port 'input', 'output' or 'inout' declaration.\r\n(Using parameter, like \"input [par_IDENTIFIER,0] IDENTIFIER \" in port range is unsupported by this engine.)"); // -*-
            while (true)
            {
                index++;
                if (tree.GetChild(index).Type == VerilogParser.COMMA)
                    continue;
                else if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER)
                {
                    p = FindOrMakePort(tree.GetChild(index).Text);
                    p.SetPortType(t);
                    p.SetRange(d, u);
                }
                else
                    break;
            }
            return index;
        }
        public int ParseRegDeclaration(CommonTree tree, int index) // parsuje cast od reg|integer|real|time po koniec  - vsetko okrem identifier
            // vracia index na prvy token po deklaracii registra 
        {
            RegType t;
            Register r;
            int u=0, d=0, l=0, h=0;
            if (tree.GetChild(index).Type == VerilogParser.T__85)
                t = RegType.integer;
            else if (tree.GetChild(index).Type == VerilogParser.T__86)
                t = RegType.real;
            else if (tree.GetChild(index).Type == VerilogParser.T__101)
                t = RegType.reg;
            else if (tree.GetChild(index).Type == VerilogParser.T__102)
                t = RegType.time;
            else if (tree.GetChild(index).Type == VerilogParser.T__87)
                t = RegType.realtime;
            else
                throw new Exception("---- ParseRegDeclaration1 ---- Syntax error! ----"); // -*-
            index++;
            if (t == RegType.reg    // hladam reg [x:y] IDENTIFIER [add:ADD] - array
                && tree.GetChild(index).Type == VerilogParser.LBRACK 
                && tree.GetChild(index + 1).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 2).Type == VerilogParser.COLON
                && tree.GetChild(index + 3).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 4).Type == VerilogParser.RBRACK
                && (tree.GetChild(index + 5).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER)
                && tree.GetChild(index + 6).Type == VerilogParser.LBRACK
                && tree.GetChild(index + 7).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 8).Type == VerilogParser.COLON
                && tree.GetChild(index + 9).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 10).Type == VerilogParser.RBRACK)
            {
                d = System.Convert.ToInt32(tree.GetChild(index + 1).Text);
                u = System.Convert.ToInt32(tree.GetChild(index + 3).Text);
                l = System.Convert.ToInt32(tree.GetChild(index + 7).Text);
                h = System.Convert.ToInt32(tree.GetChild(index + 9).Text);
                r = MakeRegister(tree.GetChild(index + 5).Text,t);
                r.SetRange(d, u);
                r.SetArrayRange(l, h);
                index = index + 10;
            }
            else if (t == RegType.reg  // hladam reg [x:y] IDENTIFIER - memory type
                && tree.GetChild(index).Type == VerilogParser.LBRACK
                && tree.GetChild(index + 1).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 2).Type == VerilogParser.COLON
                && tree.GetChild(index + 3).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 4).Type == VerilogParser.RBRACK
                && (tree.GetChild(index + 5).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER))
            {
                d = System.Convert.ToInt32(tree.GetChild(index + 1).Text);
                u = System.Convert.ToInt32(tree.GetChild(index + 3).Text);
                r = MakeRegister(tree.GetChild(index + 5).Text,t);
                r.SetRange(d, u);
                index = index + 5;
            }
            else if ((tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER) // hladam reg|time|real|integer identifier [x:y]
                && tree.GetChild(index + 1).Type == VerilogParser.LBRACK
                && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 3).Type == VerilogParser.COLON
                && tree.GetChild(index + 4).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 5).Type == VerilogParser.RBRACK)
            {
                l = System.Convert.ToInt32(tree.GetChild(index + 2).Text);
                h = System.Convert.ToInt32(tree.GetChild(index + 4).Text);
                r = MakeRegister(tree.GetChild(index).Text,t);
                r.SetArrayRange(l, h);
                index = index + 5;
            }
            else if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER) // hladam reg|time|real|realtime|integer identifier
            {
                r = MakeRegister(tree.GetChild(index).Text,t);
            }
            else
                throw new Exception("Syntax error!\r\nMaybe unsupported language construction in Register 'reg' declaration.\r\n(Using parameter, like \"reg IDENTIFIER [par_IDENTIFIER,0]\" in register range is unsupported by this engine.)"); // -*-
            while (true)
            {
                index++;
                if (tree.GetChild(index).Type == VerilogParser.COMMA)
                    continue;
                else if ((tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER) // hladam identifier [x:y]
                    && tree.GetChild(index + 1).Type == VerilogParser.LBRACK
                    && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 3).Type == VerilogParser.COLON
                    && tree.GetChild(index + 4).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 5).Type == VerilogParser.RBRACK)
                {
                    l = System.Convert.ToInt32(tree.GetChild(index + 2).Text);
                    h = System.Convert.ToInt32(tree.GetChild(index + 4).Text);
                    r = MakeRegister(tree.GetChild(index).Text, t);
                    r.SetRange(d, u);
                    r.SetArrayRange(l, h);
                    index = index + 5;
                }
                else if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER)
                {
                    r = MakeRegister(tree.GetChild(index).Text, t);
                    r.SetRange(d, u);
                }
                else
                    return index;
            }
        }
        public int ParseNetDeclaration(CommonTree tree, int index)   // parsuje cast od net_type po koniec - ;
            // vracia index na prvy token po deklaracii net 
        {
            NetType t;
            Net n;
            int l=0,h=0,d=0,u=0;
            
            switch (tree.GetChild(index).Type)
            {
                case VerilogParser.T__89:
                    t=NetType.wire; break;
                case VerilogParser.T__90:
                    t=NetType.tri; break;
                case VerilogParser.T__91:
                    t=NetType.tri1; break;
                case VerilogParser.T__92:
                    t=NetType.supply0; break;
                case VerilogParser.T__93:
                    t=NetType.wand; break;
                case VerilogParser.T__94:
                    t=NetType.triand; break;
                case VerilogParser.T__95:
                    t=NetType.tri0; break;
                case VerilogParser.T__96:
                    t=NetType.supply1; break;
                case VerilogParser.T__97:
                    t=NetType.wor; break;
                case VerilogParser.T__98:
                    t=NetType.trior; break;
                case VerilogParser.KW_TRIREG:
                    t = NetType.trireg; break;
                default:
                    return index;
            }
            index++;
            
            if(tree.GetChild(index).Type == VerilogParser.LPAREN)    // spracuje strenght driver v (), ostava: [x:x]  #()
            {
                // TO DO
                // naparsovat strenght driver, ten pre vizualizaciu nie je podstatny
                while (tree.GetChild(index++).Type != VerilogParser.RPAREN) ;

            }
            if(tree.GetChild(index).Type == VerilogParser.LBRACK    // spracuje size v [x:x], ostava #()
                && tree.GetChild(index + 1).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 2).Type == VerilogParser.COLON
                && tree.GetChild(index + 3).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 4).Type == VerilogParser.RBRACK)
            {
                d = System.Convert.ToInt32(tree.GetChild(index + 1).Text);
                u = System.Convert.ToInt32(tree.GetChild(index + 3).Text);
                index = index + 5;
            } 
            if(tree.GetChild(index).Type == VerilogParser.POUND)   // spracuje delay #(), ostavaju samotne identifikatori, prip priradenia
            {
                if (tree.GetChild(index).Type == VerilogParser.LPAREN)
                {
                    //TO DO naparsovat oneskorenie
                    while (tree.GetChild(index++).Type != VerilogParser.RPAREN) ;
                }
                else
                {
                    // TO DO naparsovat oneskorenie bez ()
                    index++;
                }
            }
            index--; // kvoli nasledujucemu cyklu, aby som nevynechal jeden token na zaciatku
            String str;
            while (true)
            {
                index++;
                str = tree.GetChild(index).Type.ToString() + " " + tree.GetChild(index).Text + "\r\n"
                    + tree.GetChild(index + 1).Type.ToString() + " " + tree.GetChild(index + 1).Text + "\r\n"
                    + tree.GetChild(index + 2).Type.ToString() + " " + tree.GetChild(index + 2).Text + "\r\n"
                    + tree.GetChild(index + 3).Type.ToString() + " " + tree.GetChild(index + 3).Text + "\r\n"
                    + tree.GetChild(index + 4).Type.ToString() + " " + tree.GetChild(index + 4).Text + "\r\n"
                    + tree.GetChild(index + 5).Type.ToString() + " " + tree.GetChild(index + 5).Text;
                if (tree.GetChild(index).Type == VerilogParser.COMMA)
                    continue;
                else if ((tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER) // hladam identifier [x:y]
                    && tree.GetChild(index + 1).Type == VerilogParser.LBRACK
                    && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 3).Type == VerilogParser.COLON
                    && tree.GetChild(index + 4).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 5).Type == VerilogParser.RBRACK)
                {
                    l = System.Convert.ToInt32(tree.GetChild(index + 2).Text);
                    h = System.Convert.ToInt32(tree.GetChild(index + 4).Text);
                    n = MakeNet(tree.GetChild(index).Text, t);
                    n.SetRange(d, u);
                    n.SetArrayRange(l, h);
                    index = index + 5;
                }
                else if ((tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER)
                    && tree.GetChild(index + 1).Type == VerilogParser.ASSIGN)
                {
                    n = MakeNet(tree.GetChild(index).Text, t);
                    n.SetRange(d, u);
                    index++;
                    // TODO naparsovat priradenie, potrebne len pri simulacii
                    while (IsInNetList(tree.GetChild(++index).Text)
                        ||tree.GetChild(index).Type == VerilogParser.MINUS
                        ||tree.GetChild(index).Type == VerilogParser.PLUS
                        ||tree.GetChild(index).Type == VerilogParser.DIV
                        ||tree.GetChild(index).Type == VerilogParser.STAR
                        || tree.GetChild(index).Type == VerilogParser.QUESTION
                        || tree.GetChild(index).Type == VerilogParser.COLON
                        //||IsInFunctionList(tree.GetChild(++index).Text)
                        // TODO  unary operator
                        //      binary operator
                        );
                    // TODO - za priradenim sa moze vyskytnut mat. vyraz, co pri simulacii bude potrebne spravne naparsovat
                    index--;
                }
                else if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER)
                {
                    n = MakeNet(tree.GetChild(index).Text, t);
                    n.SetRange(d, u);
                }
                else
                    return index;
            }
        }
        public int ParsePortRegNet(CommonTree tree, int index) // vrateny index ukazuje na prvy token po 'endmodule'
            // Funkcia parsuje portlist modulu a nasledne cely modul a hlada deklaracie input, output a inout,
            // vysledkom su zname porty aj s ich typom, registre aj ich typy, siete net aj ich typy
        {
            index--;
            if(tree.GetChild(index).Type == VerilogParser.LPAREN)
                while (true)
                {
                    ++index;
                    if (tree.GetChild(index).Type == VerilogParser.RPAREN)
                        break;
                    switch (tree.GetChild(index).Type)
                    {
                        case VerilogParser.EOF:
                            throw new Exception("---- ParsePortList ---- Unexpected end of file! ----"); // -*-
                        case VerilogParser.T__69:
                            throw new Exception("---- ParsePortList ---- Unexpected end of module! ----"); // -*-
                        case VerilogParser.COMMA:
                            continue;
                        case VerilogParser.IDENTIFIER:
                        case VerilogParser.ESCAPED_IDENTIFIER:
                            Port port = new Port(this);
                            port.SetName(tree.GetChild(index).Text);
                            PortList.Add(port);
                            break;
                        case VerilogParser.T__70:
                        case VerilogParser.T__71: 
                        case VerilogParser.T__72:
                            index = ParsePortDeclaration(tree, index) - 1; // -1, aby som nespreskocil jeden token v portliste
                            break;
                        default:
                            throw new Exception("---- ParsePortList ---- Syntax error! ---- in '" + tree.GetChild(index).Text + "' ----" + this.Name); // -*-
                    }
                }
            while (true)
            {
                ++index;
                if (tree.GetChild(index).Type == VerilogParser.T__69) // T__69 - 'endmodule'
                    break;
                // begin, task, fork a function bloky ma nezaujimaju
                if (tree.GetChild(index).Type == VerilogParser.T__156) while (tree.GetChild(++index).Type != VerilogParser.T__157);
                if (tree.GetChild(index).Type == VerilogParser.T__158) while (tree.GetChild(++index).Type != VerilogParser.T__159);
                if (tree.GetChild(index).Type == VerilogParser.T__79) while (tree.GetChild(++index).Type != VerilogParser.T__80);
                if (tree.GetChild(index).Type == VerilogParser.T__81) while (tree.GetChild(++index).Type != VerilogParser.T__82);
                if (tree.GetChild(index).Type == VerilogParser.T__83) while (tree.GetChild(++index).Type != VerilogParser.T__84);

                switch (tree.GetChild(index).Type)
                {
                    case VerilogParser.EOF:
                        throw new Exception("---- ParsePortList ---- Unexpected end of file! ---- in '" + tree.GetChild(index).Text + "' ----"); // -*-
                    case VerilogParser.T__70:
                    case VerilogParser.T__71:
                    case VerilogParser.T__72: // ak najdem input || output || inout
                        index = ParsePortDeclaration(tree, index) - 1; // -1, aby som nespreskocil jeden token
                        break;
                    /* 
                     * 
                     * Vizualizacia registrov nie je implementovana, preto je parsovat registre zbytocne
                     * 
                     * 
                    case VerilogParser.T__85:   // T__85 - Integer
                    case VerilogParser.T__86: // T__86 - real
                    case VerilogParser.T__87: // T__86 - realtime
                    case VerilogParser.T__102:    // T__102 - time
                    case VerilogParser.T__101: // T__101 - reg
                        index = ParseRegDeclaration(tree, index) - 1; // -1, aby som nespreskocil jeden token
                        break;*/
                    case VerilogParser.T__89:   // T__89 - T__98
                    case VerilogParser.T__90:
                    case VerilogParser.T__91:
                    case VerilogParser.T__92:
                    case VerilogParser.T__93:
                    case VerilogParser.T__94:
                    case VerilogParser.T__95:
                    case VerilogParser.T__96:
                    case VerilogParser.T__97:
                    case VerilogParser.T__98:
                    case VerilogParser.KW_TRIREG:
                        index = ParseNetDeclaration(tree, index) - 1; // -1, aby som nespreskocil jeden token
                        break;
                    default:
                        continue;
                }
            }
            return index; 
        }

        public int ParseModuleInstance(CommonTree tree, int index, Module mod) // vrateny index ukazuje na prvy token po instanciacii ukoncenej ';'
            // Funkcia parsuje instanciacie inych modulov, a teda hlavne ich portlisty, z ktorych je jasne, ako su tieto instancie modulov pripojene do struktury tohto modulu
        {
            ModuleInstance instance;
            int order = 0;
            instance = new ModuleInstance(mod, this, tree.GetChild(index + 1).Text);
            instance.CreatePortInstanceList(mod.PortList);  // vytvorim instanciu modulu a v nej list instancii portov, ktory je vlastne kopiou portlistu z instancovaneho modulu 
            index=index+2;
            index = instance.ParseArrayRange(tree, index);      // naparsuje [x:y] za identifierom, teda velkost pola tychto instancii modulov
            try
            {
                if (tree.GetChild(index++).Type == VerilogParser.LPAREN)
                    while (tree.GetChild(index).Type != VerilogParser.RPAREN)
                        index = instance.ParsePortConnection(tree, index, order++);
                else
                    throw new Exception("---- ParseModuleInstance - Syntax Error! ----"); // -*-
            }
            catch (Exception ex)
            {
                throw ex;
            }
            index++;
            if(tree.GetChild(index).Type == VerilogParser.SEMI)
            {
                ModuleInstanceList.Add(instance);
                return ++index;
            }
            else
                throw new Exception("---- ParseModuleInstance - Syntax Error, missing SEMI! ----"); // -*-
        }
        
        public int ParsePrimitiveInstance(CommonTree tree, int index, Module mod) // vrateny index ukazuje na prvy token po instanciacii ukoncenej ';'
            // Funkcia parsuje instancie primitiv, a teda hlavne ich portlisty, z ktorych je jasne, ako su tieto instancie pripojene do struktury tohto modulu
        {
            ModuleInstance instance;
            int order = 0;
            if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER
                && tree.GetChild(index + 1).Type == VerilogParser.IDENTIFIER)
            {
                instance = new ModuleInstance(mod, this, tree.GetChild(index + 1).Text + " (UDP)");
                instance.CreatePortInstanceList(mod.PortList);  // vytvorim instanciu primitivy a v nej list instancii portov, ktory je vlastne kopiou portlistu z instancovanej primitivy
                index = index + 2;
            }
            else
            {
                instance = new ModuleInstance(mod, this, "UDP");
                instance.CreatePortInstanceList(mod.PortList);  // ako predosla instancia, ale tentokrat je bez mena, ktore pri primitivach nie je povinne 
                index = index + 1;
            }
            index = instance.ParseArrayRange(tree, index);      // naparsuje [x:y] za identifierom, teda velkost pola tychto instancii modulov
            try
            {
                if (tree.GetChild(index++).Type == VerilogParser.LPAREN)
                    while (tree.GetChild(index).Type != VerilogParser.RPAREN)
                        index = instance.ParsePortConnection(tree, index, order++);
                else
                    throw new Exception("---- ParseModuleInstance - Syntax Error! ----"); // -*-
            }
            catch (Exception ex)
            {
                throw ex;
            }
            index++;
            if (tree.GetChild(index).Type == VerilogParser.SEMI)
            {
                ModuleInstanceList.Add(instance);
                return ++index;
            }
            else
                throw new Exception("---- ParseModuleInstance - Syntax Error, missing SEMI! ----"); // -*-
        }
                
        public List<Port> PortList;
        public List<ModuleInstance> ModuleInstanceList;
        public List<Net> NetList;
        public List<Register> RegList;

        protected CommonTree _tree;
        protected int _treeIndex;

        public CommonTree ModuleTree
        {
            get { return _tree; }
            set { _tree = value; }
        }
        public int TreeIndex
        {
            get { return _treeIndex; }
            set { _treeIndex = value; }
        }

        private bool IsInNetList(String name) // prehlada NetList
        {
            foreach (Net n in NetList)
                if (n.Name == name)
                    return true;
            return false;
        }
    }
}
