﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using AntlrVerilogParser;
using System.Drawing;

namespace VerilogToXML
{
    public class ModuleInstance // zahrna aj instancie primitiv, teda je to tiez PrimitiveInstance
    {
        public ModuleInstance(Module mod,Module parent, String name)
        {
            Module = mod;
            Parent = parent;
            InstanceName = name;
            PortInstanceList = new List<PortInstance>();
            Position = new Rectangle(-1,-1,-1,-1);
        }

        public ModuleInstance()
        {
            PortInstanceList = new List<PortInstance>();
            Position = new Rectangle(-1, -1, -1, -1);
        }

        public Module Module; // alebo primitive
        public Rectangle Position;
        public String InstanceName;   // meno instancie modulu
        public Module Parent;  // v hierarchickej architekture predstavuje tento modul rodicovsky modul
 

        private int _low = 0;
        private int _high = 0;  // rozsah adries pola instancii

        public int ParseArrayRange(CommonTree tree, int index)
        {
            if (tree.GetChild(index).Type == VerilogParser.LBRACK    // parsovanie pola [x:y]
                && tree.GetChild(index + 1).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 2).Type == VerilogParser.COLON
                && tree.GetChild(index + 3).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 4).Type == VerilogParser.RBRACK)
            {
                this._low=System.Convert.ToInt32(tree.GetChild(index + 1).Text);
                this._high=System.Convert.ToInt32(tree.GetChild(index + 3).Text);
                index = index + 5;
            }
            if (tree.GetChild(index).Type == VerilogParser.LBRACK)
                while (tree.GetChild(index++).Type == VerilogParser.RBRACK) ;    //TODO parsing [expresion:expresion]
            return index;
        }
        public int GetArrayRange()
        {
            return _high - _low;
        }

        private int ParseNamedMapping(CommonTree tree, int index)
        {
            PortInstance portinstance;
            Net net;
            portinstance = FindPortInstance(tree.GetChild(index+1).Text);

            if (tree.GetChild(index + 3).Type == VerilogParser.LCURLY) // ak su k viac bitovemu portu priradene viacere signaly, tak su vymenovane v {} a v tejto casti if ich naparsujem
            {
                int order = 0;
                index += 4;
                while (tree.GetChild(index).Type != VerilogParser.RCURLY)
                    index = ParseMappingBitInVector(tree, index, portinstance, order++);
                return tree.GetChild(index + 2).Type == VerilogParser.COMMA ? index + 3 : index + 2;
            }

            net = FindNetInParent(tree.GetChild(index+3).Text,portinstance);
            PortConnection portconnection;
            if(portinstance == null || net == null)
                throw new Exception("---- ParseNamedMapping in ModuleInstance parsing - Syntax Error! - no port or net ----"); // -*-
            portconnection = new PortConnection(net,portinstance);
            if (portconnection.EqualRange)
            {
                if(tree.GetChild(index+5).Type == VerilogParser.COMMA)  // definicia priradenia portu konciaca ','
                {
                    for (int i = 0; i < portconnection.NetBitList.Capacity; i++)
                        portconnection.NetBitList[i] = portconnection.PortBitList[i] = true;
                    net.Connections.Add(portconnection);
                    return index+6;
                }
                if(tree.GetChild(index+5).Type == VerilogParser.RPAREN) // definicia priradenia portu konciaca ')'
                {
                    for (int i = 0; i < portconnection.NetBitList.Capacity; i++)
                        portconnection.NetBitList[i] = portconnection.PortBitList[i] = true;
                    net.Connections.Add(portconnection);
                    return index+5;
                }
                throw new Exception("---- ParseNamedMapping, named mapping in ModuleInstance parsing - Syntax Error! or bad range assigment ----"); // -*-
            }
            else
            {
                if (tree.GetChild(index + 4).Type == VerilogParser.LBRACK
                    && tree.GetChild(index + 5).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 6).Type == VerilogParser.RBRACK
                    && System.Convert.ToInt32(tree.GetChild(index + 5).Text) <= portconnection.NetBitList.Capacity)
                {
                    portconnection.NetBitList[System.Convert.ToInt32(tree.GetChild(index + 5).Text)] = true;
                    portconnection.PortBitList[0] = true;
                    net.Connections.Add(portconnection);
                    return tree.GetChild(index + 8).Type == VerilogParser.COMMA ? index + 9 : index + 8;
                }
                if (tree.GetChild(index + 4).Type == VerilogParser.LBRACK
                    && tree.GetChild(index + 5).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 6).Type == VerilogParser.COLON
                    && tree.GetChild(index + 7).Type == VerilogParser.NUMBER
                    && portconnection.NetBitList.Capacity >= System.Convert.ToInt32(tree.GetChild(index + 7).Text) - System.Convert.ToInt32(tree.GetChild(index + 5).Text))
                {
                    for (int i = 0; i < Math.Abs(System.Convert.ToInt32(tree.GetChild(index + 7).Text) - System.Convert.ToInt32(tree.GetChild(index + 5).Text)); i++)
                    {
                        portconnection.NetBitList[i] = true;
                        portconnection.PortBitList[i] = true;
                    }
                    net.Connections.Add(portconnection);
                    return tree.GetChild(index + 10).Type == VerilogParser.COMMA ? index + 11 : index + 10;
                }
                throw new Exception("---- ParseNamedMapping in ModuleInstance parsing - Syntax Error! - maybe unsupported [expresion] ----"); // -*-
            }
        }
            // naparsuje mapovanie portov vymenovanim, je spustana z funkcie ParsePortConnection
        private int ParseMappingBitInVector(CommonTree tree, int index, PortInstance portinstance, int order)
        {
            Net net = FindNetInParent(tree.GetChild(index).Text, portinstance);
            PortConnection portconnection = new PortConnection(net, portinstance);
            if(order >= portinstance.GetRange())
                throw new Exception("---- ParseMappingBitInVector in ModuleInstance parsing - Syntax Error! - maybe too many assignments in one moduleinstance port  ----"); // -*-
            if (net.Range == 1)
            {
                portconnection.NetBitList[0] = true;
                portconnection.PortBitList[order] = true;
                net.Connections.Add(portconnection);
                return tree.GetChild(index + 1).Type == VerilogParser.COMMA ? index + 2 : index + 1;
            }
            else if (tree.GetChild(index + 1).Type == VerilogParser.LBRACK
                && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                && tree.GetChild(index + 3).Type == VerilogParser.RBRACK)
            {
                portconnection.NetBitList[Convert.ToInt32(tree.GetChild(index + 2).Text)] = true;
                portconnection.PortBitList[order] = true;
                net.Connections.Add(portconnection);
                return tree.GetChild(index + 4).Type == VerilogParser.COMMA ? index + 5 : index + 4;
            }
            else
                throw new Exception("---- ParseMappingBitInVector in ModuleInstance parsing - Syntax Error! - maybe unsupported [expresion] ----"); // -*-
        }
        
        private int ParseImplicitMapping(CommonTree tree, int index, int order)
        {
            PortInstance portinstance;
            Net net;
            portinstance = FindPortInstance(order);

            if (tree.GetChild(index).Type == VerilogParser.LCURLY)  // ak su k viac bitovemu portu priradene viacere signaly, tak su vymenovane v {} a v tejto casti if ich naparsujem
            {
                int suborder = 0;
                index++;
                while (tree.GetChild(index).Type != VerilogParser.RCURLY)
                    index = ParseMappingBitInVector(tree, index, portinstance, suborder++);
                return tree.GetChild(index + 1).Type == VerilogParser.COMMA ? index + 2 : index + 1;
            }

            net = FindNetInParent(tree.GetChild(index).Text, portinstance);
            PortConnection portconnection;
            if(portinstance == null || net == null)
                throw new Exception("---- ParseImplicitMapping in ModuleInstance parsing - Syntax Error! - no port or net ----"); // -*-
            portconnection = new PortConnection(net,portinstance);
            if(portconnection.EqualRange)
            {
                //debug = tree.GetChild(index).Text;
                if(tree.GetChild(index+1).Type == VerilogParser.COMMA)  // definicia priradenia portu konciaca ','
                {
                    for (int i = 0; i < portconnection.NetBitList.Capacity; i++)
                        portconnection.NetBitList[i] = portconnection.PortBitList[i] = true;
                    net.Connections.Add(portconnection);
                    return index+2;
                }
                if(tree.GetChild(index+1).Type == VerilogParser.RPAREN) // definicia priradenia portu konciaca ')'
                {
                    for (int i = 0; i < portconnection.NetBitList.Capacity; i++)
                        portconnection.NetBitList[i] = portconnection.PortBitList[i] = true;
                    net.Connections.Add(portconnection);
                    return index+1;
                }
                throw new Exception("---- ParseImplicitMapping, named mapping in ModuleInstance parsing - Syntax Error! or bad range assigment ----"); // -*-
            }
            else
            {
                if (tree.GetChild(index + 1).Type == VerilogParser.LBRACK
                    && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 3).Type == VerilogParser.RBRACK
                    && System.Convert.ToInt32(tree.GetChild(index + 2).Text) < portconnection.NetBitList.Capacity)
                {
                    portconnection.NetBitList[System.Convert.ToInt32(tree.GetChild(index + 2).Text)]=true;
                    portconnection.PortBitList[0] = true;
                    net.Connections.Add(portconnection);
                    return tree.GetChild(index + 4).Type == VerilogParser.COMMA ? index + 5 : index + 4;
                }
                if (tree.GetChild(index + 1).Type == VerilogParser.LBRACK
                    && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 3).Type == VerilogParser.COLON
                    && tree.GetChild(index + 4).Type == VerilogParser.NUMBER
                    && tree.GetChild(index + 5).Type == VerilogParser.RBRACK
                    && portconnection.NetBitList.Capacity >= System.Convert.ToInt32(tree.GetChild(index + 4).Text) - System.Convert.ToInt32(tree.GetChild(index + 2).Text))
                {
                    for (int i = 0; i < Math.Abs((System.Convert.ToInt32(tree.GetChild(index + 4).Text) - System.Convert.ToInt32(tree.GetChild(index + 2).Text))); i++)
                    {
                        portconnection.NetBitList[i] = true;
                        portconnection.PortBitList[i] = true;
                    }
                    net.Connections.Add(portconnection);
                    return tree.GetChild(index + 6).Type == VerilogParser.COMMA ? index + 7 : index + 6;
                }
                if (tree.GetChild(index + 1).Type == VerilogParser.LBRACK)
                {
                    while (tree.GetChild(++index).Type != VerilogParser.RBRACK);
                    return tree.GetChild(++index).Type == VerilogParser.COMMA ? index + 1 : index;
                }
                throw new Exception("---- ParseImplicitMapping in ModuleInstance parsing - Syntax Error! - maybe unsupported [expresion] ----"); // -*-
            }
        }

        private int ParseConstant(CommonTree tree, int index, int order)
        {
            return tree.GetChild(index + 1).Type == VerilogParser.COMMA ? index + 2 : index + 1;
            //TODO: parsovanie konstanty priradenej k portu
        }

        public int ParsePortConnection(CommonTree tree, int index, int order)
            // vrati index na prvy token po ',', alebo po 'identifier', teda na prvy index po definicii pripojenia portu 
            // parsuje definiciu pripojenia portu instancii, najde prislusny port alebo net a vytvori v objekte net zaznam o tomto pripojeni
        {
            try
            {
                if (tree.GetChild(index).Type == VerilogParser.BNOT)
                    index++;
                if (tree.GetChild(index).Type == VerilogParser.DOT)
                    return ParseNamedMapping(tree, index);
                else if (tree.GetChild(index).Type == VerilogParser.IDENTIFIER || tree.GetChild(index).Type == VerilogParser.ESCAPED_IDENTIFIER)
                    return ParseImplicitMapping(tree, index, order);
                else if (tree.GetChild(index).Type == VerilogParser.NUMBER)
                    return ParseConstant(tree, index, order);
                else
                    throw new Exception("---- ParsePortConnection in ModuleInstance parsing - Syntax Error! ----"); // -*-
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public List<PortInstance> PortInstanceList;

        public void CreatePortInstanceList(List<Port> portlist)
        {
            PortInstance pi;
            foreach (Port p in portlist)
            {
                pi = new PortInstance(this);
                pi.SetName(p.GetName());
                pi.SetPortType(p.GetPortType());
                pi.SetRange(p.GetDown(), p.GetUp());
                PortInstanceList.Add(pi);
            }
        }
        public PortInstance FindPortInstance(String name)
        {
            foreach (PortInstance pi in PortInstanceList)
                if (pi.GetName() == name)
                    return pi;
            return null;
        }
        public PortInstance FindPortInstance(int order)
        {
            if(order < PortInstanceList.Count)
                return PortInstanceList[order];
            else
                return null;
        }

        public Net FindNetInParent(String name, PortInstance pi)
            // najde prislusnu siet, alebo sa vysporiada s implicitnym priradenim k portu rodica
        {
            Net net;
            if (pi == null)
                return null;
            foreach (Net n in Parent.NetList) // najde siet net s danym menom
                if (n.Name == name)
                    return n;
            foreach (Port p in Parent.PortList)    // ak sa nenajde net s tymto menom, znameno to, ze ide o implicitne priradenie k portu rodic. modulu,
                // v pripade, ak sa nenajde ani port s tymto menom, ide o syntakticku chybu
            {
                if (p.GetName() == name)
                {
                    net = new Net(Parent, NetType.wire, name);
                    net.SetRange(p.GetDown(), p.GetUp());
                    PortConnection pc = new PortConnection(net, p);
                    pc.EqualRange = true;
                    for (int i = 0; i < p.GetRange(); i++)
                        pc.NetBitList[i] = pc.PortBitList[i] = true;
                    net.Connections.Add(pc);
                    Parent.NetList.Add(net);
                    return net;
                }
            }
            net = new Net(Parent, NetType.wire, name);
            net.SetRange(pi.GetDown(), pi.GetUp());
            Parent.NetList.Add(net);
            return net;
        }
    }
}
