﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using AntlrVerilogParser;

namespace VerilogToXML
{
    public class Primitive : Module
    {
        public Primitive(String name, int index, CommonTree tree)
        {
            Name = name;
            _treeIndex = index;
            _tree = tree;
            PortList = new List<Port>();
            ModuleInstanceList = new List<ModuleInstance>();
            NetList = new List<Net>();
            RegList = new List<Register>();
        }
        public int ParsePort(CommonTree tree, int index) // vrateny index ukazuje na prvy token po 'endmodule'
        // Funkcia parsuje portlist modulu a nasledne cely modul a hlada deklaracie input, outpu a inout,
        // vysledkom su zname porty aj s ich typom, registre aj ich typy, siete net aj ich typy
        {
            index--;
            if (tree.GetChild(index).Type == VerilogParser.LPAREN)
                while (true)
                {
                    ++index;
                    if (tree.GetChild(index).Type == VerilogParser.RPAREN)
                        break;
                    switch (tree.GetChild(index).Type)
                    {
                        case VerilogParser.EOF:
                            throw new Exception("---- ParsePortList ---- Unexpected end of file! ----"); // -*-
                        case VerilogParser.T__74:
                            throw new Exception("---- ParsePortList ---- Unexpected end of primitive! ----"); // -*-
                        case VerilogParser.COMMA:
                            continue;
                        case VerilogParser.IDENTIFIER:
                        case VerilogParser.ESCAPED_IDENTIFIER:
                            Port port = FindOrMakePort(tree.GetChild(index).Text);
                            if (tree.GetChild(index+1).Type == VerilogParser.LBRACK // hladam [x:y]
                                && tree.GetChild(index + 2).Type == VerilogParser.NUMBER
                                && tree.GetChild(index + 3).Type == VerilogParser.COLON
                                && tree.GetChild(index + 4).Type == VerilogParser.NUMBER
                                && tree.GetChild(index + 5).Type == VerilogParser.RBRACK)
                            {
                                port.SetRange(System.Convert.ToInt32(tree.GetChild(index + 1).Text), System.Convert.ToInt32(tree.GetChild(index + 3).Text));
                                index = index + 5;
                            }
                            break;
                        case VerilogParser.T__70:
                        case VerilogParser.T__71:
                        case VerilogParser.T__72:
                            index = ParsePortDeclaration(tree, index) - 1; // -1, aby som nespreskocil jeden token v portliste
                            break;
                        default:
                            throw new Exception("---- ParsePortList ---- Syntax error! ---- in '" + tree.GetChild(index).Text + "' ----" + this.Name); // -*-
                    }
                }
            while (true)
            {
                ++index;
                if (tree.GetChild(index).Type == VerilogParser.T__74) // T__74 - 'endprimitive'
                    break;
                switch (tree.GetChild(index).Type)
                {
                    case VerilogParser.EOF:
                        throw new Exception("---- ParsePortList ---- Unexpected end of file! ---- in '" + tree.GetChild(index).Text + "' ----"); // -*-
                    case VerilogParser.T__70:
                    case VerilogParser.T__71:
                    case VerilogParser.T__72: // ak najdem input || output || inout
                        index = ParsePortDeclaration(tree, index) - 1; // -1, aby som nespreskocil jeden token
                        break;
                    default:
                        continue;
                }
            }
            return index;
        }


    }
}
