﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Linq;
using AntlrVerilogParser;
using Antlr;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using System.Diagnostics;
using System.IO;

namespace VerilogToXML
{
    public class Verilog2XML
    {
        public Verilog2XML(String filename)
        {
            _verilogcode = new VerilogCode();
            _verilogcode.AddVerilogFile(filename);
            _modules = new List<Module>();
            _primitives = new List<Primitive>();
        }
        public Verilog2XML()
        {
            _verilogcode = new VerilogCode();
            _modules = new List<Module>();
            _primitives = new List<Primitive>();
        }

        public event EventHandler ProgressChanged;

        private List<Module> _modules;
        private List<Primitive> _primitives;
        private VerilogCode _verilogcode;

        public void AddVerilogFile(String file)
        {
            _verilogcode.AddVerilogFile(file);
        }

        public void Parse()
        {
            try
            {
                _modules.Clear();
                _primitives.Clear();
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
                VerilogLexer lex = new VerilogLexer(new ANTLRStringStream(_verilogcode.GetVerilogCode()));
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
                CommonTokenStream tokens = new CommonTokenStream(lex);
                if (ProgressChanged != null)
                    ProgressChanged.Invoke(this, new EventArgs());
                VerilogParser parser = new VerilogParser(tokens);

                CommonTree tree = (CommonTree)parser.source_text().Tree;

                for (int i = 0; i < tree.Children.Count; i++)
                {
                    if (tree.GetChild(i).Type == VerilogParser.T__156) while (tree.GetChild(++i).Type != VerilogParser.T__157) ;
                    if (tree.GetChild(i).Type == VerilogParser.T__158) while (tree.GetChild(++i).Type != VerilogParser.T__159) ;
                    if (tree.GetChild(i).Type == VerilogParser.T__79) while (tree.GetChild(++i).Type != VerilogParser.T__80) ;
                    if (tree.GetChild(i).Type == VerilogParser.T__81) while (tree.GetChild(++i).Type != VerilogParser.T__82) ;
                    if (tree.GetChild(i).Type == VerilogParser.T__83) while (tree.GetChild(++i).Type != VerilogParser.T__84) ;
                    if (tree.GetChild(i).Type == 0)
                        throw new Exception("Error: Unsupported Verilog language construction!\r\n:" + tree.GetChild(i).ToString() + "\r\nText: " + tree.GetChild(i).Text);
                    if (tree.GetChild(i).Type == VerilogParser.T__67)  // T__67 - 'module'
                    {
                        AddModule(tree, tree.GetChild(i).ChildIndex);
                    }
                    if (tree.GetChild(i).Type == VerilogParser.T__73)  // T__73 - 'primitive'
                    {
                        AddPrimitive(tree, tree.GetChild(i).ChildIndex);
                    }
                    if (ProgressChanged != null)
                        ProgressChanged.Invoke(this, new EventArgs());
                }
                // prve parsovanie je za nami, nasleduje druhe,
                // pri tomto parsovani je uz znamy pocet modulov, ich porty, registre, siete nets
                // este je potrebne naparsovat instancie modulov a primitiv.
                foreach (Module module in _modules)
                    ParseModule(module);

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private int AddModule(CommonTree tree, int index) // Vytvori objekt modul a da mu meno, tree je strom
        // Predstavuje prve parsovanie, v ktorom sa naparsuju portlist, netlist, reglist, mozno neskor tu bude doimplementovane parsovanie assign ...
        {
            Module module;
            module = new Module(tree.GetChild(index + 1).Text, index, tree);
            try
            {
                module.ParsePortRegNet(tree, index + 3);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            //module.SetModuleTree(mtree);
            _modules.Add(module);
            return _modules.Count;

        }
        
        private int AddPrimitive(CommonTree tree, int index) // Vytvori objekt primitive a da mu meno, tree je strom
        // Predstavuje prve parsovanie, v ktorom sa naparsuju portlist, netlist, reglist, mozno neskor tu bude doimplementovane parsovanie assign ...
        {
            Primitive primitive;
            primitive = new Primitive(tree.GetChild(index + 1).Text, index, tree);
            try
            {
                primitive.ParsePort(tree, index + 3);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            //primitive.SetModuleTree(ptree);
            _primitives.Add(primitive);
            return _primitives.Count;

        }

        private Module GetModule(String name)    // prehlada moduly v _modules a vrati modul s danym menom
        // v pripade neuspechu vracia null
        {
            foreach (Module mod in _modules)
                if (name == mod.Name)
                    return mod;
            return null;
        }
        
        private Module GetPrimitive(String name)    // prehlada primitivy v _primitivess a vrati primitivu s danym menom
        // v pripade neuspechu vracia null
        {
            foreach (Primitive prim in _primitives)
                if (name == prim.Name)
                    return prim;
            return null;
        }

        private void ParseModule(Module module)
        // Predstavuje druhe parsovanie, v ktorom sa vytvaraju instancie modulov
        {
            CommonTree tree;
            int index;
            tree = module.ModuleTree;
            index = module.TreeIndex;

            if (tree.GetChild(index + 1).Text != module.Name)
                throw new Exception("---- Symstem error! ----"); //  -*-
            for (int i = index + 2; ((ITree)tree.Children[i]).Type !=
                VerilogParser.T__69 && i + 2 < tree.ChildCount; i++)
            {
                if (tree.GetChild(i).Type == VerilogParser.IDENTIFIER
                    && tree.GetChild(i + 1).Type == VerilogParser.IDENTIFIER
                    && tree.GetChild(i + 2).Type == VerilogParser.LPAREN
                    && GetModule(tree.GetChild(i).Text) != null // ak tento modul existuje, tak vrati jeho meno, inak vrati null
                    )
                {
                    module.ParseModuleInstance(tree, i, GetModule(tree.GetChild(i).Text));
                }
                if (tree.GetChild(i).Type == VerilogParser.IDENTIFIER
                    && GetPrimitive(tree.GetChild(i).Text) != null// ak tento modul existuje, tak vrati jeho meno, inak vrati null
                    )
                {
                    module.ParsePrimitiveInstance(tree, i, GetPrimitive(tree.GetChild(i).Text));
                }

                if (tree.GetChild(i).Type > 115 && tree.GetChild(i).Type < 142)
                {
                    PrimitiveInstance PI = new PrimitiveInstance(tree.GetChild(i + 1).Text,tree.GetChild(i).Text, module);
                    PI.ParsePortList(tree,i+2);
                    module.ModuleInstanceList.Add(PI);
                }

            }
        }

        public bool CheckSyntax(string file)
        {
            string path = Environment.CurrentDirectory.ToString().Substring(0, Environment.CurrentDirectory.ToString().Length - 9);
            string iverilog_path = System.IO.Path.Combine(path + "lib\\iverilog\\bin\\iverilog.exe");

            Process exeProcess = new Process();
            exeProcess.StartInfo.CreateNoWindow = true;
            exeProcess.StartInfo.UseShellExecute = false;
            exeProcess.StartInfo.FileName = "\"" + iverilog_path + "\"";;
            exeProcess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
            exeProcess.StartInfo.Arguments = "\"" + file + "\"";
            exeProcess.StartInfo.RedirectStandardError = true;

            exeProcess.Start();
            string str;

            str = exeProcess.StandardError.ReadToEnd();

            while ((str = exeProcess.StandardError.ReadLine()) != null)
            {
                if (str.Length > 0)
                {
                    return false;
                }
            }
            exeProcess.WaitForExit(5000);
            exeProcess.Close();

            return true;
        }

        public string VerilogToVCD(string verilog_file, string testbench_file, int time)
        {

            if (CheckSyntax(verilog_file) && CheckSyntax(testbench_file))
            {
                string error, path, iverilog_path, vvp_path, vvp_file, vcd_file;

                path = Environment.CurrentDirectory.ToString().Substring(0, Environment.CurrentDirectory.ToString().Length - 9);
                iverilog_path = path + "lib\\iverilog\\bin\\iverilog.exe";
                vvp_path = path + "lib\\iverilog\\bin\\vvp.exe";
                vvp_file = verilog_file.Substring(0, verilog_file.LastIndexOf('.')) + ".vvp";
                vcd_file = verilog_file.Substring(0, verilog_file.LastIndexOf('.')) + ".vcd";
                

                //zistenie nazvu vystupneho .vcd suboru
                StreamReader sr;
                int index, index2;
                string code;
                if (testbench_file == "")
                    sr = new StreamReader(verilog_file);
                else
                    sr = new StreamReader(testbench_file);
                code = sr.ReadToEnd();
                sr.Close();
                index = code.IndexOf("$dumpfile");
                if (index != -1)
                {
                    index = code.IndexOf('"', index);
                    index2 = code.IndexOf('"', index + 1);
                    vcd_file = Path.GetDirectoryName(vcd_file) + "\\" + code.Substring(index + 1, index2 - index - 1);
                }
                else
                    vcd_file = Path.GetDirectoryName(vcd_file) + "\\" + "dump.vcd";

                //pridat $dumpvars
                bool docasny = false;
                StreamWriter sw;
                if (testbench_file == "")
                {                 
                    sr = new StreamReader(verilog_file);
                    code = sr.ReadToEnd();
                    sr.Close();
                    index = code.IndexOf("$dumpvars");
                    if (index == -1)
                    {
                        index = code.IndexOf("initial begin");
                        index2 = code.IndexOf("begin", index + 1) + 5;
                        string s1 = code.Substring(0, index2);
                        string s2 = code.Substring(index2, code.Length - index2);
                        code = s1 + "\n   $dumpvars;\n" + s2;
                        docasny = true;
                        verilog_file = verilog_file.Substring(0, verilog_file.Length - 2) + "_doc.v";
                        sw = new StreamWriter(verilog_file);
                        sw.Write(code);
                        sw.Close();
                    }
                }
                else
                {
                    sr = new StreamReader(testbench_file);
                    code = sr.ReadToEnd();
                    sr.Close();
                    index = code.IndexOf("$dumpvars");
                    if (index == -1)
                    {
                        index = code.IndexOf("initial begin");
                        index2 = code.IndexOf("begin", index + 1) + 5;
                        string s1 = code.Substring(0, index2);
                        string s2 = code.Substring(index2, code.Length - index2);
                        code = s1 + "\n   $dumpvars;\n" + s2;
                        docasny = true;
                        testbench_file = testbench_file.Substring(0, testbench_file.Length - 2) + "_doc.v";
                        sw = new StreamWriter(testbench_file);
                        sw.Write(code);
                        sw.Close();
                    }
                }

                //pridat $finish
                if (testbench_file == "")
                {
                    sr = new StreamReader(verilog_file);
                    code = sr.ReadToEnd();
                    sr.Close();
                    index = code.IndexOf("$finish");
                    if (index == -1)
                    {
                        index = code.IndexOf("initial begin");
                        index2 = code.IndexOf("end", index + 1);
                        string s1 = code.Substring(0, index2);
                        string s2 = code.Substring(index2, code.Length - index2);
                        code = s1 + "\n   # " + time.ToString() + " $finish;\n" + s2;
                        if (!docasny)
                        {
                            docasny = true;
                            verilog_file = verilog_file.Substring(0, verilog_file.Length - 2) + "_doc.v";
                        }
                        sw = new StreamWriter(verilog_file);
                        sw.Write(code);
                        sw.Close();
                    }
                }
                else
                {
                    sr = new StreamReader(testbench_file);
                    code = sr.ReadToEnd();
                    sr.Close();
                    index = code.IndexOf("$finish");
                    if (index == -1)
                    {
                        index = code.IndexOf("initial begin");
                        index2 = code.IndexOf("end", index + 1);
                        string s1 = code.Substring(0, index2);
                        string s2 = code.Substring(index2, code.Length - index2);
                        code = s1 + "\n   # " + time.ToString() + " $finish;\n" + s2;
                        if (!docasny)
                        {
                            docasny = true;
                            testbench_file = testbench_file.Substring(0, testbench_file.Length - 2) + "_doc.v";
                        }
                        sw = new StreamWriter(testbench_file);
                        sw.Write(code);
                        sw.Close();
                    }
                }


                //kompilacia
                Process iverilogProcess = new Process();
                iverilogProcess.StartInfo.CreateNoWindow = true;
                iverilogProcess.StartInfo.UseShellExecute = false;
                iverilogProcess.StartInfo.FileName = "\"" + iverilog_path + "\"";
                iverilogProcess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
                iverilogProcess.StartInfo.RedirectStandardError = true;
                iverilogProcess.StartInfo.Arguments = "-o " + "\"" + vvp_file + "\"" + " " + "\"" + verilog_file + "\"" + " " + "\"" + testbench_file + "\"";

                iverilogProcess.Start();

                error = iverilogProcess.StandardError.ReadToEnd();
                iverilogProcess.WaitForExit(5000);

                //simulacia
                Process vvp = new Process();
                vvp.StartInfo.CreateNoWindow = true;
                vvp.StartInfo.UseShellExecute = false;
                vvp.StartInfo.FileName = "\"" + vvp_path + "\"";
                vvp.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
                vvp.StartInfo.RedirectStandardError = true;
                vvp.StartInfo.RedirectStandardOutput = true;
                vvp.StartInfo.Arguments = "-n " + "\"" + vvp_file + "\"";
                vvp.StartInfo.WorkingDirectory = verilog_file.Substring(0, verilog_file.LastIndexOf('\\'));

                vvp.Start();
                
                error = vvp.StandardError.ReadToEnd();
                vvp.WaitForExit(5000);
                
                //odstranenie docasneho .vvp suboru
                File.Delete(vvp_file);

                //ak bol vytvoreny docasny .v subor, tak ho odstranime
                if(docasny)
                    if(testbench_file == "")
                        File.Delete(verilog_file);
                    else
                        File.Delete(testbench_file);

                if (error.Length != 0)
                    return error;
                else
                    return vcd_file;
            }
            else
                return "Syntax error!\n";
        }

        public List<ModuleInstance> mi = new List<ModuleInstance>();
        List<string> mipath = new List<string>();

        public List<Module> getTopModules()
        {
            List<Module> mod = new List<Module>();
            bool top = true;

            foreach (Module m1 in _modules)
            {
                top = true;
                foreach (Module m2 in _modules)
                {
                    foreach (ModuleInstance mi in m2.ModuleInstanceList)
                    {
                        if (mi.Module == m1)
                        {
                            top = false;
                        }
                    }
                }
                if (top && m1.ModuleInstanceList.Count > 0)
                    mod.Add(m1);
            }

            return mod;
        }

        public XDocument GenerateXML(string filename)
        {
            List<XElement> mods = new List<XElement>();
            List<XElement> ports = new List<XElement>();
            List<XElement> signals = new List<XElement>();

            List<Module> TopModules = getTopModules();

            foreach (Module m in TopModules)
                mipath.Add(m.Name);

            foreach(Module m in TopModules)
                AddInstances(m, m.Name);

            int j = 0;
            foreach (Module m in TopModules)
            {
                if (m.NetList != null)
                {
                    foreach (Net net in m.NetList)
                    {
                        if (net.Connections.Count > 0)
                        {
                            List<XElement> PortMap = new List<XElement>();

                            List<int> signal = new List<int>();
                            List<List<int>> signallist = new List<List<int>>();
                            List<int> remove = new List<int>();

                            for (int k = 0; k < net.Range; k++)
                            {
                                signal = new List<int>();
                                signal.Add(k);
                                signallist.Add(signal);
                            }

                            for (int k = 1; k < net.Range; k++)
                            {
                                foreach (PortConnection pc in net.Connections)
                                {
                                    if (pc.NetBitList[k])
                                    {
                                        for (int l = 0; l < k; l++)
                                        {
                                            if (pc.NetBitList[l])
                                            {
                                                if (!signallist[l].Contains(k))
                                                {
                                                    signallist[l].Add(k);
                                                    remove.Add(k);
                                                }
                                                break;
                                            }
                                        }
                                    }
                                }
                            }

                            for (int r = remove.Count; r > 0; r--)
                            {
                                signallist.RemoveAt(remove[r - 1]);
                            }

                            string path = "";
                            List<string> added_paths = new List<string>();

                            for (int k = 0; k < signallist.Count; k++)
                            {
                                foreach (int s in signallist[k])
                                {
                                    foreach (PortConnection pc in net.Connections)
                                    {
                                        if (pc.NetBitList[s])
                                        {
                                            added_paths.Add(path);
                                            path = mipath[j];

                                            if (pc.GetModuleInstanceName() != "")
                                                path = path + "." + pc.GetModuleInstanceName();

                                            if (!added_paths.Contains(path))
                                            {
                                                PortMap.Add(new XElement("Map",
                                                new XElement("PortName", pc.Port.GetName()),
                                                new XElement("ModulePath", path)));
                                            }
                                        }
                                    }
                                }

                                signals.Add(new XElement("Signal",
                                new XElement("Name", net.Name),
                                new XElement("DataType", net.getType()),
                                new XElement("IsClock", "false"),
                                new XElement("PortMap", PortMap.ToArray()),
                                new XElement("Route")));
                                PortMap = new List<XElement>();
                                added_paths.Clear();
                                path = "";
                            }

                        }
                    }
                }

                foreach (Port p in m.PortList)
                {

                    ports.Add(new XElement("Port",
                            new XElement("Name", p.GetName()),
                            new XElement("Orientation", p.GetPortXVerilogMode()),
                            new XElement("DataType", p.GetPortXVerilogType()),
                            new XElement("Width", p.GetRange()),
                            new XElement("ModulePath", mipath[j]),
                            new XElement("PosX1"),
                            new XElement("PosY1"),
                            new XElement("PosX2"),
                            new XElement("PosY2"),
                            new XElement("Rotate")));
                }
                mods.Add(new XElement("Module",
                            new XElement("Name", m.Name),
                            new XElement("NameLang", m.Name),
                            new XElement("Type", m.Name),
                            new XElement("Path", mipath[j]),
                            new XElement("PosX1"),
                            new XElement("PosY1"),
                            new XElement("PosX2"),
                            new XElement("PosY2")));
                j++;
            }

            foreach (ModuleInstance i in mi)
            {
                if (i.Module.NetList != null)
                {
                    foreach (Net net in i.Module.NetList)
                    {
                        if (net.Connections.Count > 0)
                        {
                            List<XElement> PortMap = new List<XElement>();

                            List<int> signal = new List<int>();
                            List<List<int>> signallist = new List<List<int>>();
                            List<int> remove = new List<int>();
                            
                            for (int k = 0; k < net.Range; k++)
                            {
                                signal = new List<int>();
                                signal.Add(k);
                                signallist.Add(signal);
                            }

                            for (int k = 1; k < net.Range; k++)
                            {
                                foreach(PortConnection pc in net.Connections)
                                {
                                    if (pc.NetBitList[k])
                                    {
                                        for (int l = 0; l < k; l++)
                                        {
                                            if (pc.NetBitList[l])
                                            {
                                                if (signallist[l].Contains(k))
                                                {
                                                    signallist[l].Add(k);
                                                    remove.Add(k);
                                                }
                                                break;
                                            }
                                        }
                                    }
                                }
                            }

                            for(int r = remove.Count ; r>0 ; r--)
                            {
                                signallist.RemoveAt(remove[r - 1]);
                            }

                            string path = "";
                            List<string> added_paths = new List<string>();

                            for (int k = 0; k < signallist.Count; k++)
                            {
                                foreach (int s in signallist[k])
                                {
                                    foreach (PortConnection pc in net.Connections)
                                    {
                                        if (pc.NetBitList[s])
                                        {
                                            added_paths.Add(path);
                                            path = mipath[j];
                                            
                                            if (pc.GetModuleInstanceName() != "")
                                                path = path + "." + pc.GetModuleInstanceName();

                                            if (!added_paths.Contains(path))
                                            {
                                                PortMap.Add(new XElement("Map",
                                                new XElement("PortName", pc.Port.GetName()),
                                                new XElement("ModulePath", path)));
                                            }
                                        }
                                    }
                                }

                                signals.Add(new XElement("Signal",
                                new XElement("Name", net.Name),
                                new XElement("DataType", net.getType()),
                                new XElement("IsClock", "false"),
                                new XElement("PortMap", PortMap.ToArray()),
                                new XElement("Route")));
                                PortMap = new List<XElement>();
                                added_paths.Clear();
                                path = "";
                            }
                        }
                    }
                }
                
                foreach (PortInstance pi in i.PortInstanceList)
                {
                    ports.Add(new XElement("Port",
                            new XElement("Name", pi.GetName()),
                            new XElement("Orientation",pi.GetPortXVerilogMode()),
                            new XElement("DataType",pi.GetPortXVerilogType()),
                            new XElement("Width",pi.GetRange()),
                            new XElement("ModulePath", mipath[j]),
                            new XElement("PosX1"),
                            new XElement("PosY1"),
                            new XElement("PosX2"),
                            new XElement("PosY2"),
                            new XElement("Rotate")));
                }
                mods.Add(new XElement("Module",
                            new XElement("Name", i.InstanceName),
                            new XElement("NameLang", i.InstanceName),
                            new XElement("Type", i.Module.Name),
                            new XElement("Path", mipath[j]),
                            new XElement("PosX1"),
                            new XElement("PosY1"),
                            new XElement("PosX2"),
                            new XElement("PosY2")));
                j++;
            }

            return new XDocument(new XElement("Root", new XElement("Lang", "Verilog"), new XElement("Modules", mods.ToArray()), new XElement("Ports", ports.ToArray()), new XElement("Signals", signals.ToArray())));
        }

        private void AddInstances(Module m, string path)
        {
            string p;
            foreach (ModuleInstance i in m.ModuleInstanceList)
            {
                mi.Add(i);
                p = path + "." + i.InstanceName;
                mipath.Add(p);

                if (i.Module.ModuleInstanceList.Count > 0)
                    AddInstances(i.Module,p);
            }
        }

        public List<Module> getModules()
        {
            return _modules;
        }
    }
}
