﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using GraphicalObjects;
using System.Collections;
using HDLVisualizerDefinitions;

/* testing comment */
namespace HDL_XML_Parser
{
    public static class XMLParser
    {
        public static void ParseFile( string file, out List<Modules> ModuleList, out List<Ports> PortList, out List<Signals> SignalList, out string HardwareDescriptionLanguageName)
        {
            using( XmlTextReader xmlRead = new XmlTextReader(file) )
            {
                ModuleList = new List<Modules>();
                PortList = new List<Ports>();
                SignalList = new List<Signals>();
                HardwareDescriptionLanguageName = "Unknown HDL";
                
                Modules tempModule = null;
                Signals tempSignal = null;
                Ports tempPort = null;
                Stack elements = new Stack();
                string lastElement = "Root";
                string tempPortName = null;
                string tempModulePath = null;
                List<Ports> copyOfPortList = new List<Ports>(); // for signalls <Route> <\Route> section
                List<Ports> copyOfPortList2 = new List<Ports>(); // for signalls <PortMap> <\PortMap> section
                List<Ports> copyOfPortList3 = new List<Ports>(); // for maping ports to modules

                while (xmlRead.Read())
                {
                    if (xmlRead.IsEmptyElement) continue;

                    try
                    {
                    switch (xmlRead.NodeType)
                    {
                        /*******
                         * Node
                         ******/
                        case XmlNodeType.Element:
                            switch (xmlRead.Name)
                            {
                                case "Ports":
                                    //if (ModuleList.Count == 0) throw new ExceptionExpectedAndTranslated("Chyba formátu XML súboru! Definícia portov sa nachádza pred definíciou modulov, alebo sa v súbore nachádza definícia portov bez definície modulov!");
                                    if (ModuleList.Count == 0) throw new ExceptionExpectedAndTranslated(" XML file format error! Definition of ports occured without definition of modules, otherwise ports are defined, but modules not!");
                                    break;

                                case "Signals":
                                    //if( PortList.Count == 0) throw new ExceptionExpectedAndTranslated("Chyba formátu XML súboru! Definícia signálov sa nachádza pred definíciou portov, alebo sa v súbore nachádza definícia signálov bez definície portov!");
                                    if (PortList.Count == 0) throw new ExceptionExpectedAndTranslated(" XML file format error! Definition of signals occured without definition of ports, otherwise signals are defined, but ports not!");
                                    break;

                                case "Module":
                                    tempModule = new Modules();
                                    break;

                                case "Signal":
                                    tempSignal = new Signals();
                                    break;

                                case "Port":
                                    tempPort = new Ports();
                                    break;

                                case "Route":                                    
                                    //if (tempSignal.GetStartNode() != null) throw new ExceptionExpectedAndTranslated( "Nesprávny formát XML súboru! Viacnásobný výskyt elementu <Route> v jednom elemente <Signal>!");
                                    if (tempSignal.GetStartNode() != null) throw new ExceptionExpectedAndTranslated("Bad XML file format! Multiple occurence of <Route> element in one <Signal> element!");
                                    tempSignal.SetStartNode(parseSignalRoute(xmlRead, copyOfPortList));
                                    break;

                            }
                            if (xmlRead.NodeType == XmlNodeType.EndElement) break;
                            elements.Push(lastElement.ToString());
                            lastElement = xmlRead.Name;
                            break;

                        /***********
                         * Value
                         **********/
                        case XmlNodeType.Text:

                            switch (lastElement)
                            {
                                case "Lang":
                                    HardwareDescriptionLanguageName = xmlRead.Value;
                                    break;

                                /*
                                 * Module attributes, or common attributes with Signals and Ports
                                 */
                                case "Name":
                                    if (tempModule != null) tempModule.SetName(xmlRead.Value);
                                    else if (tempPort != null) tempPort.SetName(xmlRead.Value);
                                    else tempSignal.SetName(xmlRead.Value);
                                    break;

                                case "NameLang":
                                    tempModule.SetNameLang(xmlRead.Value);
                                    break;

                                case "Type":
                                    tempModule.SetModuleType(xmlRead.Value);
                                    break;

                                case "Path":
                                    tempModule.SetPath(xmlRead.Value);
                                    break;

                                case "PosX1":
                                    if (tempModule != null) tempModule.SetPosX1(int.Parse(xmlRead.Value));
                                    else if (tempPort != null) tempPort.SetPosX1(int.Parse(xmlRead.Value));
                                    break;

                                case "PosY1":
                                    if (tempModule != null) tempModule.SetPosY1(int.Parse(xmlRead.Value));
                                    else if (tempPort != null) tempPort.SetPosY1(int.Parse(xmlRead.Value));
                                    break;

                                case "PosX2":
                                    if (tempModule != null) tempModule.SetPosX2(int.Parse(xmlRead.Value));
                                    else if (tempPort != null) tempPort.SetPosX2(int.Parse(xmlRead.Value));
                                    break;

                                case "PosY2":
                                    if (tempModule != null) tempModule.SetPosY2(int.Parse(xmlRead.Value));
                                    else if (tempPort != null) tempPort.SetPosY2(int.Parse(xmlRead.Value));
                                    break;

                                /*
                                 * Port attributes or common attributes with signals
                                 */
                                case "Orientation":
                                    tempPort.SetOrientation(xmlRead.Value);
                                    break;

                                case "DataType":
                                    if (tempPort != null) tempPort.SetDataType(xmlRead.Value);
                                    else if( tempSignal != null ) tempSignal.SetDataType(xmlRead.Value);
                                    break;

                                case "Width":
                                    tempPort.SetWidth(int.Parse(xmlRead.Value));
                                    break;
                                
                                case "ModulePath":
                                    if(tempPort != null) tempPort.SetModulePath(xmlRead.Value);
                                    else if (tempSignal != null)
                                    {
                                        tempModulePath = xmlRead.Value;
                                        if (tempPortName != null)
                                        {
                                            for (int i = 0; i < copyOfPortList2.Count; i++)
                                            {
                                                if (copyOfPortList2[i].GetName() == tempPortName && copyOfPortList2[i].GetModulePath() == tempModulePath)
                                                {
                                                    tempSignal.AddPort(copyOfPortList2[i]);
                                                    //copyOfPortList2[i].SetSignal(tempSignal);
                                                    copyOfPortList2[i].AddSignal(tempSignal);
                                                    //copyOfPortList2.RemoveAt(i);
                                                    //break;
                                                }
                                            }
                                            tempPortName = null;
                                            tempModulePath = null;
                                        }
                                    }
                                    break;

                                case "Rotate":
                                    //if ((int.Parse(xmlRead.Value) > 4) || (int.Parse(xmlRead.Value) < 1)) throw new ExceptionExpectedAndTranslated( "Nesprávna hodnota atribútu Rotate na riadku " + xmlRead.LineNumber + " !" );
                                    //if ((int.Parse(xmlRead.Value) > 4) || (int.Parse(xmlRead.Value) < 1)) throw new ExceptionExpectedAndTranslated("Bad value of Rotate attribute at line " + xmlRead.LineNumber + "!");
                                    //tempPort.SetRotate( (Direction) int.Parse(xmlRead.Value)); // fixme
                                    //docasne riesenie 
                                    
                                    if (xmlRead.Value == "right")
                                        tempPort.SetRotate(Direction.Right);
                                    else if (xmlRead.Value == "left")
                                        tempPort.SetRotate(Direction.Left);
                                    else
                                        tempPort.SetRotate((Direction)int.Parse(xmlRead.Value));
                                    break;

                                /*
                                 * Signals attributes
                                 */ 

                                case "PortName":
                                        tempPortName = xmlRead.Value;
                                        if (tempModulePath != null)
                                        {
                                            for (int i = 0; i < copyOfPortList2.Count; i++)
                                            {
                                                if (copyOfPortList2[i].GetName() == tempPortName && copyOfPortList2[i].GetModulePath() == tempModulePath)
                                                {
                                                    tempSignal.AddPort(copyOfPortList2[i]);
                                                    //copyOfPortList2[i].SetSignal(tempSignal);
                                                    copyOfPortList2[i].AddSignal(tempSignal);
                                                    //copyOfPortList2.RemoveAt(i);
                                                    //break;
                                                }
                                            }
                                            tempPortName = null;
                                            tempModulePath = null;
                                        }
                                    break;

                                case "IsClock":
                                    try
                                    {
                                        tempSignal.SetIsClock(bool.Parse(xmlRead.Value));
                                    }
                                    catch ( System.FormatException)
                                    {
                                        //throw new ExceptionExpectedAndTranslated("Nesprávna hodnota atribútu IsClock na riadku " + xmlRead.LineNumber + " !");
                                        throw new ExceptionExpectedAndTranslated("Bad value of IsClock attribute at line " + xmlRead.LineNumber + "!");
                                    }
                                    break;
                          
                            }
                            break;

                        
                        /******************
                         * End Node
                         *****************/
                        case XmlNodeType.EndElement:
                            switch (xmlRead.Name)
                            {
                                case "Module":
                                    //if (tempModule == null) throw new ExceptionExpectedAndTranslated( "Neočakávaný výskyt <\\Module> na riadku " + xmlRead.LineNumber + " !");
                                    if (tempModule == null) throw new ExceptionExpectedAndTranslated("Unexpected occurence of <\\Module> at line " + xmlRead.LineNumber + "!");
                                    ModuleList.Add(tempModule);
                                    tempModule = null;
                                    break;

                                case "Signal":
                                    //if( tempSignal == null) throw new ExceptionExpectedAndTranslated( "Neočakávaný výskyt <\\Signal> na riadku " + xmlRead.LineNumber + " !");
                                    if (tempSignal == null) throw new ExceptionExpectedAndTranslated("Unexpected occurence of <\\Signal> at line " + xmlRead.LineNumber + "!");
                                    SignalList.Add(tempSignal);
                                    tempSignal = null;
                                    break;

                                case "Port":
                                    //if( tempPort == null) throw new ExceptionExpectedAndTranslated( "Neočakávaný výskyt <\\Port> na riadku " + xmlRead.LineNumber + " !");
                                    if (tempPort == null) throw new ExceptionExpectedAndTranslated("Unexpected occurence of <\\Port> at line " + xmlRead.LineNumber + "!");
                                    PortList.Add(tempPort);
                                    tempPort = null;
                                    break;

                                case "Map":
                                    tempModulePath = null;
                                    tempPortName = null;
                                    break;

                                case "Ports":
                                    for (int i = 0; i < PortList.Count; i++)
                                    {
                                        copyOfPortList.Add(PortList[i]);
                                        copyOfPortList2.Add(PortList[i]);
                                        copyOfPortList3.Add(PortList[i]);
                                    }


                                    /* Map modules to ports and ports to modules */
                                    for (int i = 0; i < ModuleList.Count; i++)
                                    {
                                        for (int j = 0; j < copyOfPortList3.Count; j++)
                                        {
                                            if (ModuleList[i].GetPath() == copyOfPortList3[j].GetModulePath())
                                            {
                                                ModuleList[i].AddPort(copyOfPortList3[j]);
                                                copyOfPortList3[j].SetModule(ModuleList[i]);
                                                copyOfPortList3.RemoveAt(j);
                                                j--;
                                            }
                                        }
                                    }
                                    break;
                            }
                            lastElement = (string)elements.Pop();
                            break;

                    }// end of switch

                    } // end of try
                    catch (System.NullReferenceException)
                    {
                        //throw new ExceptionExpectedAndTranslated("Chyba formátu XML súboru!");
                        throw new ExceptionExpectedAndTranslated("XML file format error!");
                    }

                } //end of while

            }// end of using

        }// end of ParseFile


        /**************************************************************************************************************************************************************************************
         * Recursive function for parsing XML node <Route> in <Signal> node, it receive and portList like a 2.nd argunet, which will be modified, so it must be a copy of PortList of all ports
         ****************************************************************************************************************************************************************************************/
        static Nodes parseSignalRoute( XmlTextReader xmlReader, List<Ports> portList)
        {
            Nodes startNode = null;
            Nodes tempNode = null;
            Nodes actualNode = null;
            int crossCount = 0;
            string lastElement = "";
            string tempModulePath = null;
            string tempPortName = null;

            while (xmlReader.Read() && !(xmlReader.NodeType == XmlNodeType.EndElement && (xmlReader.Name == "Route" || xmlReader.Name == "Cross")))
            {
                switch( xmlReader.NodeType )
                {
                    case XmlNodeType.Element:

                        switch(xmlReader.Name)
                        {
                            case "Cross":
                                //if (actualNode == null) throw new ExceptionExpectedAndTranslated("Nesprávny formát XML súboru, pravdepodobne sa vyskytol element <Cross> mimo element <Node>!");
                                if (actualNode == null) throw new ExceptionExpectedAndTranslated("Bad XML file format, probably occurence of <Cross> element outside of <Node> element!");
                                actualNode.AddNextNode(parseSignalRoute(xmlReader, portList));
                                crossCount++;
                                break;

                            case "Node":
                                //if (crossCount != 0) throw new ExceptionExpectedAndTranslated("Nesprávny formát XML súboru, pravdepodobne sa neukončený element <Cross>!");
                                if (crossCount != 0) throw new ExceptionExpectedAndTranslated("Bad XML file format, probably <Cross> element without end element <\\Cross>!");
                                if (startNode == null)
                                {
                                    actualNode = new Nodes();
                                    startNode = actualNode;
                                }
                                else
                                {
                                    tempNode = actualNode;
                                    actualNode = new Nodes();
                                    tempNode.AddNextNode(actualNode);
                                }                         
                                break;

                        }

                        lastElement = xmlReader.Name;
                        break;


                    case XmlNodeType.Text:

                        switch (lastElement)
                        {
                            case "PosX":
                                actualNode.SetPosX(int.Parse(xmlReader.Value));
                                break;

                            case "PosY":
                                actualNode.SetPosY(int.Parse(xmlReader.Value));
                                break;

                            case "Width":
                                actualNode.SetWidth(int.Parse(xmlReader.Value));
                                break;
                            
                            case "PortName":
                                tempPortName = xmlReader.Value;
                                if (tempModulePath != null)
                                {
                                    for (int i = 0; i < portList.Count; i++)
                                    {
                                        if( portList[i].GetModulePath() == tempModulePath && portList[i].GetName() == tempPortName)
                                        {
                                            //if (portList[i].GetSignal() == null) throw new ExceptionExpectedAndTranslated("Chyba XML súboru! Port ktorý je pripojený k signálu pravdepodobne nie je definovaný v PortListe tohto signálu!");
                                            //docasne riesenie
                                            if (portList[i].GetSignal(0) == null) throw new ExceptionExpectedAndTranslated("XML file error! A port, which is connected to signal, probably is not defined in the PortList of this signal!");
                                            actualNode.SetPort(portList[i]);
                                            portList.RemoveAt(i);
                                            break;
                                        }
                                    }
                                    tempPortName = null;
                                    tempModulePath = null;
                                }
                                break;

                            case "ModulePath":
                                tempModulePath = xmlReader.Value;
                                if (tempPortName != null)
                                {
                                    for (int i = 0; i < portList.Count; i++)
                                    {
                                        if( portList[i].GetModulePath() == tempModulePath && portList[i].GetName() == tempPortName)
                                        {
                                            //if (portList[i].GetSignal() == null) throw new ExceptionExpectedAndTranslated("Chyba XML súboru! Port ktorý je pripojený k signálu pravdepodobne nie je definovaný v PortListe tohto signálu!");
                                            //docasne riesenie
                                            if (portList[i].GetSignal(0) == null) throw new ExceptionExpectedAndTranslated("XML file error! A port, which is connected to signal, probably is not defined in the PortList of this signal!");
                                            actualNode.SetPort(portList[i]);
                                            portList.RemoveAt(i);
                                            break;
                                        }
                                    }
                                    tempPortName = null;
                                    tempModulePath = null;
                                }
                                break;

                        }

                        break;

                    case XmlNodeType.EndElement:
                        //if (xmlReader.Name == "Route" && crossCount < 2) throw new ExceptionExpectedAndTranslated("Chyba XML súboru, počet vetiev rozvetveného signálu je menší ako 2!");
                        if (xmlReader.Name == "Route" && crossCount < 2) throw new ExceptionExpectedAndTranslated("XML file error, number of branched signal's branches  is lower than 2!");
                        lastElement = "";
                        if (xmlReader.Name == "Map")
                        {
                            tempModulePath = null;
                            tempPortName = null;
                        }
                        break;
                }
            }

            return startNode;
        }

    }// end of class XMLParser
}//end of namespace
