﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace HDLVisualizer
{
    public partial class frmtestbench : Form
    {
        private string code;
        private VHDL2XML.VHDLToXML vhdlToxml = null;
        private List<Int32> SignalWidth = new List<Int32>();
        private VHDL2XML.EntityDeclaration SelectedEntity = null;
        private List<Event> events = new List<Event>();
        private VerilogToXML.Verilog2XML verilog2xml = null;
        private VerilogToXML.Module SelectedModule = null;
        private List<VerilogEvent> VerilogEvents = new List<VerilogEvent>();
        private Project project;
        public struct Event
        {
            public string signal;
            public List<string> values;
            public List<int> times;
        }

        public struct VerilogEvent
        {
            public string signal;
            public string value;
            public string time;
        }

        public frmtestbench(string cd, ref Project pr)
        {
            project = pr;
            InitializeComponent();
            code = cd;
            if (project.getType() == "verilog")
            {
                this.list.MultiColumn = true;
                this.list.ColumnWidth = 150;
            }
            switch (System.IO.Path.GetExtension(code))
            {
                    //veci pre VHDL ------------------------------------------
                case ".vhd":
                case ".vhdl":
                    detectedcode.Text = "Detected code: VHDL";

                    vhdlToxml = new VHDL2XML.VHDLToXML();
                    TextReader fs = new StreamReader(code);
                    vhdlToxml.AddDesignItem(code, main.WorkingLibrary, fs.ReadToEnd());
                    vhdlToxml.Parse();
                    if (vhdlToxml.Architectures.Count == 0)
                    {
                        MessageBox.Show("There are no architectures in the design described by structure.");
                        break;
                    }
                    //nacitanie entit
                    foreach (VHDL2XML.EntityDeclaration ent in vhdlToxml.Entities)
                    {
                        if (!ent.Name.Equals("testbench"))
                            entity.Items.Add(ent.Name);
                    }
                    SelectedEntity = vhdlToxml.Entities[0];
                    entity.SelectedItem = entity.Items[0];
                    break;

                    //veci pre Verilog ---------------------------------------
                case ".v":
                    detectedcode.Text = "Detected code: Verilog";

                    verilog2xml = new VerilogToXML.Verilog2XML();
                    verilog2xml.AddVerilogFile(code);
                    verilog2xml.Parse();
                    if (verilog2xml.getModules().Count == 0)
                    {
                        MessageBox.Show("There are no Modules in the design described by structure.");
                        break;
                    }
                    //nacitanie modulov
                    
                    foreach (VerilogToXML.Module mod in verilog2xml.getModules())
                        if(mod.PortList.Count > 0)
                            entity.Items.Add(mod.Name);
                    SelectedModule = verilog2xml.getModules()[0];
                    entity.SelectedItem = entity.Items[0];
                    break;

                    //veci pre SystemC ---------------------------------------
                case ".cpp":
                    detectedcode.Text = "Detected code: SystemC";
                    break;
                default:
                    detectedcode.Text = "Detected code: none";
                    break;
            }
        }

        private void TBtype1_CheckedChanged(object sender, EventArgs e)
        {
            signal1.Enabled = true;
            value1.Enabled = true;
            interval1.Enabled = true;
            list.Enabled = true;
            interval2.Enabled = false;
            label2.Enabled = true;
            label5.Enabled = true;
            label6.Enabled = true;
            label7.Enabled = true;
            label8.Enabled = true;
            add.Enabled = true;
            remove.Enabled = true;
            label3.Enabled = false;
            label10.Enabled = false;
            clock.Enabled = true;
            if (clock.Checked == true)
            {
                clock1.Enabled = true;
                clock2.Enabled = true;
                label9.Enabled = true;
                label11.Enabled = true;
                clocksignal.Enabled = true;
                label12.Enabled = true;
            }
            autoclock.Enabled = false;
            autoclocksignal.Enabled = false;
        }

        private void TBtype2_CheckedChanged(object sender, EventArgs e)
        {
            signal1.Enabled = false;
            value1.Enabled = false;
            interval1.Enabled = false;
            list.Enabled = false;
            interval2.Enabled = true;
            label2.Enabled = false;
            label5.Enabled = false;
            label6.Enabled = false;
            label7.Enabled = false;
            label8.Enabled = false;
            add.Enabled = false;
            remove.Enabled = false;
            label3.Enabled = true;
            label10.Enabled = true;
            clock.Enabled = false;
            clock1.Enabled = false;
            clock2.Enabled = false;
            label9.Enabled = false;
            label11.Enabled = false;
            autoclock.Enabled = true;
            label12.Enabled = false;
            clocksignal.Enabled = false;
            if(autoclock.Checked == true)
                autoclocksignal.Enabled = true;
        }

        private void add_Click(object sender, EventArgs e)
        {
            if (interval1.Text == "")
            {
                MessageBox.Show("Define the time for the event");
            }
            else
            {
                if (System.IO.Path.GetExtension(code) == ".vhdl" ||
                    System.IO.Path.GetExtension(code) == ".vhd")
                {
                    int fl = 0;
                    if (value1.Text.Length == value1.MaxLength)
                    {
                        foreach (Event even in events)
                            if (even.signal.Equals(signal1.SelectedItem.ToString()))
                            {
                                even.times.Add(System.Convert.ToInt32(interval1.Text));
                                even.values.Add(value1.Text);
                                fl = 1;
                                break;
                            }
                        if (fl == 0)
                        {
                            Event ev = new Event();
                            ev.signal = signal1.SelectedItem.ToString();
                            ev.times = new List<int>();
                            ev.times.Add(System.Convert.ToInt32(interval1.Text));
                            ev.values = new List<string>();
                            ev.values.Add(value1.Text);
                            events.Add(ev);
                        }
                        WriteEvents();
                        WriteToClock();
                    }
                    else
                        MessageBox.Show("Valid signal value length: " + value1.MaxLength);
                }
                else if (System.IO.Path.GetExtension(code) == ".v")
                {
                    if (value1.Text.Length == value1.MaxLength)
                    {
                        VerilogEvent ve = new VerilogEvent();
                        ve.signal = signal1.SelectedItem.ToString();
                        ve.time = interval1.Text;
                        ve.value = value1.Text;
                        VerilogEvents.Add(ve);
                        WriteEventsVerilog();
                        WriteToClockVerilog();
                    }
                    else
                        MessageBox.Show("Valid signal value length: " + value1.MaxLength);
                }
            }            
        }

        private void WriteToClock()
        {
            int flag = 0;
            clocksignal.Items.Clear();
            foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
            {
                if (port.Direction == VHDL2XML.portDirection.IN)
                {
                    foreach (Event ev in events)
                    {
                        if (port.Name.Equals(ev.signal))
                        {
                            flag = 1;
                            break;
                        }
                    }
                    if (flag == 0)
                        clocksignal.Items.Add(port.Name);
                    flag = 0;
                }
            }
            if(clocksignal.Items.Count > 0)
                clocksignal.SelectedItem = clocksignal.Items[0];
        }

        private void WriteToClockVerilog()
        {
            int flag = 0;
            clocksignal.Items.Clear();
            foreach (VerilogToXML.Port port in SelectedModule.PortList)
            {
                if (port.GetPortType() == VerilogToXML.PortType.input)
                {
                    foreach (VerilogEvent ev in VerilogEvents)
                    {
                        if (port.GetName().Equals(ev.signal))
                        {
                            flag = 1;
                            break;
                        }
                    }
                    if (flag == 0)
                        clocksignal.Items.Add(port.GetName());
                    flag = 0;
                }
            }
            if (clocksignal.Items.Count > 0)
                clocksignal.SelectedItem = clocksignal.Items[0];
        }

        private void WriteEvents()
        {
            list.Items.Clear();
            SortEvents();
            string record = "";
            foreach (Event ev in events)
            {
                record = ev.signal + " <= ";
                for (int i=0; i<ev.values.Count; i++)
                    record += "'" + ev.values[i] + "' after " + ev.times[i] + " ns,";
                record = record.Substring(0, record.Length - 1);
                record += ";";
                list.Items.Add(record);
            }
        }

        private void WriteEventsVerilog()
        {
            list.Items.Clear();
            string record = "";
            foreach (VerilogEvent ve in VerilogEvents)
            {
                if(ve.value.Length == 1)
                    record = "#" + ve.time + " " + ve.signal + " = " + ve.value + ";";
                else if(ve.value.Length > 1)
                    record = "#" + ve.time + " " + ve.signal + " = " + ve.value.Length + "'b" + ve.value + ";";
                list.Items.Add(record);
            }
        }

        private void SortEvents()
        {
            int i = 0, time;
            string value;
            foreach (Event ev in events)
            {
                while (i < ev.times.Count-1)
                {
                    if (ev.times[i] > ev.times[i + 1])
                    {
                        time = ev.times[i];
                        value = ev.values[i];
                        ev.times[i] = ev.times[i + 1];
                        ev.values[i] = ev.values[i + 1];
                        ev.times[i + 1] = time;
                        ev.values[i + 1] = value;
                        i = 0;
                    }
                    else
                        i++;
                }
            }
        }

        private void CreateEvents()
        {
            Random random = new Random();
            Event ev;
            int i, simtime, pom;
            string val;
            if (!interval2.Text.Equals(""))
            {
                simtime = System.Convert.ToInt32(interval2.Text);
                foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
                {
                    if (port.Direction == VHDL2XML.portDirection.IN)
                    {
                        if ((autoclock.Checked == true && !port.Name.Equals(autoclocksignal.SelectedItem.ToString())) ||
                            autoclock.Checked == false)
                        {
                            i = 0;
                            val = "";
                            ev = new Event();
                            ev.values = new List<string>();
                            ev.times = new List<int>();
                            ev.signal = port.Name;
                            while (i <= simtime)
                            {
                                for (int j = 0; j < port.pinCount; j++)
                                    val += random.Next(2).ToString();
                                ev.values.Add(val);
                                val = "";
                                pom = random.Next(simtime / 100, simtime / 10);
                                ev.times.Add(i);
                                i += pom;
                            }
                            events.Add(ev);
                        }
                    }
                }
            }
        }

        private void CreateEventsVerilog()
        {
            Random random = new Random();
            VerilogEvent ev;
            int time, simtime, value;
            VerilogToXML.Port port = null;
            if (!interval2.Text.Equals(""))
            {
                simtime = System.Convert.ToInt32(interval2.Text);
                int min = simtime / 100;
                int max = simtime / 10;
                if (min == 0)
                    min = 1;
                while (simtime > 0)
                {
                    port = null;
                    while (port == null || port.GetPortType() != VerilogToXML.PortType.input || (autoclock.Checked && port.GetName() == autoclocksignal.Text))
                    {
                        port = SelectedModule.PortList[random.Next(SelectedModule.PortList.Count - 1)];
                    }
                    ev = new VerilogEvent();
                    ev.signal = port.GetName();

                    time = random.Next(min, max);
                    if (time > simtime)
                        time = simtime - 1;
                    ev.time = time.ToString();

                    value = random.Next(2);
                    for(int i =  VerilogEvents.Count - 1; i >= 0 ;i--)
                    {
                        if (VerilogEvents[i].signal == ev.signal)
                        {
                            if (Convert.ToInt32(VerilogEvents[i].value) == 0)
                                value = 1;
                            else
                                value = 0;
                            break;
                        }
                    }

                    ev.value = value.ToString();

                    VerilogEvents.Add(ev);
                    simtime -= time;
                }
            }
        }

        private void remove_Click(object sender, EventArgs e)
        {
            string signal;
            int index;
            if (list.SelectedItem != null)
            {
                if (System.IO.Path.GetExtension(code) == ".vhdl" ||
                    System.IO.Path.GetExtension(code) == ".vhd")
                {
                    index = list.SelectedItem.ToString().IndexOf(" ");
                    signal = list.SelectedItem.ToString().Substring(0, index);
                    foreach (Event ev in events)
                        if (ev.signal.Equals(signal))
                        {
                            events.Remove(ev);
                            break;
                        }
                    WriteEvents();
                    WriteToClock();
                }
                else if (System.IO.Path.GetExtension(code) == ".v")
                {
                    VerilogEvents.RemoveAt(list.SelectedIndex);
                    WriteEventsVerilog();
                    WriteToClockVerilog();
                }
            }
        }

        private void entity_SelectedIndexChanged(object sender, EventArgs e)
        {
            signal1.Items.Clear();
            SignalWidth.Clear();
            clocksignal.Items.Clear();
            autoclocksignal.Items.Clear();

            switch (System.IO.Path.GetExtension(code))
            {
                    //veci pre VHDL ------------------------------------------
                case ".vhd":
                case ".vhdl":
                    if (vhdlToxml != null)
                    {
                        foreach (VHDL2XML.EntityDeclaration ent in vhdlToxml.Entities)
                            if (ent.Name == entity.SelectedItem.ToString())
                            {
                                SelectedEntity = ent;
                                break;
                            }
                        //iba pre vstupne porty
                        foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
                            if (port.Direction == VHDL2XML.portDirection.IN)
                            {
                                signal1.Items.Add(port.Name);
                                clocksignal.Items.Add(port.Name);
                                autoclocksignal.Items.Add(port.Name);
                                SignalWidth.Add(port.pinCount);
                            }
                        signal1.SelectedItem = signal1.Items[0];
                        clocksignal.SelectedItem = clocksignal.Items[0];
                        autoclocksignal.SelectedItem = autoclocksignal.Items[0];
                    }
                    break;
                    //veci pre Verilog ---------------------------------------
                case ".v":
                    if (verilog2xml != null)
                    {
                        foreach (VerilogToXML.Module mod in verilog2xml.getModules())
                            if (mod.Name == entity.SelectedItem.ToString())
                            {
                                SelectedModule = mod;
                                break;
                            }
                        //iba pre vstupne porty
                        foreach (VerilogToXML.Port port in SelectedModule.PortList)
                            if (port.GetPortType() == VerilogToXML.PortType.input)
                            {
                                signal1.Items.Add(port.GetName());
                                clocksignal.Items.Add(port.GetName());
                                autoclocksignal.Items.Add(port.GetName());
                                SignalWidth.Add(port.GetRange()); //GetRange pocet bitov portu?
                            }
                        signal1.SelectedItem = signal1.Items[0];
                        clocksignal.SelectedItem = clocksignal.Items[0];
                        autoclocksignal.SelectedItem = autoclocksignal.Items[0];
                    }
                    break;
                    //veci pre SystemC ---------------------------------------
                case ".cpp":
                    detectedcode.Text = "Detected code: SystemC";
                    break;
                default:
                    detectedcode.Text = "Detected code: none";
                    break;
            }
        }

        private void create_Click(object sender, EventArgs e)
        {
            if (TBtype1.Checked && clock.Checked && (clocksignal.Text == "" || clock1.Text == "" || clock2.Text == ""))
            {
                MessageBox.Show("Define a clock or deselect the \"Define clock signal\"!");
            }
            else if (TBtype2.Checked && autoclock.Checked && autoclocksignal.Text == "")
            {
                MessageBox.Show("Select a clock signal or deselect the \"Create clock signal\"!");
            }
            else if (TBtype2.Checked && interval2.Text == "")
            {
                MessageBox.Show("Define the simulation time!");
            }
            else
            {
                string msg = "";
                switch (System.IO.Path.GetExtension(code))
                {
                    //veci pre VHDL ------------------------------------------
                    case ".vhd":
                    case ".vhdl":
                        if (TBtype2.Checked == true)
                            CreateEvents();
                        if (events.Count > 0)
                        {
                            msg = CreateTestbenchVHDL();
                            this.Close();
                            if (msg.Equals("Error"))
                                MessageBox.Show("Error occured!");
                            else
                                MessageBox.Show("TestBech created succesfully!");

                            //add file to project
                            project.AddSourceFile(msg);
                        }
                        else
                            MessageBox.Show("No events defined!", "Error");
                        break;

                    //veci pre Verilog ---------------------------------------
                    case ".v":
                        if (TBtype2.Checked == true)
                            CreateEventsVerilog();
                        if (VerilogEvents.Count > 0)
                        {
                            msg = CreateTestbenchVerilog();
                            this.Close();
                            if (msg.Equals("Error"))
                                MessageBox.Show("Error occured!");
                            else
                                MessageBox.Show("TestBech created succesfully!");

                            //add file to project
                            project.AddSourceFile(msg);
                        }
                        else
                            MessageBox.Show("No events defined!", "Error");
                        CreateTestbenchVerilog();
                        break;

                    //veci pre SystemC ---------------------------------------
                    case ".cpp":
                        detectedcode.Text = "Detected code: SystemC";
                        break;
                    default:
                        detectedcode.Text = "Detected code: none";
                        break;
                }
            }
        }

        private string CreateTestbenchVHDL() //create file_tb.vhdl
        {
            List<string> lines = new List<string>();
            lines.Add("-- TestBench created by HDLVisualizer --");
            lines.Add("");
            lines.Add("entity testbench is");
            lines.Add("end testbench;");
            lines.Add("");
            lines.Add("architecture stimulus of testbench is");
            lines.Add("");
            lines.Add("component " + SelectedEntity.Name + " is");
            lines.Add("\tport (");
            //porty v komponente
            for (int i = 0; i < SelectedEntity.Ports.Count; i++)
            {
                if (i == SelectedEntity.Ports.Count - 1)
                    lines.Add("\t" + SelectedEntity.Ports[i].Name + ": " + SelectedEntity.Ports[i].Direction + " " + SelectedEntity.Ports[i].Type);
                else
                    lines.Add("\t" + SelectedEntity.Ports[i].Name + ": " + SelectedEntity.Ports[i].Direction + " " + SelectedEntity.Ports[i].Type + ";");
            }
            lines.Add("\t);");
            lines.Add("end component;");
            lines.Add("");
            int j = 1;
            //DUT pre kazdu architekturu -> deklaracia
            foreach (VHDL2XML.ArchitectureDeclaration arch in vhdlToxml.Architectures)
                if (arch.EntityReference == SelectedEntity)
                {
                    lines.Add("for DUT" + j.ToString() + ":" + SelectedEntity.Name + " use entity " + arch.EntityFullName + "(" + arch.ArchitectureName + ");");
                    j++;
                }
            lines.Add("");
            //vstupne signaly, resp signaly s direction == IN

            foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
                if (port.Direction == VHDL2XML.portDirection.IN)
                {
                    if (port.pinCount < 2)
                        lines.Add("signal " + port.Name + ": Bit;");
                    else
                        lines.Add("signal " + port.Name + ": Bit_vector(" + (port.pinCount-1).ToString() + " downto 0);");
                }
            //ostatne signaly, resp signaly s direction != IN
            foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
                if (port.Direction != VHDL2XML.portDirection.IN)
                    foreach (VHDL2XML.ArchitectureDeclaration arch in vhdlToxml.Architectures)
                        if (arch.EntityReference == SelectedEntity)
                        {
                            if (port.pinCount < 2)
                                lines.Add("signal " + port.Name + "_" + arch.ArchitectureName + ": Bit;");
                            else
                                lines.Add("signal " + port.Name + "_" + arch.ArchitectureName + ": Bit_vector(" + (port.pinCount - 1).ToString() + " downto 0);");
                        }
            lines.Add("");
            lines.Add("begin");
            //signal clock
            string one = "", zero = "", clockname = "";
            if (clock.Checked == true && TBtype1.Checked == true || 
                autoclock.Checked == true && TBtype2.Checked == true)
            {
                if (TBtype1.Checked == true && clock.Checked == true)
                {
                    one = clock1.Text;
                    zero = clock2.Text;
                    clockname = clocksignal.SelectedItem.ToString();
                }
                if (TBtype2.Checked == true && autoclock.Checked == true)
                {
                    Random random = new Random();
                    one = random.Next(10, 100).ToString();
                    zero = random.Next(10, 100).ToString();
                    clockname = autoclocksignal.SelectedItem.ToString();
                }
                lines.Add("");
                lines.Add("p:process(" + clockname + ")");
                lines.Add("begin");
                lines.Add("if " + clockname + "='0' then " + clockname + "<='1' after " + one + " ns;");
                lines.Add("elsif " + clockname + "='1' then " + clockname + "<='0' after " + zero + " ns;");
                lines.Add("end if;");
                lines.Add("end process;");
                lines.Add("");
            }
            j = 1;
            string pom = "";
            //DUT pre kazdu architekturu -> port map
            foreach (VHDL2XML.ArchitectureDeclaration arch in vhdlToxml.Architectures)
                if (arch.EntityReference == SelectedEntity)
                {
                    pom = "DUT" + j.ToString() + ":" + SelectedEntity.Name + " port map (";
                    foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
                        if (port.Direction == VHDL2XML.portDirection.IN)
                            pom += " " + port.Name + ",";
                    foreach (VHDL2XML.EntityPort port in SelectedEntity.Ports)
                        if (port.Direction != VHDL2XML.portDirection.IN)
                            pom += " " + port.Name + "_" + arch.ArchitectureName + ",";
                    pom = pom.Substring(0, pom.Length - 1);
                    pom += " );";
                    lines.Add(pom);
                    j++;
                }
            lines.Add("");
            //STIMULUS pre kazdy event
            j = 1;
            SortEvents();
            foreach (Event ev in events)
            {
                pom = "STIMULUS" + j.ToString() + ": " + ev.signal + " <=";
                for (int k = 0; k < ev.times.Count; k++)
                    pom += " '" + ev.values[k] + "' after " + ev.times[k] + " ns,";
                pom = pom.Substring(0, pom.Length - 1);
                pom += ";";
                lines.Add(pom);
                j++;
            }
            lines.Add("end stimulus;");
            string path = System.IO.Path.GetDirectoryName(code) + "\\" + System.IO.Path.GetFileNameWithoutExtension(code) + "_" + SelectedEntity.Name + "_tb.vhdl";
            System.IO.File.WriteAllLines(path, lines);
            if (System.IO.File.Exists(path))
                return path;
            else
                return "Error";
        }

        private string CreateTestbenchVerilog()
        {
            List<string> lines = new List<string>();
            lines.Add("// TestBench created by HDLVisualizer --");
            lines.Add("");
            lines.Add("module " + SelectedModule.Name + "_tb;");
            lines.Add("");

            foreach(VerilogToXML.Port port in SelectedModule.PortList)
            {
                if (port.GetPortType() == VerilogToXML.PortType.input)
                {
                    if (port.GetRange() == 1)
                        lines.Add("reg " + port.GetName() + ";");
                    else
                        lines.Add("reg [" + (port.GetRange()-1).ToString() + ":0] " + port.GetName() + ";");
                }
                else
                {
                    if (port.GetRange() == 1)
                        lines.Add("wire " + port.GetName() + ";");
                    else
                        lines.Add("wire [" + (port.GetRange() - 1).ToString() + ":0] " + port.GetName() + ";");
                }
            }
            lines.Add("");
            //add module with ports
            string line = "";
            foreach (VerilogToXML.Port port in SelectedModule.PortList)
            {
                if (line == "")
                    line += port.GetName();
                else
                    line += ", " + port.GetName();
            }
            lines.Add(SelectedModule.Name + " DUT(" + line + ");");


            //nastavit pociatocne hodnoty pre vstupy
            lines.Add("");
            lines.Add("initial begin");
            lines.Add("$dumpfile(\"" + SelectedModule.Name + "_tb.vcd\");");
            lines.Add("$dumpvars(0, " + SelectedModule.Name + "_tb);");
            
            foreach (VerilogToXML.Port port in SelectedModule.PortList)
            {
                if (port.GetPortType() == VerilogToXML.PortType.input)
                {
                    lines.Add(port.GetName() + " = 0;");
                }
            }

            //Zmena hodnot signalov
            SortEvents();
            line = "";
            foreach (VerilogEvent ev in VerilogEvents)
            {
                line = "#" + ev.time + " " + ev.signal + " = " + ev.value + ";";
                lines.Add(line);
            }

            lines.Add("end");
            lines.Add("");


            //signal clock
            string one = "", zero = "", clockname = "";
            if (clock.Checked == true && TBtype1.Checked == true ||
                autoclock.Checked == true && TBtype2.Checked == true)
            {
                if (TBtype1.Checked == true && clock.Checked == true)
                {
                    one = clock1.Text;
                    zero = clock2.Text;
                    clockname = clocksignal.SelectedItem.ToString();
                }
                if (TBtype2.Checked == true && autoclock.Checked == true)
                {
                    int min = Convert.ToInt32(interval2.Text)/30;
                    if (min == 0)
                        min = 1;
                    int max = Convert.ToInt32(interval2.Text)/15;
                    Random random = new Random();
                    one = random.Next(Convert.ToInt32(interval2.Text)/100, 100).ToString();
                    zero = random.Next(10, 100).ToString();
                    clockname = autoclocksignal.SelectedItem.ToString();
                }
                lines.Add("always begin");
                lines.Add("#" + one + " " + clockname + " = 1;");
                lines.Add("#" + zero + " " + clockname + " = 0;");
                lines.Add("end");

            }

            lines.Add("");
            lines.Add("initial");
            if (TBtype2.Checked)
                lines.Add("#" + interval2.Text + " $finish;");
            else
            {
                int finish = 1 ;
                foreach (VerilogEvent ev in VerilogEvents)
                    finish += Convert.ToInt32(ev.time);
                lines.Add("#" + finish + " $finish;");
            }

            lines.Add("");
            lines.Add("endmodule");
            
            string path = System.IO.Path.GetDirectoryName(code) + "\\" + SelectedModule.Name + "_tb.v";
            System.IO.File.WriteAllLines(path, lines);
            if (System.IO.File.Exists(path))
                return path;
            else
                return "Error";
        }

        private void clock_CheckedChanged(object sender, EventArgs e)
        {
            if (clock.Checked == true)
            {
                clocksignal.Enabled = true;
                label9.Enabled = true;
                label11.Enabled = true;
                clock1.Enabled = true;
                clock2.Enabled = true;
                label12.Enabled = true;
            }
            else
            {
                clocksignal.Enabled = false;
                label9.Enabled = false;
                label11.Enabled = false;
                clock1.Enabled = false;
                clock2.Enabled = false;
                label12.Enabled = false;
            }
        }

        private void autoclock_CheckedChanged(object sender, EventArgs e)
        {
            if (autoclock.Checked == true)
                autoclocksignal.Enabled = true;
            else
                autoclocksignal.Enabled = false;
        }

        private void signal1_SelectedIndexChanged(object sender, EventArgs e)
        {
            value1.Text = "";
            value1.MaxLength = SignalWidth[signal1.SelectedIndex];
            interval1.Text = "";
        }

        private bool nonNumberEntered = false;
        private void value1_KeyDown(object sender, KeyEventArgs e)
        {
            nonNumberEntered = false;
            if (e.KeyCode < Keys.D0 || e.KeyCode > Keys.D1)
            {
                if (e.KeyCode < Keys.NumPad0 || e.KeyCode > Keys.NumPad1)
                {
                    if (e.KeyCode != Keys.Back)
                    {
                        nonNumberEntered = true;
                    }
                }
            }
        }

        private void value1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (nonNumberEntered == true)
            {
                e.Handled = true;
            }
        }

        string old_value = "";

        private void TimeChanged(object sender, EventArgs e)
        {
            TextBox tb = (TextBox)sender;
            string text = tb.Text;
            int time;

            if (Int32.TryParse(tb.Text, out time) || tb.Text == "")
            {
                old_value = tb.Text;
            }
            else
                tb.Text = old_value;
        }
    }
}
