/*
 * sc_to_xml.cpp
 *
 *  Created on: 10.3.2012
 *      Author: Bc. Tom Janiga
 *      Company: STU FIIT Bratislava
 */

#include"sc_to_xml.h"
#include<typeinfo>
#include <cxxabi.h>
#include<fstream>
#include<string>
#include<vector>
#include "sysc/communication/sc_port.h"
#include "sysc/communication/sc_export.h"
//#include "sysc/communication/sc_signal.h"
#include "sysc/utils/sc_list.h"
#include "sysc/kernel/sc_object_manager.h"
#include "sysc/kernel/sc_module_registry.h"
#include "sysc/communication/sc_prim_channel.h"
#include "sysc/kernel/sc_module.h"
#include "sysc/kernel/sc_object.h"
#include "sysc/communication/sc_clock.h"


using namespace sc_core;

/* structure used for defined interfaces and their ports or exports */
struct interface_port_vector
{
	sc_interface* i_f;
	vector<sc_object*> port_vector;
};

/* structure used for (ex)port to (ex)port binding */
//struct port_port
//{
//	sc_object* port1;
//	sc_object* port2;
//};

char str[512];


char* lt_gt(const char * base)
{
	int i,j = 0;
	while(base[i] != '\0'){
		if(base[i] == '<') {
			str[j++] = '&';
			str[j++] = 'l';
			str[j++] = 't';
			str[j++] = ';';
		} else if(base[i] == '>') {
			str[j++] = '&';
			str[j++] = 'g';
			str[j++] = 't';
			str[j++] = ';';
		} else str[j++] = base[i];

		i++;
	}
	str[j] = '\0';
	return str;
}


namespace sc_core {

int sc_to_xml::simcontext_to_xml( sc_simcontext* context )
{
	if( !context ) return 1;
	
	sc_object* obj = NULL;
	int demangling_status = 0;
	char* tmp_str;
	if((obj = context->first_object()) == NULL) return 2;

	ofstream output_file("generated_xml.xhdl", ios_base::out);

	if(!output_file.is_open()) {
		return 3;
	}

	sc_plist<sc_object*> port_list;
	sc_plist<sc_prim_channel*> prim_channel_list;
	/*binging vectors*/
	vector<vector<sc_object*>*> p2p_binding;
	vector<interface_port_vector*> p2i_binding;

	/***************
	 * Print modules
	 ***************/
	output_file << "<Root>\n\t<Lang>SystemC</Lang>\n";
	output_file << "\t<Modules>\n";
	while( obj ) {
		if( dynamic_cast<sc_module*>(obj) )
		{
			output_file << "\t\t<Module>\n";

				output_file << "\t\t\t<Name>" 
							<< obj->basename()
							<< "</Name>\n";

				output_file << "\t\t\t<NameLang>"
							<< "</NameLang>\n";

				output_file << "\t\t\t<Type>";
				tmp_str = abi::__cxa_demangle(typeid(*obj).name(), 0, 0, &demangling_status);
				if( !demangling_status ) output_file << lt_gt(tmp_str);
				else output_file << lt_gt(typeid(*obj).name());
				output_file << "</Type>\n";

				output_file << "\t\t\t<Path>"
							<< obj->name()
							<< "</Path>\n";

				output_file << "\t\t\t<PosX1/>\n"
							<< "\t\t\t<PosY1/>\n"
							<< "\t\t\t<PosX2/>\n"
							<< "\t\t\t<PosY2/>\n";

			output_file << "\t\t</Module>\n";
		}
		/********************************************
		 * Store ports, exports and signals in lists
		 ********************************************/
		else if( dynamic_cast<sc_port_base*>(obj) ) 
		{
			//port_list.push_front(dynamic_cast<sc_port_base*>(obj));
			port_list.push_front(obj);
		}
		else if( dynamic_cast<sc_export_base*>(obj) )
		{
			//export_list.push_front(dynamic_cast<sc_export_base*>(obj));
			port_list.push_front(obj);
		}
		else if( dynamic_cast<sc_prim_channel*>(obj) )
		{
			prim_channel_list.push_front(dynamic_cast<sc_prim_channel*>(obj));
		}
		obj = context->next_object();
	}

	output_file << "\t</Modules>\n";

	/*************************
	 * Print ports and exports
	 *************************/
	output_file << "\t<Ports>\n";
	sc_port_base* temp_port_base = NULL;
	sc_export_base* temp_export_base = NULL;
	string tmp_module_path;
	size_t position;
    vector<sc_object*>* tmp_p2p;
	interface_port_vector* tmp_p2i;
	bool parent_bint;
	bool actual_bint;


	while( port_list.size()) {
		obj = port_list.pop_back();
		temp_port_base = dynamic_cast<sc_port_base*>(obj);
		temp_export_base = dynamic_cast<sc_export_base*>(obj);

		output_file << "\t\t<Port>\n";

			output_file << "\t\t\t<Name>"
						<< obj->basename()
						<< "</Name>\n";

			if( temp_port_base ){ /* Port binding, orientation and datatype */

				for(unsigned int i=0; (temp_port_base->m_bind_info && i<temp_port_base->m_bind_info->vec.size()); i++) {	/* walk actual port_bind_info */

					if( temp_port_base->m_bind_info->vec[i]->parent ) { /* if it is port to port binding */
						
						tmp_p2p = NULL;
						parent_bint = false;
						actual_bint = false;

						/* check actual stored bindings */
						for(unsigned int j=0; j<p2p_binding.size(); j++)	{
							for(unsigned int k=0; k < (p2p_binding[j]->size()); k++) {
								if(p2p_binding[j]->at(k) == temp_port_base) {
									actual_bint = true;
									tmp_p2p = p2p_binding[j];
								}
								else if(p2p_binding[j]->at(k) == (temp_port_base->m_bind_info->vec[i]->parent)) {
									parent_bint = true;								
									tmp_p2p = p2p_binding[j];
								}
							}
						} /* end of check actual stored bindings */

						if(actual_bint && parent_bint) output_file << "<!-- TRACING: undefined state, parent and port are bint again -->" << endl;
						else if(actual_bint) tmp_p2p->push_back(temp_port_base->m_bind_info->vec[i]->parent);
						else if(parent_bint) tmp_p2p->push_back(temp_port_base);
						else {
							tmp_p2p = new vector<sc_object*>;
							tmp_p2p->push_back(temp_port_base);
							tmp_p2p->push_back(temp_port_base->m_bind_info->vec[i]->parent);
							p2p_binding.push_back(tmp_p2p);
						}
					} /* end of port to port binding */

					if( temp_port_base->m_bind_info->vec[i]->iface ) { /* port to interface binding */
						tmp_p2i = NULL;
						for(unsigned int j=0; j < p2i_binding.size(); j++) {
							if( (p2i_binding[j]->i_f) == (temp_port_base->m_bind_info->vec[i]->iface) ) {
								tmp_p2i = p2i_binding[j];
								break;
							}
						}

						if(tmp_p2i) { /* if interface is already used for bind some ports together */
							/* test if this port is not in the vector ???? */
							tmp_p2i->port_vector.push_back(temp_port_base); 
						}
						else { /* if interface is not there */
							tmp_p2i = new interface_port_vector;
							tmp_p2i->i_f = temp_port_base->m_bind_info->vec[i]->iface;
							tmp_p2i->port_vector.push_back(temp_port_base);
							p2i_binding.push_back(tmp_p2i);
						}

					}/*end of port to interface binding */

				} /* end walk actual port_bind_info */

				output_file << "\t\t\t<Orientation>"
							<< temp_port_base->get_orientation()
							<< "</Orientation>\n";

				output_file << "\t\t\t<DataType>";
				tmp_str = abi::__cxa_demangle(temp_port_base->get_data_type(), 0, 0, &demangling_status);
				if( !demangling_status ) output_file << lt_gt(tmp_str);
				else output_file << lt_gt(temp_port_base->get_data_type());
				output_file << "</DataType>\n";

			} else if( temp_export_base ){ /* Export binding, orientation and datatype */

				if(temp_export_base->get_interface()) {
					tmp_p2i = NULL;
					/*interface binding */
					for(unsigned int i=0; i< p2i_binding.size(); i++) { /*check actual stored interface bindings */
						if(p2i_binding[i]->i_f == temp_export_base->get_interface()) {
							tmp_p2i = p2i_binding[i];
							break;
						}
					}

					if(tmp_p2i) { /* if interface is already there */
						/* try to check port vector ???? */
						tmp_p2i->port_vector.push_back(temp_export_base);
					}
					else { /* if interface is not stored in binding vector */
						tmp_p2i = new interface_port_vector;
						tmp_p2i->i_f = temp_export_base->get_interface();
						tmp_p2i->port_vector.push_back(temp_export_base);
						p2i_binding.push_back(tmp_p2i);
					}

					output_file << "\t\t\t<Orientation>"
								<< temp_export_base->get_interface()->get_orientation()
								<< "</Orientation>\n";

					output_file << "\t\t\t<DataType>";
					tmp_str = abi::__cxa_demangle(temp_export_base->get_interface()->get_data_type(), 0, 0, &demangling_status);
					if( !demangling_status ) output_file << lt_gt(tmp_str);
					else output_file <<  lt_gt(temp_export_base->get_interface()->get_data_type());
					output_file << "</DataType>\n";

				} else {
					output_file << "\t\t\t<Orientation/>\n"
								<< "\t\t\t<DataType/>\n";
				}
			}

			/* Module path */
			tmp_module_path = obj->name();
			if((position = tmp_module_path.rfind(obj->basename())) != string::npos) {
				tmp_module_path = tmp_module_path.substr(0, position - 1 );
			}

			output_file << "\t\t\t<ModulePath>"
						<< tmp_module_path
						<< "</ModulePath>\n";

		    output_file << "\t\t\t<PosX1/>\n"
						<< "\t\t\t<PosY1/>\n"
						<< "\t\t\t<PosX2/>\n"
						<< "\t\t\t<PosY2/>\n";

			output_file << "\t\t\t<Rotate/>\n";

		output_file << "\t\t</Port>\n";
	}
	output_file << "\t</Ports>\n";

	/*************************
	 * Print signals
	 *************************/
	output_file << "\t<Signals>\n";
	sc_interface *tmp_prim;
	sc_prim_channel *tmp_sc_prim_channel;

	while( prim_channel_list.size() ) { /* walk prim channel list */
		output_file << "\t\t<Signal>\n";

		tmp_prim = dynamic_cast<sc_interface*>(prim_channel_list.pop_back());
		if( !tmp_prim ) {
			output_file << "<!-- TRACE: error occur! can't to dynamic_cast prim channel to sc_interface -->\n";
			continue;
		}

		if( tmp_sc_prim_channel = dynamic_cast<sc_prim_channel*>(tmp_prim) ) {
			output_file << "\t\t\t<Name>";
			output_file << tmp_sc_prim_channel->basename();
			output_file << "</Name>\n";
		}
		else output_file << "\t\t\t<Name/>\n";

		if(tmp_prim) {
			output_file << "\t\t\t<DataType>";
			tmp_str = abi::__cxa_demangle(tmp_prim->get_data_type(), 0, 0, &demangling_status);
			if( !demangling_status ) output_file << lt_gt(tmp_str);
			else output_file << lt_gt(tmp_prim->get_data_type());
			output_file << "</DataType>\n";
		}
		else output_file << "<\t\t\t<DataType/>\n";


		output_file << "\t\t\t<IsClock>";
		if(dynamic_cast<sc_clock*>(tmp_prim)) output_file << "true";
		else output_file << "false";
		output_file << "</IsClock>\n";

		/*Port Mapping */
		output_file << "\t\t\t<PortMap>\n";
		tmp_p2i = NULL;
		for(unsigned int i=0; i<p2i_binding.size(); i++) { /* find actual prim_channel in binding vector */
			if( (p2i_binding.at(i)->i_f) == tmp_prim ) {
				tmp_p2i = p2i_binding.at(i);
				p2i_binding.erase(p2i_binding.begin()+i);
				/* print Maps */
				for(unsigned int j=0; j<tmp_p2i->port_vector.size(); j++) {
					output_file << "\t\t\t\t<Map>\n";
						output_file << "\t\t\t\t\t<PortName>"
									<< tmp_p2i->port_vector.at(j)->basename()
									<< "</PortName>\n";

						tmp_module_path = tmp_p2i->port_vector.at(j)->name();
						if((position = tmp_module_path.rfind(tmp_p2i->port_vector.at(j)->basename())) != string::npos) {
							tmp_module_path = tmp_module_path.substr(0, position - 1 );
						}

						output_file << "\t\t\t\t\t<ModulePath>"
									<< tmp_module_path
									<< "</ModulePath>\n";
					output_file << "\t\t\t\t</Map>\n";
				} /* end print Maps */

				delete tmp_p2i;
				tmp_p2i = NULL;
				break;
			}
		}

		output_file << "\t\t\t</PortMap>\n";

		output_file << "\t\t\t<Route/>\n\t\t</Signal>\n";
	} /*end walk prim channel list */

	//output_file << "TEST: " << p2p_binding.size() << endl;

	tmp_p2p = NULL;
	/* walk port to port binding vector */
	while( p2p_binding.size() ){
		output_file << "\t\t<Signal>\n";
		output_file << "\t\t\t<Name/>\n";
		output_file << "\t\t\t<DataType/>\n";
		output_file << "\t\t\t<IsClock/>\n";

		output_file << "\t\t\t<PortMap>\n";
		tmp_p2p = p2p_binding.at(0);
		p2p_binding.erase(p2p_binding.begin());

		/* walk binded ports together */
		for(unsigned int i=0; i<tmp_p2p->size(); i++) {
			output_file << "\t\t\t\t<Map>\n";
				output_file << "\t\t\t\t\t<PortName>"
							<< tmp_p2p->at(i)->basename()
							<< "</PortName>\n";
						
			tmp_module_path = tmp_p2p->at(i)->name();
			if((position = tmp_module_path.rfind(tmp_p2p->at(i)->basename())) != string::npos) {
				tmp_module_path = tmp_module_path.substr(0, position - 1 );
			}
			output_file << "\t\t\t\t\t<ModulePath>"
						<< tmp_module_path
						<< "</ModulePath>\n";
			output_file << "\t\t\t\t</Map>\n";
		}

		delete tmp_p2p;
		tmp_p2p = NULL;
		output_file << "\t\t\t</PortMap>\n\t\t\t<Route/>\n\t\t</Signal>\n";
	} /*end walk port to port binding vector */

	output_file << "\t</Signals>\n";

	//output_file << "TEST: " << prim_channel_list.size() << endl;

	output_file << "</Root>";
	output_file.close();
	return 0;
}

} //end namespace sc_core
