﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;



using Dataweb.NShape;
using Dataweb.Utilities;
using Dataweb.NShape.GeneralShapes;
using Dataweb.NShape.GeneralModelObjects;
using Dataweb.NShape.Advanced;
using Dataweb.NShape.WinFormsUI;
using Dataweb.NShape.Layouters;

using GraphicalObjects;

using HDL_XML_Parser;
using HDLVisualizerDefinitions;
using HDLVisualizer.Properties;
using HDLVisualizer.SystemCToXML;
using System.Diagnostics;

namespace HDLVisualizer
{
    public partial class main : Form
    {

        List<Project> Projects = new List<Project>();
        Project SelectedProject = null;
        TabPageProject SelectedTab;


        XDocument XMLProject;

        public static string WorkingLibrary = "Work";
        public static string SystemCLibraryDir = @"D:\skola\Teamovy projekt\trunk\lib\SystemC";
        public static string CygwinBash = @"D:\cygwin\bin\bash.exe";
        public static string SystemCXHDLFileName = "generated_xml.xhdl";
        public static int VisualisationHorizontalLimit = 0;
        public static int VisualisationVerticalLimit = 0;

        public main()
        {
            InitializeComponent(); 
        }

        #region new

        //create a new Project
        private void projectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewProjectDialog newproject = new NewProjectDialog();

            if (newproject.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Project p = new Project(newproject.name, newproject.type, "unsaved");
                Projects.Add(p);

                TabPageProject tab = new TabPageProject(newproject.name, p);
                Project_tabs.Controls.Add(tab);
                SelectedProject = Projects[Projects.Count - 1];
                Project_tabs.SelectedIndex = Project_tabs.TabCount - 1;
                SelectedTab = (TabPageProject)Project_tabs.SelectedTab;

                saveToolStripMenuItem.Enabled = true;
            }
        }

        //create a new source file
        private void vHDLSourceCodeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null)
            {
                MessageBox.Show("Create a project first!");
            }
            else
            {
                NewProjectDialog newproject = new NewProjectDialog(SelectedProject.getType());
                newproject.Text = "Create New File";
                newproject.label1.Text = "File name:";
                newproject.label2.Text = "File type:";

                if (newproject.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    if (SelectedProject.getType() != newproject.type)
                        MessageBox.Show("Select a " + SelectedProject.getType() + " project!");
                    else
                    {
                        switch (SelectedProject.getType())
                        {
                            case "vhdl":
                                SelectedProject.AddSourceFile(SelectedProject.getPath() + "\\" + newproject.name + ".vhdl");
                                break;
                            case "verilog":
                                SelectedProject.AddSourceFile(SelectedProject.getPath() + "\\" + newproject.name + ".v");
                                break;
                            case "systemc":
                                SelectedProject.AddSourceFile(SelectedProject.getPath() + "\\" + newproject.name + ".cpp");
                                break;
                        }
                        
                        SelectedTab.RefreshList();

                        checkSyntaxToolStripMenuItem.Enabled = true;
                        generateXHDLToolStripMenuItem.Enabled = true;
                        createTestbenchToolStripMenuItem.Enabled = true;
                        createVCDToolStripMenuItem.Enabled = true;
                    }
                }
            }
        }

        #endregion

        #region open

        //open a Project
        private void projectToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            OpenFileDialog open = new OpenFileDialog();
            open.Filter = "HDL Visualizer project files (*.phdl)|*.phdl";
            open.Title = "Select a HDL Visualizer project file";

            XmlTextReader xmlRead;
            string type = "";
            string xhdl = "";
            string vcd = "";
            List<string> sources = new List<string>();

            if (open.ShowDialog() == DialogResult.OK)
            {

                xmlRead = new XmlTextReader(open.FileName);
                string last = "";
                while (xmlRead.Read())
                {
                    if (xmlRead.NodeType == XmlNodeType.Element)
                    {
                        last = xmlRead.Name;
                    }
                    if (xmlRead.NodeType == XmlNodeType.Text)
                    {
                        if (last == "Type")
                            type = xmlRead.Value;
                        if (last == "Source_file")
                            sources.Add(xmlRead.Value);
                        if (last == "XHDL_file")
                            xhdl = xmlRead.Value;
                        if (last == "VCD_file")
                            vcd = xmlRead.Value;
                    }
                }

                Project p = new Project(Path.GetFileNameWithoutExtension(open.FileName), type, Path.GetDirectoryName(open.FileName));
                if (vcd != "")
                {
                    p.addVCD(p.getPath() + "\\" + vcd);
                    runGTKWaveToolStripMenuItem.Enabled = true;
                }
                if (xhdl != "")
                {
                    p.addXHDL(p.getPath() + "\\" + xhdl);
                    visualizeToolStripMenuItem.Enabled = true;
                }
                foreach (string s in sources)
                {
                    p.AddSourceFile(p.getPath() + "\\" + s);
                    checkSyntaxToolStripMenuItem.Enabled = true;
                    generateXHDLToolStripMenuItem.Enabled = true;
                    createTestbenchToolStripMenuItem.Enabled = true;
                    createVCDToolStripMenuItem.Enabled = true;
                }
                Projects.Add(p);
                Project_tabs.Controls.Add(new TabPageProject(Path.GetFileNameWithoutExtension(open.FileName),p));

                SelectedProject = p;
                Project_tabs.SelectedIndex = Project_tabs.TabCount - 1;
                SelectedTab = (TabPageProject)Project_tabs.SelectedTab;
            }

            saveToolStripMenuItem.Enabled = true;

        }

        //open a VHDL source file
        private void vHDLSourceFilevhdlhdlToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null || SelectedProject.getType() != "vhdl")
            {
                MessageBox.Show("Create or select a VHDL project!");
            }
            else
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Filter = "VHDL source files (*.vhd,*.vhdl)|*.vhd;*.vhdl";
                open.Title = "Select a VHDL source file";

                if (open.ShowDialog() == DialogResult.OK)
                {
                    OpenFile(open, "vhdl");
                }

                checkSyntaxToolStripMenuItem.Enabled = true;
                generateXHDLToolStripMenuItem.Enabled = true;
                createTestbenchToolStripMenuItem.Enabled = true;
                createVCDToolStripMenuItem.Enabled = true;
            }
        }

        //open a Verilog source file
        private void verilogSourceFilevToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null || SelectedProject.getType() != "verilog")
            {
                MessageBox.Show("Create or select a Verilog project!");
            }
            else
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Filter = "Verilog source files (*.v)|*.v";
                open.Title = "Select a Verilog source file";

                if (open.ShowDialog() == DialogResult.OK)
                {
                    OpenFile(open, "verilog");
                }

                checkSyntaxToolStripMenuItem.Enabled = true;
                generateXHDLToolStripMenuItem.Enabled = true;
                createTestbenchToolStripMenuItem.Enabled = true;
                createVCDToolStripMenuItem.Enabled = true;
            }

        }

        //open a SystemC source file
        private void systemCSourceFilecppToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null || SelectedProject.getType() != "systemc")
            {
                MessageBox.Show("Create or select a SystemC project!");
            }
            else
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Multiselect = true;
                open.Filter = "SystemC source files (*.cpp, *.h)|*.cpp;*.h";
                open.Title = "Select a SystemC source files";

                if (open.ShowDialog() == DialogResult.OK)
                {
                    if (SelectedProject.GetSystemCContext().GetFileNames() == null)
                    {
                        SelectedProject.GetSystemCContext().SetSystemC_Library(SystemCLibraryDir);
                        SelectedProject.GetSystemCContext().SetFileNames(open.FileNames);

                        string str = new string(open.FileNames[0].ToCharArray());
                        str = str.Remove(open.FileNames[0].LastIndexOf('\\'));
                        SelectedProject.GetSystemCContext().SetWorkingDir(str);
                    }
                    else SelectedProject.GetSystemCContext().AddFiles(open.FileNames);

                        foreach (string s in open.FileNames)
                        {
                            SelectedProject.AddSourceFile(s);
                            SelectedTab.RefreshList();
                        }

                }

                checkSyntaxToolStripMenuItem.Enabled = true;
                generateXHDLToolStripMenuItem.Enabled = true;
                createTestbenchToolStripMenuItem.Enabled = true;
                createVCDToolStripMenuItem.Enabled = true;
            }
        }

        //open a xhdl source file
        private void xHDLSourceFilexhdlToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null)
            {
                MessageBox.Show("Create or select a project!");
            }
            else
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Filter = "xHDL source files (*.xhdl)|*.xhdl";
                open.Title = "Select a xHDL source file";

                if (open.ShowDialog() == DialogResult.OK)
                {
                    OpenFile(open, "xhdl");
                }

                visualizeToolStripMenuItem.Enabled = true;
            }
        }

        //open a vcd source file
        private void vCDSourceFilevcdToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null)
            {
                MessageBox.Show("Create or select a project!");
            }
            else
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Filter = "VCD source files (*.vcd)|*.vcd";
                open.Title = "Select a VCD source file";

                if (open.ShowDialog() == DialogResult.OK)
                {
                    OpenFile(open, "vcd");
                }

                runGTKWaveToolStripMenuItem.Enabled = true;
                if (SelectedTab.groupBox1.Enabled == true)
                    visualizeSimulationToolStripMenuItem.Enabled = true;
            }
        }

        private void OpenFile(OpenFileDialog open, string type)
        {
            List<string> lines = new List<string>();
            StreamReader reader = new StreamReader(open.FileName);
            while (reader.Peek() >= 0)
                lines.Add(reader.ReadLine());

            //add file to project

            if (type == "verilog" || type == "vhdl" || type == "systemc")
                SelectedProject.AddSourceFile(open.FileName);
            else if (type == "xhdl")
                SelectedProject.addXHDL(open.FileName);
            else if (type == "vcd")
                SelectedProject.addVCD(open.FileName);

            SelectedTab.RefreshList();
        }

        #endregion

        #region save

        //save project
        private void saveProjectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog save = new SaveFileDialog();

            string path;

            if (SelectedProject.getPath() == "unsaved")
            {
                save.Filter = "HDL Visualizer project files (*.phdl)|*.phdl";
                save.Title = "Save a HDL Visualizer project file";
                save.FileName = SelectedProject.getName();

                if (save.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    SelectedProject.setPath(Path.GetDirectoryName(save.FileName) + "\\" + Path.GetFileNameWithoutExtension(save.FileName));
                    SelectedProject.setName(Path.GetFileNameWithoutExtension(save.FileName));
                    Directory.CreateDirectory(SelectedProject.getPath());
                    path = SelectedProject.getPath() + "\\" + SelectedProject.getName() + ".phdl";
                }
                else
                    return;
            }
            else
                path = SelectedProject.getPath() + "\\" + SelectedProject.getName() + ".phdl";

            saveFileToolStripMenuItem_Click(null, null);

            //save all files to project path
            foreach (File f in SelectedProject.getAllFiles())
            {
                if (Path.GetDirectoryName(f.getPath()) != Path.GetDirectoryName(path))
                {
                    StreamReader reader = new StreamReader(f.getPath());
                    string code = reader.ReadToEnd();
                    reader.Close();
                    f.setPath(Path.GetDirectoryName(path) + "\\" + f.getName());
                    StreamWriter writer = new StreamWriter(f.getPath());
                    writer.Write(code);
                    writer.Close();
                }
            }

            XDocument phdl;

            List<File> sourcefiles = SelectedProject.getSourceFiles();

            List<XElement> sf = new List<XElement>();

            foreach (File f in sourcefiles)
            {
                sf.Add(new XElement("Source_file", f.getName()));
            }

            string vcd = "";
            string xhdll = "";

            if (SelectedProject.getXHDL() != null)
                xhdll = SelectedProject.getXHDL().getName();

            if (SelectedProject.getVCD() != null)
                vcd = SelectedProject.getVCD().getName();

            phdl = new XDocument(
                new XElement(SelectedProject.getName(),
                new XElement("Type", SelectedProject.getType()),
                sf.ToArray(),
                new XElement("XHDL_file", xhdll),
                new XElement("VCD_file", vcd)));


            phdl.Save(path);

            MessageBox.Show("Project saved!");

        }

        //save project as
        private void saveProjectAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileToolStripMenuItem_Click(null, null);

            string path;

            SaveFileDialog save = new SaveFileDialog();
            save.Filter = "HDL Visualizer project files (*.phdl)|*.phdl";
            save.Title = "Save a HDL Visualizer project file";
            save.FileName = SelectedProject.getName();

            if (save.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                SelectedProject.setPath(Path.GetDirectoryName(save.FileName) + "\\" + Path.GetFileNameWithoutExtension(save.FileName));
                SelectedProject.setName(Path.GetFileNameWithoutExtension(save.FileName));
                Directory.CreateDirectory(SelectedProject.getPath());
                path = SelectedProject.getPath() + "\\" + SelectedProject.getName() + ".phdl";
            }
            else
                return;

            saveFileToolStripMenuItem_Click(null, null);

            //save all files to project path
            foreach (File f in SelectedProject.getAllFiles())
            {
                if (Path.GetDirectoryName(f.getPath()) != Path.GetDirectoryName(path))
                {
                    StreamReader reader = new StreamReader(f.getPath());
                    string code = reader.ReadToEnd();
                    reader.Close();
                    f.setPath(Path.GetDirectoryName(path) + "\\" + f.getName());
                    StreamWriter writer = new StreamWriter(f.getPath());
                    writer.Write(code);
                    writer.Close();
                }
            }

            XDocument phdl;

            List<File> sourcefiles = SelectedProject.getSourceFiles();

            List<XElement> sf = new List<XElement>();

            foreach (File f in sourcefiles)
            {
                sf.Add(new XElement("Source_file", f.getName()));
            }

            string vcd = "";
            string xhdll = "";

            if (SelectedProject.getXHDL() != null)
                xhdll = SelectedProject.getXHDL().getName();

            if (SelectedProject.getVCD() != null)
                vcd = SelectedProject.getVCD().getName();

            phdl = new XDocument(
                new XElement(SelectedProject.getName(),
                new XElement("Type", SelectedProject.getType()),
                sf.ToArray(),
                new XElement("XHDL_file", xhdll),
                new XElement("VCD_file", vcd)));
            
            phdl.Save(path);

            MessageBox.Show("Project saved!");
        }

        //save all projects
        private void saveAllProjectsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            int i = 0;
            foreach (Project p in Projects)
            {
                
                SaveFileDialog save = new SaveFileDialog();

                string path;

                if (p.getPath() == "unsaved")
                {
                    save.Filter = "HDL Visualizer project files (*.phdl)|*.phdl";
                    save.Title = "Save a HDL Visualizer project file";
                    save.FileName = p.getName();

                    if (save.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        p.setPath(Path.GetDirectoryName(save.FileName) + "\\" + Path.GetFileNameWithoutExtension(save.FileName));
                        p.setName(Path.GetFileNameWithoutExtension(save.FileName));
                        Directory.CreateDirectory(p.getPath());
                        path = p.getPath() + "\\" + p.getName() + ".phdl";
                    }
                    else
                        return;
                }
                else
                    path = p.getPath() + "\\" + p.getName() + ".phdl";


                //save displayed files
                string path2;
                TabPageProject tab = (TabPageProject)Project_tabs.TabPages[i];
                if (tab.SourceFiles.SelectedItem.ToString().LastIndexOf("unsaved") != -1)
                {
                    SaveFileDialog save2 = new SaveFileDialog();
                    if (tab.SourceFiles.SelectedItem.ToString().LastIndexOf(".vhdl") != -1)
                    {
                        save2.Filter = "VHDL source files (*.vhdl)|*.vhdl";
                        save2.Title = "Save a VHDL source file";
                    }
                    else if (tab.SourceFiles.SelectedItem.ToString().LastIndexOf(".v") != -1)
                    {
                        save2.Filter = "Verilog source files (*.v)|*.v";
                        save2.Title = "Save a Verilog source file";
                    }
                    else if (tab.SourceFiles.SelectedItem.ToString().LastIndexOf(".cpp") != -1)
                    {
                        save2.Filter = "SystemC source files (*.cpp)|*.cpp";
                        save2.Title = "Save a SystemC source file";
                    }
                    if (save2.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                        path2 = save2.FileName;
                    else
                        return;
                }
                else
                    path2 = p.getAllFiles()[tab.SourceFiles.SelectedIndex].getPath();

                StreamWriter writer2 = new StreamWriter(path2);
                string text = tab.code.Text;
                writer2.Write(text);
                writer2.Close();

                SelectedProject.getAllFiles()[tab.SourceFiles.SelectedIndex].setName(System.IO.Path.GetFileName(path2));
                SelectedProject.getAllFiles()[tab.SourceFiles.SelectedIndex].setPath(path2);

                //save all files to project path
                foreach (File f in p.getAllFiles())
                {
                    if (Path.GetDirectoryName(f.getPath()) != Path.GetDirectoryName(path))
                    {
                        StreamReader reader = new StreamReader(f.getPath());
                        string code = reader.ReadToEnd();
                        reader.Close();
                        f.setPath(Path.GetDirectoryName(path) + "\\" + f.getName());
                        StreamWriter writer = new StreamWriter(f.getPath());
                        writer.Write(code);
                        writer.Close();
                    }
                }

                XDocument phdl;

                List<File> sourcefiles = p.getSourceFiles();

                List<XElement> sf = new List<XElement>();

                foreach (File f in sourcefiles)
                {
                    sf.Add(new XElement("Source_file", f.getName()));
                }

                string vcd = "";
                string xhdll = "";

                if (p.getXHDL() != null)
                    xhdll = p.getXHDL().getName();

                if (p.getVCD() != null)
                    vcd = p.getVCD().getName();

                phdl = new XDocument(
                    new XElement(p.getName(),
                    new XElement("Type", p.getType()),
                    sf.ToArray(),
                    new XElement("XHDL_file", xhdll),
                    new XElement("VCD_file", vcd)));


                phdl.Save(path);

                MessageBox.Show("Project saved!");
                i++;
            }
        }

        //save file
        private void saveFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string Path;
            if (SelectedTab.SourceFiles.SelectedItem.ToString().LastIndexOf("unsaved") != -1)
            {
                SaveFileDialog save = new SaveFileDialog();
                if (SelectedTab.SourceFiles.SelectedItem.ToString().LastIndexOf(".vhdl") != -1)
                {
                    save.Filter = "VHDL source files (*.vhdl)|*.vhdl";
                    save.Title = "Save a VHDL source file";
                }
                else if (SelectedTab.SourceFiles.SelectedItem.ToString().LastIndexOf(".v") != -1)
                {
                    save.Filter = "Verilog source files (*.v)|*.v";
                    save.Title = "Save a Verilog source file";
                }
                else if (SelectedTab.SourceFiles.SelectedItem.ToString().LastIndexOf(".cpp") != -1)
                {
                    save.Filter = "SystemC source files (*.cpp)|*.cpp";
                    save.Title = "Save a SystemC source file";
                }

                if (save.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    Path = save.FileName;
                else
                    return;
            }
            else
                Path = SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath();

            StreamWriter writer = new StreamWriter(Path);
            string text = SelectedTab.code.Text;
            writer.Write(text);
            writer.Close();

            SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].setName(System.IO.Path.GetFileName(Path));
            SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].setPath(Path);
        }

        //save file as
        private void saveFileAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string Path;
            SaveFileDialog save = new SaveFileDialog();
            save.Filter = "VHDL source files (*.vhdl,*.vhd)|*.vhdl|Verilog source files (*.v)|*.v|SystemC source files (*.cpp)|*.cpp|Text files (*.txt)|*.txt|Any files (*.*)|*.*";
            save.Title = "Save a source file as";

            if (save.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                Path = save.FileName;
            else
                return;

            Path = save.FileName;

            StreamWriter writer = new StreamWriter(Path);
            string text = SelectedTab.code.Text;
            writer.Write(text);
            writer.Close();

            SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].setName(System.IO.Path.GetFileName(Path));
            SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].setPath(Path);

        }

        #endregion

        #region exit

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("All unsaved changes will be lost! Do you want to continue?", "Exit HDL Visualizer", MessageBoxButtons.YesNo);

            if (result == System.Windows.Forms.DialogResult.Yes)
            {
                try
                {
                    Directory.Delete("unsaved", true);
                }
                catch
                {
                }
                Close();
            }
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            DialogResult result = MessageBox.Show("All unsaved changes will be lost! Do you want to continue?", "Exit HDL Visualizer", MessageBoxButtons.YesNo);

            if (result == System.Windows.Forms.DialogResult.Yes)
            {
                try
                {
                    Directory.Delete("unsaved", true);
                }
                catch
                {
                }
            }
            else
                e.Cancel = true;
        }

        #endregion

        #region visualization

        //check syntax
        private void checkSyntaxToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string output, dir_path;
            dir_path = Environment.CurrentDirectory.ToString().Substring(0, Environment.CurrentDirectory.ToString().Length - 9);

            //check syntax for VHDL file
            if (SelectedProject == null)
            {
                MessageBox.Show("Open a project first!");
            }
            else if (SelectedTab.SourceFiles.SelectedItem.ToString().IndexOf(".v") == -1 && SelectedTab.SourceFiles.SelectedItem.ToString().IndexOf(".vcd") != -1)
            {
                MessageBox.Show("Select a VHDL or Verilog source file!");
            }
            else if (SelectedProject.getType() == "vhdl")
            {
                string ghdlexe = dir_path + "lib\\ghdl\\bin\\ghdl.exe";
                VHDL2XML.Simulation GhdlSimul = new VHDL2XML.Simulation();

                if (System.IO.File.Exists(ghdlexe))
                {
                    output = GhdlSimul.GHDLSyntax("\"" + SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath() + "\"");
                    if (output.IndexOf("success") != -1)
                        MessageBox.Show("REPORT: NO ERRORS !\n" + output);
                    else
                        MessageBox.Show("REPORT: ERRORS FOUND !\n" + output);
                }
                else
                {
                    MessageBox.Show("External simulator GHDL is not located in  ProjectDirectory\\lib\\ghdl\n" + ghdlexe);
                }
            }
            else if (SelectedProject.getType() == "verilog")
            {

                VerilogToXML.Verilog2XML verilog2xml = new VerilogToXML.Verilog2XML(SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath());

                if (verilog2xml.CheckSyntax(SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath()))
                {
                    MessageBox.Show("REPORT: NO ERRORS !\n");
                }
                else
                {
                    MessageBox.Show("REPORT: ERRORS FOUND !\n");
                }
            }
        }

        //generate .xhdl
        private void generateXHDLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StreamReader reader;
            if (SelectedProject == null)
            {
                MessageBox.Show("Open a project first!");
            }
            else
            {
                switch (SelectedProject.getType())
                {
                    case "vhdl":
                        if (SelectedTab.SourceFiles.SelectedItem.ToString().IndexOf(".vhd") == -1)
                        {
                            MessageBox.Show("Select a VHDL source file!");
                        }
                        else
                        {
                            VHDL2XML.VHDLToXML vhdlToxml = new VHDL2XML.VHDLToXML();
                            reader = new StreamReader(SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath());
                            vhdlToxml.AddDesignItem(SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath(), WorkingLibrary, reader.ReadToEnd());
                            vhdlToxml.Parse();
                            if (vhdlToxml.Architectures.Count == 0)
                            {
                                MessageBox.Show("There are no architectures in the design described by structure.");
                                return;
                            }
                            XMLProject = vhdlToxml.GenerateXML();
                        }
                        break;
                    case "verilog":
                        if (SelectedTab.SourceFiles.SelectedItem.ToString().IndexOf(".v") == -1 && SelectedTab.SourceFiles.SelectedItem.ToString().IndexOf(".vcd") != -1)
                        {
                            MessageBox.Show("Select a Verilog source file!");
                        }
                        else
                        {
                            VerilogToXML.Verilog2XML verilog2xml = new VerilogToXML.Verilog2XML();
                            verilog2xml.AddVerilogFile(SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath());
                            verilog2xml.Parse();
                            XMLProject = verilog2xml.GenerateXML(SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath());
                        }
                        break;
                    case "systemc":
                        if (SelectedProject.GetSystemCContext().MakeProject() == 0)
                        {

                            ProcessStartInfo info = new ProcessStartInfo(CygwinBash, "start_make.sh");
                            info.WorkingDirectory = SelectedProject.GetSystemCContext().GetWorkingDir();
                            //info.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden;                        
                            Process.Start(info);

                        }
                        break;
                    default:
                        new XDocument(
                            new XElement("Root",
                                new XElement("Lang", "ERROR"),
                                new XElement("Modules"),
                                new XElement("Ports"),
                                new XElement("Signals")
                            )
                        );
                        break;
                }
            }

            string NewPath;
            if (SelectedProject.getType() != "systemc")
            {

                string OldPath = SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath();
                string OldName = SelectedProject.getAllFiles()[SelectedTab.SourceFiles.SelectedIndex].getName();
                NewPath = OldPath.Remove(OldPath.Length - OldName.Length);
                string name = OldName.Trim();
                int dot = name.LastIndexOf('.');
                string NewName = name.Substring(0, dot);
                NewName += ".xhdl";
                NewPath += NewName;
                XMLProject.Save(NewPath);

            }
            else NewPath = SelectedProject.GetSystemCContext().GetWorkingDir() + "\\" + SystemCXHDLFileName;

            //add to project
            SelectedProject.addXHDL(NewPath);
            SelectedTab.RefreshList();

            visualizeToolStripMenuItem.Enabled = true;
        }

        private void visualizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SelectedTab.splitContainer2.Panel2Collapsed = false;
            //vytvorenie zalozky
            TabPage pg = new TabPage();
            pg.Name = "test";
            pg.Text = "Diagram";


            //vytvorenie zoznamov
            List<Modules> ModulesList = new List<Modules>();
            List<Modules> ActualModuleList = new List<Modules>();
            List<Ports> PortsList = new List<Ports>();
            List<Signals> SignalsList = new List<Signals>();
            Modules ParentModule = null;

            /* tu musi nasledovat vytvorenie toto diagramu, pricom nazov xhdl
             * je v SourceFiles.SelectedTab.Name, je to cesta k suboru, teda
             * vizualizuje sa ten xhdl subor, ktory je nalavo v zalozke aktivny.
             * nazov a text zalozky diagramu daj nazov xhdl suboru bez koncovky,
             * teda napr test.xhdl ked das vizualizovat sa bude volat test
             */

            string language = "VHDL";
            try
            {
                XMLParser.ParseFile(SelectedProject.getXHDL().getPath(), out ModulesList, out PortsList, out SignalsList, out language);
            }
            catch (ExceptionExpectedAndTranslated E)
            {
                MessageBox.Show(E.GetTranslatedMessage(), "Parsing Error !");
            }
            catch (Exception E)
            {
                MessageBox.Show(E.Message);
            }
            /* 
                         * Vytvorenie Objektu : 
                         * CachedRepository - Ulozne miesto pre veci z diagramu
                         * Project - Cely projekt diagramu
                         * Diagram - konkretný diagramu, ktory sa bude zobrazovat
                         * Display - jednotka v ktorej sa display zobrazuje
            */
            CachedRepository Repository1 = new CachedRepository();
            Dataweb.NShape.Project Project1 = new Dataweb.NShape.Project();
            Diagram Diagram1 = new Diagram(SelectedProject.getName());
            VHDLDisplay disp = new VHDLDisplay();
            //this.Text = Project1.IsOpen.ToString();

            /*
             * PositionGenerator - Vytvorenie objektu generatora pozicí, ktory rozmiestnuje porty,moduly a signaly po obrazovke
             * DisplayInitializator - Sluzi na spravne nadstavenie display, a na prepojenie jeho prvkov
             */
            PositionGenerator Generator = new PositionGenerator();
            DisplayInitializator Initializator = new DisplayInitializator(ref Diagram1, ref Project1, ref Repository1, ref ModulesList, SelectedProject.getName(), ParentModule);


            // Povolit GUI pre vizualizaciu
            SelectedTab.visualization(ref disp, ref ModulesList, ref Initializator, ref Generator);
            


            Initializator.InitializeDisplay();
            diagramSetController1.Project = Project1;

            disp.Width = 600;
            disp.Height = 700;
            disp.SetInitializator(ref Initializator);
            disp.SetGenerator(ref Generator);
            disp.DiagramSetController = diagramSetController1;
            disp.ShapeDoubleClick += new EventHandler<Dataweb.NShape.Controllers.DiagramPresenterShapeClickEventArgs>(DiagramDoubleClicked);



            /*
             *  Vykreslovanie najvyssiej hierarchickej jednotky
             */

            ActualModuleList.Clear();
            foreach (Modules module in ModulesList)
            {
                if (module.GetPath().IndexOfAny(".".ToCharArray()) == -1)
                {
                    ActualModuleList.Add(module);
                }
            }
            disp.CurrentTool = new PointerTool();

            Generator.SetLimits(VisualisationHorizontalLimit, VisualisationVerticalLimit);
            Initializator.VisualizeAllGraphicalObjects(ref disp, ref Generator, ActualModuleList, ParentModule);



            //pridanie displayu do zalozky a zalozky do tabcontrolu
            pg.Controls.Add(disp);
            pg.Text = SelectedProject.getName();
            SelectedTab.tabControl2.Controls.Add(pg);
            SelectedTab.tabControl2.SelectedTab = pg;

            SelectedTab.groupBox1.Enabled = true;

            if (SelectedProject.getVCD() != null)
                visualizeSimulationToolStripMenuItem.Enabled = true;

        }












        private void DiagramDoubleClicked(object sender, EventArgs e)
        {

            VHDLDisplay disp = (VHDLDisplay)sender;
            PositionGenerator g = disp.GetGenerator();
            Shape shape;
            List<Modules> actualList = new List<Modules>();
            int count;
            if (disp.SelectedShapes.Count() == 0)
            {
                return;
            }
            shape = disp.SelectedShapes.First();
            if (shape.Tag != null)
            {
                actualList.Clear();
                Modules m = (Modules)shape.Tag;
                count = m.GetPath().ToCharArray().Count(x => x == '.');
                foreach (Modules module in disp.GetInitializator().GetModulesList())
                {
                    if (((module.GetPath().StartsWith(m.GetPath()) == true) && (module.GetPath().ToCharArray().Count(x => x == '.') == count + 1)))
                    {
                        actualList.Add(module);
                    }
                }
                if (actualList.Count != 0)
                {
                    g.SetLimits(VisualisationHorizontalLimit, VisualisationVerticalLimit);
                    disp.GetInitializator().VisualizeAllGraphicalObjects(ref disp, ref g, actualList, m);
                }
            }
        }







        void DisplayModification(object sender, EventArgs e)
        {

            VHDLDisplay disp = (VHDLDisplay)sender;
            PositionGenerator g = disp.GetGenerator();
            Shape shape;
            shape = disp.SelectedShapes.First();
            this.Text = "T";


        }











        #endregion

        #region simulation

        //create testbench
        private void createTestbenchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedTab.SourceFiles.SelectedItem.ToString().Contains(".v"))
            {
                string code = SelectedProject.getSourceFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath();
                frmtestbench testbench = new frmtestbench(code,ref SelectedProject);
                testbench.ShowDialog(this);
                SelectedTab.RefreshList();
            }
            else
            {
                MessageBox.Show("Select a VHDL or Verilog file!");
            }
        }

        //create VCD file
        private void createVCDToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedTab.SourceFiles.SelectedItem.ToString().Contains(".v"))
            {
                frmsimtime frmsim = new frmsimtime(SelectedProject.getType(), ref SelectedProject, SelectedProject.getSourceFiles()[SelectedTab.SourceFiles.SelectedIndex].getPath());
                frmsim.Name = SelectedProject.getName();
                if (frmsim.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    SelectedTab.RefreshList();

                    runGTKWaveToolStripMenuItem.Enabled = true;
                    if (SelectedTab.groupBox1.Enabled == true)
                        visualizeSimulationToolStripMenuItem.Enabled = true;
                }
            }
            else
            {
                MessageBox.Show("Select a VHDL or Verilog file!");
            }
        }

        //run GTKWave
        private void runGTKWaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SelectedProject.getVCD() != null)
            {
                RunGTKWave(SelectedProject.getVCD().getPath());
            }
            else
            {
                MessageBox.Show("Create a VCD file!");
            }
        }

        //vizualizovat obsah VCD suboru v scheme
        private void visualizeSimulationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SelectedTab.splitContainer3.Panel2Collapsed = false;
            SelectedTab.simulation(SelectedProject.getVCD().getPath());
        }

        public void RunGTKWave(string vcd_file)
        {
            string path = Environment.CurrentDirectory.ToString().Substring(0, Environment.CurrentDirectory.ToString().Length - 9);
            string gtk_path = path + "lib\\iverilog\\bin\\gtkwave.exe";

            Process gtk = new Process();
            ProcessStartInfo info = new ProcessStartInfo();
            info.FileName = "\"" + gtk_path + "\"";
            info.Arguments = "\"" + vcd_file + "\"";
            gtk.StartInfo = info;
            gtk.Start();
        }

        #endregion

        #region options

        private void vHDLOptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vhdloptions options = new vhdloptions();
            options.ShowDialog(this);
        }

        private void systemCOptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SystemC_Options options = new SystemC_Options();
            options.Show();
        }

        private void visualizationOptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            visualizationOptions options = new visualizationOptions();
            options.ShowDialog(this);
        }

        #endregion

        #region help

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout about = new frmAbout();
            about.ShowDialog(this);
        }

        #endregion

        private void Project_tabs_SelectedIndexChanged(object sender, EventArgs e)
        {
            SelectedTab = (TabPageProject) Project_tabs.SelectedTab;
            SelectedProject = Projects[Project_tabs.SelectedIndex];
        }

    }
}
